package org.bench4q.agent.parameterization.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import org.bench4q.agent.parameterization.SessionObject;

public class InstanceControler implements SessionObject {
	private String userDefineParameterFolderPath = "/home/yxsh/git/Bench4Q-Agent/parameterClass/";
	private UUID instandid = java.util.UUID.randomUUID();
	private Set<String> usedClassName = new HashSet<String>();
	private Map<String, Object> objMap = new HashMap<String, Object>();
	private Map<String, String> runtimeParaMap = new HashMap<String, String>();
	private Map<String, Object> cacheObjMap = new HashMap<String, Object>();
	private ReentrantReadWriteLock mapRWLock = new ReentrantReadWriteLock();

	String instanceLevelGetParameter(String name, String className,
			String functionName, Object[] args) {

		boolean hasThisClass = false;
		mapRWLock.readLock().lock();
		hasThisClass = objMap.containsKey(className);
		mapRWLock.readLock().unlock();
		if (false == hasThisClass) {
			createObj(className);
		}
		Object instance = getObj(className);
		Object result = null;
		try {
			Class<?>[] argTypeArr = new Class<?>[args.length + 2];
			argTypeArr[0] = instandid.getClass();
			argTypeArr[1] = this.cacheObjMap.getClass();
			Object[] totalArgs = new Object[args.length + 2];
			totalArgs[0] = instandid;
			totalArgs[1] = this.cacheObjMap;
			for (int i = 2; i < args.length + 2; i++) {
				argTypeArr[i] = args[i - 2].getClass();
				totalArgs[i] = args[i - 2];
			}
			Method m = instance.getClass().getMethod(functionName, argTypeArr);
			result = m.invoke(instance, totalArgs);
		} catch (Exception ex) {
			System.out.println(ex.getMessage() + ex.getStackTrace());
			System.out.println(((InvocationTargetException) ex)
					.getTargetException().getMessage());
			return null;
		}
		runtimeParaMap.put(name, (String) result);
		return (String) result;

	}

	String getParameterByContext(String name) {
		if (false == this.runtimeParaMap.containsKey(name)) {
			return null;
		}
		return runtimeParaMap.get(name);
	}

	private boolean createObj(String className) {
		try {
			MyFileClassLoader cl = new MyFileClassLoader();
			cl.setClassPath(userDefineParameterFolderPath);
			Class<?> cls = cl.loadClass(className);
			Object instance = cls.newInstance();
			mapRWLock.writeLock().lock();
			objMap.put(className, instance);

		} catch (Exception ex) {
			return false;
		}
		mapRWLock.writeLock().unlock();
		return true;
	}

	private Object getObj(String className) {
		Object result = null;
		mapRWLock.readLock().lock();
		objMap.get(className);
		mapRWLock.readLock().unlock();
		return result;
	}

	String getParameter(String name, String className, String functionName,
			Object[] args) {

		ParametersFactory pf = ParametersFactory.getInstance();
		boolean hasThisClass = pf.containObj(className);
		if (false == hasThisClass) {
			pf.createObj(className);
		}

		Object instance = pf.getObj(className);
		Object result = null;
		try {
			Class<?>[] argTypeArr = new Class<?>[args.length + 2];
			argTypeArr[0] = instandid.getClass();
			argTypeArr[1] = this.cacheObjMap.getClass();
			Object[] totalArgs = new Object[args.length + 2];
			totalArgs[0] = instandid;
			totalArgs[1] = this.cacheObjMap;
			for (int i = 2; i < args.length + 2; i++) {
				argTypeArr[i] = args[i - 2].getClass();
				totalArgs[i] = args[i - 2];
			}

			Method m = instance.getClass().getMethod(functionName, argTypeArr);

			result = m.invoke(instance, totalArgs);

		} catch (Exception ex) {
			System.out.println(ex.getMessage() + ex.getStackTrace());
			System.out.println(((InvocationTargetException) ex)
					.getTargetException().getMessage());
			return null;
		}

		usedClassName.add(className);
		runtimeParaMap.put(name, (String) result);
		return (String) result;
	}

	/**
	 * Implement SessionObject begin
	 */

	public String getParam(String name) {
		if (!(name.startsWith("<parameters") && name.endsWith("/>")))
			return name;
		return ParameterizationParser.parse(name, this);
	}

	public void saveRuntimeParam(String name, String value) {
		runtimeParaMap.put(name, value);
	}

	public void saveRuntimeParams(Map<String, String> runTimeParams) {
		runtimeParaMap.putAll(runTimeParams);
	}

	public void doCleanUp() {
		this.releaseAll();
	}

	/**
	 * Implement SessionObject end
	 */

	private void releaseAll() {
		for (String xx : usedClassName) {
			release(xx);
		}
	}

	private void release(String className) {
		ParametersFactory pf = ParametersFactory.getInstance();
		boolean hasThisClass = pf.containObj(className);
		if (false == hasThisClass) {
			pf.createObj(className);
		}

		Object instance = pf.getObj(className);

		try {

			Method m = instance.getClass().getMethod("unreg",
					instandid.getClass());

			m.invoke(instance, instandid);

		} catch (Exception ex) {
			System.out.println("realse failed");
		}

	}

}
