package org.bench4q.agent.parameterization.impl;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentHashMap;

public class Para_Table {
	public final int cacheCap = 5000;// 1 sequence 2 random
	public final int cacheSize = 1000;// 1 sequence 2 random
	public final int readCharSize = 10000;
	public ConcurrentHashMap<String, Reader> readerMap = new ConcurrentHashMap<String, Para_Table.Reader>();

	public abstract class Reader {
		public final Table t;

		public Reader(Table table) {
			this.t = table;
		}

		public abstract TableRow nextRow();

	}

	public class RandomReader extends Reader {
		public ArrayBlockingQueue<TableRow> queue;

		public RandomReader(ArrayBlockingQueue<TableRow> queue, Table t) {
			super(t);
			this.queue = queue;
		}

		@Override
		public TableRow nextRow() {
			TableRow result = null;
			do {
				if (queue.size() == 0) {
					synchronized (queue) {
						if (queue.size() == 0) {
							List<TableRow> rows = t.readRows();
							int n = rows.size();
							Random r = new Random();
							for (int i = 0; i < n; i++) {
								int next = r.nextInt(n);
								TableRow tempRow = rows.get(i);
								rows.set(i, rows.get(next));
								rows.set(next, tempRow);
							}
							queue.addAll(rows);
						}
					}
				}
				result = queue.poll();
			} while (result == null);
			return result;
		}

	}

	public class SequenceReader extends Reader {
		public ArrayBlockingQueue<TableRow> queue;

		public SequenceReader(ArrayBlockingQueue<TableRow> queue, Table t) {
			super(t);
			this.queue = queue;
		}

		@Override
		public TableRow nextRow() {
			TableRow result = null;
			do {
				if (queue.size() == 0) {
					synchronized (queue) {
						if (queue.size() == 0) {
							List<TableRow> rows = t.readRows();
							queue.addAll(rows);
						}
					}
				}
				result = queue.poll();
			} while (result == null);
			return result;
		}

	}

	public abstract class Table {
		String sourceValue;
		int firstRow;

		public Table(String sourceValue, int firstRow) {
			this.sourceValue = sourceValue;
		}

		public abstract List<TableRow> readRows();

		public List<TableRow> rows = new ArrayList<TableRow>();

		public int getTime = 0;

		public boolean add(TableRow r) {
			return rows.add(r);
		}
	}

	public class FileTable extends Table {
		private BufferedReader bfr;
		char splitChar;
		char lineChar;

		public FileTable(String sourceValue, int firstRow, char splitChar,
				char lineChar) {
			super(sourceValue, firstRow);
			this.splitChar = splitChar;
			this.lineChar = lineChar;
			try {
				createBFR();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		private void createBFR() throws IOException {
			FileInputStream fis = new FileInputStream(sourceValue);
			InputStreamReader ir = new InputStreamReader(fis);
			bfr = new BufferedReader(ir);
			for (int i = 0; i < firstRow;) {
				char readChar = (char) bfr.read();
				if (readChar == lineChar)
					i++;
			}

		}

		@Override
		public List<TableRow> readRows() {
			StringBuilder target = new StringBuilder();
			List<TableRow> result = new ArrayList<TableRow>();
			TableRow tempTableRow = new TableRow();
			try {
				for (int i = 0; i < cacheSize;) {
					int tt = bfr.read();
					char readBuff = (char) tt;
					if (tt == -1) {
						createBFR();
						break;
					} else if (readBuff == this.splitChar) {
						String cellString = target.toString();
						tempTableRow.AddCell(cellString);
						target = new StringBuilder();
						continue;
					} else if (readBuff == this.lineChar) {
						tempTableRow.AddCell(target.toString());
						result.add(tempTableRow);
						tempTableRow = new TableRow();
						target = new StringBuilder();
						i++;
						continue;
					}
					target.append(readBuff);
				}
			} catch (IOException ioex) {
				return result;
			}
			if (target.length() > 0) {
				tempTableRow.AddCell(target.toString());
				result.add(tempTableRow);
				target = new StringBuilder();
			}
			return result;
		}

	}

	public class StringTable extends Table {

		char splitChar;
		char lineChar;

		public StringTable(String sourceValue, int firstRow, char splitChar,
				char lineChar) {
			super(sourceValue, firstRow);
			this.splitChar = splitChar;
			this.lineChar = lineChar;
		}

		@Override
		public List<TableRow> readRows() {
			String readStr = this.sourceValue;
			String[] tokens = readStr.split(String.valueOf(lineChar));
			List<TableRow> result = new ArrayList<TableRow>();
			for (String line : tokens) {
				TableRow tempRow = new TableRow();
				String[] columns = line.split(String.valueOf(splitChar));
				for (String column : columns) {
					tempRow.AddCell(column);
				}
				result.add(tempRow);
			}
			return result;
		}

	}

	public class TableRow {
		public List<String> cells = new ArrayList<String>();

		public void AddCell(String s) {
			cells.add(s);
		}
	}

	// org.bench4q.agent.parameterization.impl.Para_Table.getTableColumnValue(java.util.UUID,
	// java.util.HashMap, java.lang.String, java.lang.String, java.lang.String,
	// java.lang.String, java.lang.String, java.lang.String, java.lang.String)

	public String getTableColumnValue(UUID id,
			HashMap<String, Object> objCache, String source,
			String sourceValue, String firstRow, String nextRow,
			String splitChar, String lineChar, String column) {
		int fRow = Integer.parseInt(firstRow);
		char sChar = splitChar.charAt(0);
		char lChar = lineChar.charAt(0);
		int col = Integer.parseInt(column);
		TableRow resultRow = null;
		Table table = null;
		Reader reader = null;
		if (objCache.containsKey(sourceValue) == true) {
			resultRow = (TableRow) objCache.get(sourceValue);
			return resultRow.cells.get(col);
		}

		if (this.readerMap.containsKey(sourceValue)) {
			reader = readerMap.get(sourceValue);

		} else {
			if (source.equals("file")) {
				table = new FileTable(sourceValue, fRow, sChar, lChar);
			} else if (source.equals("input")) {
				table = new StringTable(sourceValue, fRow, sChar, lChar);
			}

			if (nextRow.equals("random")) {
				ArrayBlockingQueue<TableRow> queue = new ArrayBlockingQueue<Para_Table.TableRow>(
						cacheCap);
				reader = new RandomReader(queue, table);
			} else if (nextRow.equals("sequence")) {

				ArrayBlockingQueue<TableRow> queue = new ArrayBlockingQueue<Para_Table.TableRow>(
						cacheCap);
				reader = new SequenceReader(queue, table);
			}
			readerMap.put(sourceValue, reader);
		}
		resultRow = reader.nextRow();
		objCache.put(sourceValue, resultRow);
		return resultRow.cells.get(col);
	}

	public void unreg(UUID id) {

	}
}
