/*
 * Author Coderfengyun
 * 
 * And th parmas has some special split.
 * Like, QueryParams and bodyParams should be split by ';'
 * And Headers should be split by '|;'
 */
package org.bench4q.agent.plugin.basic.http;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.bench4q.agent.plugin.Behavior;
import org.bench4q.agent.plugin.Parameter;
import org.bench4q.agent.plugin.Plugin;
import org.bench4q.agent.plugin.result.HttpReturn;
import org.bench4q.agent.scenario.util.ParameterParser;
import org.bench4q.agent.scenario.util.Table;

@Plugin("Http")
public class HttpPlugin {
	private HttpClient httpClient;

	public HttpClient getHttpClient() {
		return httpClient;
	}

	private void setHttpClient(HttpClient httpClient) {
		this.httpClient = httpClient;
	}

	public HttpPlugin() {
		this.setHttpClient(new HttpClient());
	}

	void setHeaders(HttpMethod method, String headers) {
		if (headers != null) {
			List<NameValuePair> nvPairs = parseHeaders(headers);
			for (NameValuePair nv : nvPairs) {
				method.addRequestHeader(nv.getName(), nv.getValue());
			}
			// New add
			String cookieValue = "";
			for (Cookie cookie : this.getHttpClient().getState().getCookies()) {
				if (!cookieValue.isEmpty()) {
					cookieValue += ";";
				}
				cookieValue += cookie.getName() + "=" + cookie.getValue();
			}
			if (!cookieValue.isEmpty()) {
				method.addRequestHeader("Cookie", cookieValue);
			}
		}
	}

	List<NameValuePair> parseHeaders(String value) {
		List<NameValuePair> nvPairs = new LinkedList<NameValuePair>();
		Table headerTable = ParameterParser.buildTable(value,
				new LinkedList<String>() {
					private static final long serialVersionUID = 3498984411571605459L;
					{
						add("header");
						add("value");
					}
				});
		for (List<String> entry : headerTable.getRealContent()) {
			if (entry.size() != headerTable.getColumnCount()) {
				continue;
			}
			nvPairs.add(new NameValuePair(entry.get(0).trim(), entry.get(1)
					.trim()));
		}
		return nvPairs;
	}

	private static NameValuePair[] setInputParameters(
			List<String> inputParameters) {
		Set<NameValuePair> res = new HashSet<NameValuePair>();
		Iterator<String> paramIter = inputParameters.iterator();

		while (paramIter.hasNext()) {
			String token = paramIter.next();
			int index = token.indexOf('=');
			res.add(new NameValuePair(token.substring(0, index), token
					.substring(index + 1)));
		}
		return res.toArray(new NameValuePair[res.size()]);
	}

	private static String completeUri(String uri) {
		String scheme = uri.substring(0, 5);
		if (!scheme.equals("http:") && !scheme.equals("https"))
			uri = "http://" + uri;
		return uri;
	}

	/**
	 * 
	 * @param url
	 * @param queryParams
	 *            (NField) QueryParams should be split by ';'
	 * @param headers
	 *            (Table)has two columns, such as header and value
	 * 
	 * @param respVarsToSaveInSession
	 *            (Table) has four columns, such as varName,
	 *            varRegularExpression, leftBoundry, and rightBoundry
	 * @return
	 */
	@Behavior("Get")
	public HttpReturn get(@Parameter("url") String url,
			@Parameter("queryParams") String queryParams,
			@Parameter("headers") String headers,
			@Parameter("respVarsToSaveInSession") String respVarsToSaveInSession) {

		GetMethod method = new GetMethod(completeUri(url));
		if (isValid(queryParams)) {
			method.setQueryString(queryParams);
		}
		setHeaders(method, headers);
		method.getParams().makeLenient();
		method.setFollowRedirects(false);
		return excuteMethod(method, respVarsToSaveInSession);
	}

	private HttpReturn excuteMethod(HttpMethod method,
			String respVarsToSaveInSession) {
		int responseCode = -1;
		long contentLength = 0;
		String contentType = "";
		Map<String, String> respVars = new LinkedHashMap<String, String>();

		try {
			responseCode = this.getHttpClient().executeMethod(method);
			method.getStatusLine().toString();
			Header[] responseHeaders = method.getResponseHeaders();
			if (method.getResponseHeader("Content-Length") != null) {
				contentLength = Long.parseLong(method.getResponseHeader(
						"Content-Length").getValue());
			}
			if (method.getResponseHeader("Content-Type") != null) {
				contentType = method.getResponseHeader("Content-Type")
						.getValue();
			}
			if (isValid(respVarsToSaveInSession)) {
				respVars = doExtractResponseVariables(respVarsToSaveInSession,
						method.getResponseBodyAsString());
			}
			return new HttpReturn(responseCode > 0, responseCode,
					contentLength, contentType, responseHeaders, respVars);
		} catch (HttpException e) {
			e.printStackTrace();
			return new HttpReturn(false, 400, contentLength, contentType);
		} catch (IOException e) {
			e.printStackTrace();
			return new HttpReturn(false, 400, contentLength, contentType);
		} finally {
			method.releaseConnection();
		}
	}

	private Map<String, String> doExtractResponseVariables(
			String respVarsToSaveInSession, String responseBodyAsString) {
		Map<String, String> keyValues = new LinkedHashMap<String, String>();
		List<List<String>> varExtractExpressionList = ParameterParser
				.buildTable(respVarsToSaveInSession, new LinkedList<String>() {
					private static final long serialVersionUID = -923211006660227362L;
					{
						add("varName");
						add("varRegularExpression");
						add("leftBoundry");
						add("rightBoundry");
					}
				}).getRealContent();
		for (List<String> row : varExtractExpressionList) {
			keyValues.putAll(doExtractParamByRow(row, responseBodyAsString));
		}
		return keyValues;
	}

	private Map<String, String> doExtractParamByRow(List<String> row,
			String responseBody) {
		Map<String, String> result = new LinkedHashMap<String, String>();
		if (row.size() <= 3) {
			return result;
		}
		String varName = row.get(0);
		String varExpression = row.get(1);
		String varLeftBoundry = row.get(2);
		String varRightBoundry = row.get(3);

		if (!isValid(varExpression)) {
			result.put(
					varName,
					extractExactlyValueWith(varLeftBoundry, varRightBoundry,
							responseBody));
			return result;
		}

		Matcher matcher = Pattern.compile(varExpression).matcher(responseBody);
		if (!matcher.find()) {
			result.put(
					varName,
					extractExactlyValueWith(varLeftBoundry, varRightBoundry,
							responseBody));
			return result;
		}
		result.put(
				varName,
				extractExactlyValueWith(varLeftBoundry, varRightBoundry,
						matcher.group()));
		return result;
	}

	private String extractExactlyValueWith(String varLeftBoundry,
			String varRightBoundry, String matchedVariable) {
		int indexOfLB = matchedVariable.indexOf(varLeftBoundry);
		int indexOfRB = matchedVariable.indexOf(varRightBoundry);
		return matchedVariable.substring(indexOfLB + varLeftBoundry.length(),
				indexOfRB);
	}

	/**
	 * 
	 * @param url
	 * @param queryParams
	 * @param headers
	 * @param bodyContent
	 *            , when the Content-Length in header G.T. zero, then use this
	 * @param bodyParams
	 *            , else use this
	 * @return
	 */
	@Behavior("Post")
	public HttpReturn post(@Parameter("url") String url,
			@Parameter("queryParams") String queryParams,
			@Parameter("headers") String headers,
			@Parameter("bodyContent") String bodyContent,
			@Parameter("bodyparameters") String bodyParams,
			@Parameter("respVarsToSaveInSession") String respVarToSaveInSession) {
		PostMethod method = new PostMethod(completeUri(url));
		setHeaders(method, headers);
		if (isValid(queryParams)) {
			method.setQueryString(setInputParameters(ParameterParser
					.buildNField(queryParams)));
		}
		if (isValid(bodyParams)) {
			method.setRequestBody(setInputParameters(ParameterParser
					.buildNField(bodyParams)));
		}
		String contentType = getMethodContentType(method);
		if (isValid(bodyContent) && isValid(contentType)) {
			try {
				method.setRequestEntity(new StringRequestEntity(bodyContent,
						contentType, null));
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
		method.getParams().makeLenient();
		method.setFollowRedirects(false);
		return excuteMethod(method, respVarToSaveInSession);
	}

	private String getMethodContentType(HttpMethod method) {
		Header contentTypeHeader = method.getRequestHeader("Content-Type");
		String contentType = null;
		if (contentTypeHeader != null
				&& !contentTypeHeader.toExternalForm().isEmpty()) {
			contentType = contentTypeHeader.toExternalForm();
		}
		return contentType;
	}

	private boolean isValid(String content) {
		return content != null && content.length() != 0;
	}

	@Behavior("Put")
	public HttpReturn put(@Parameter("url") String url,
			@Parameter("queryParams") String queryParams,
			@Parameter("headers") String headers,
			@Parameter("bodyContent") String bodyContent,
			@Parameter("bodyparameters") String bodyParams,
			@Parameter("respVarsToSaveInSession") String respVarToSaveInSession) {
		PutMethod method = new PutMethod(completeUri(url));
		setHeaders(method, headers);
		if (isValid(queryParams)) {
			method.setQueryString(setInputParameters(ParameterParser
					.buildNField(queryParams)));
		}
		String contentType = getMethodContentType(method);
		if (isValid(bodyContent) && isValid(contentType)) {
			try {
				method.setRequestEntity(new StringRequestEntity(bodyContent,
						contentType, null));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
		method.getParams().makeLenient();
		return excuteMethod(method, respVarToSaveInSession);
	}

	@Behavior("Delete")
	public HttpReturn delete(@Parameter("url") String url,
			@Parameter("queryParams") String queryParams,
			@Parameter("headers") String headers,
			@Parameter("respVarsToSaveInSession") String respVarToSaveInSession) {
		DeleteMethod method = new DeleteMethod(completeUri(url));
		setHeaders(method, headers);
		if (isValid(queryParams)) {
			method.setQueryString(setInputParameters(ParameterParser
					.buildNField(queryParams)));
		}
		method.getParams().makeLenient();
		return excuteMethod(method, respVarToSaveInSession);
	}
}
