package org.bench4q.agent.scenario;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import org.apache.log4j.Logger;
import org.bench4q.agent.parameterization.impl.InstanceControler;
import org.bench4q.agent.storage.StorageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class ScenarioEngine {
	private Map<UUID, ScenarioContext> runningTests;
	private StorageHelper storageHelper;
	private Logger logger;

	@SuppressWarnings("unused")
	private Logger getLogger() {
		return logger;
	}

	private void setLogger(Logger logger) {
		this.logger = logger;
	}

	public ScenarioEngine() {
		this.setRunningTests(new HashMap<UUID, ScenarioContext>());
		this.setLogger(Logger.getLogger(ScenarioEngine.class));
	}

	public StorageHelper getStorageHelper() {
		return storageHelper;
	}

	@Autowired
	public void setStorageHelper(StorageHelper storageHelper) {
		this.storageHelper = storageHelper;
	}

	public Map<UUID, ScenarioContext> getRunningTests() {
		return runningTests;
	}

	private void setRunningTests(Map<UUID, ScenarioContext> runningTests) {
		this.runningTests = runningTests;
	}

	public void submitScenario(UUID runId, final Scenario scenario, int poolSize) {
		try {
			final ScenarioContext scenarioContext = ScenarioContext
					.buildScenarioContext(runId, scenario, poolSize);
			this.getRunningTests().put(runId, scenarioContext);

			System.out.println(poolSize);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean runWith(UUID runId) {
		if (!this.getRunningTests().containsKey(runId)) {
			return false;
		}
		final ScenarioContext context = this.getRunningTests().get(runId);
		return runWith(context);
	}

	private boolean runWith(final ScenarioContext scenarioContext) {
		if (scenarioContext == null) {
			logger.error("The context required is null");
			return false;
		}
		ExecutorService taskMaker = Executors.newSingleThreadExecutor();

		taskMaker.execute(new Runnable() {
			public void run() {
				while (!scenarioContext.isFinished()) {
					scenarioContext.getExecutor()
							.execute(
									new Worker(scenarioContext,
											new InstanceControler()));
				}
			}
		});
		return true;
	}

}
