package org.bench4q.agent.scenario.util;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.bench4q.agent.plugin.basic.http.ParameterConstant;

public class Table {
	public static final String ROW_SEPARATOR = "|;";
	public static final String COLUMN_SEPARATOR = "|";
	public static final String ROW_SEPARATOR_TAIL = ";";

	public static final String ESCAPE_ROW_SEPARATOR = ParameterConstant.escape
			+ ROW_SEPARATOR;
	public static final String ESCAPE_COLUMN_SEPARATOR = ParameterConstant.escape
			+ COLUMN_SEPARATOR;
	private List<List<String>> realContent;
	private List<String> columnLables;

	public List<List<String>> getRealContent() {
		return realContent;
	}

	private void setRealContent(List<List<String>> realContent) {
		this.realContent = realContent;
	}

	private List<String> getColumnLables() {
		return columnLables;
	}

	private void setColumnLables(List<String> columnLables) {
		this.columnLables = columnLables;
	}

	private Table() {
		this.setColumnLables(new LinkedList<String>());
	}

	static Table buildTable(String value, List<String> columnLablesInSequence) {
		Table result = new Table();
		result.setRealContent(buildTableContent(value));
		result.setColumnLables(columnLablesInSequence);
		return result;
	}

	/**
	 * This method analyzes the value to be set in the table, and remove all
	 * escape characters
	 * 
	 * @param value
	 *            The value containing the serialization of all values of the
	 *            table
	 * @return The vector containing all the entries, each entries is in a
	 *         vector, which contains the string values
	 */
	private static List<List<String>> buildTableContent(String value) {
		List<String> tempEntries = ParameterParser.getRealTokens(
				Table.ESCAPE_ROW_SEPARATOR, value, false);
		List<List<String>> allValues = new ArrayList<List<String>>();
		for (int i = 0; i < tempEntries.size(); i++) {
			String tempEntry = tempEntries.get(i);
			List<String> values = ParameterParser.getRealTokens(
					Table.ESCAPE_COLUMN_SEPARATOR, tempEntry, true);
			arrangeTableWithRowSeparatorTail(values);
			allValues.add(values);
		}
		// now analyze all values, and get the '=' separator
		return extractValueFromWellFormedTable(allValues);
	}

	private static List<List<String>> extractValueFromWellFormedTable(
			List<List<String>> allValues) {
		List<List<String>> result = new LinkedList<List<String>>();
		for (int i = 0; i < allValues.size(); i++) {
			List<String> resultValues = new ArrayList<String>();
			List<String> tempValues = allValues.get(i);
			for (int j = 0; j < tempValues.size(); j++) {
				String v = tempValues.get(j);
				List<String> temp = ParameterParser
						.getRealTokens("=", v, false);
				String res;
				if (temp.size() <= 1) {
					continue;
				}
				res = temp.get(1);
				resultValues.add(res);
			}
			result.add(resultValues);
		}
		return result;
	}

	/**
	 * This method check if the table_separator after escape is split by the
	 * table_entry_separator
	 * 
	 * @param values
	 */
	private static void arrangeTableWithRowSeparatorTail(List<String> values) {
		for (int i = 0; i < values.size(); i++) {

			String currentToken = values.get(i);
			if (currentToken.equals(Table.ROW_SEPARATOR_TAIL) && i > 0) {
				values.set(i - 1, values.get(i - 1) + Table.ROW_SEPARATOR_TAIL);
			}
		}

	}

	public int getColumnCount() {
		return this.getColumnLables().size();
	}
}
