package org.bench4q.agent.test.plugin;

import java.util.Map;

import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.cookie.MalformedCookieException;
import org.bench4q.agent.plugin.basic.http.HttpPlugin;
import org.bench4q.agent.plugin.result.HttpReturn;
import org.bench4q.agent.plugin.result.PluginReturn;
import org.bench4q.share.helper.TestHelper;
import org.junit.Test;

import static org.junit.Assert.*;

public class Test_HttpPlugin {
	private static final int UnAuthorized = 401;
	private HttpPlugin httpPlugin;

	private HttpPlugin getHttpPlugin() {
		return httpPlugin;
	}

	private void setHttpPlugin(HttpPlugin httpPlugin) {
		this.httpPlugin = httpPlugin;
	}

	public Test_HttpPlugin() {
		this.setHttpPlugin(new HttpPlugin());
	}

	@Test
	public void testGet() {
		PluginReturn return1 = this.getHttpPlugin().get(
				"http://www.baidu.com/s", "wd=ok", "", "");
		assertTrue(return1.isSuccess());
	}

	@Test
	public void testPost() {
		PluginReturn result = this.getHttpPlugin().post(
				"http://133.133.12.2:8080/bench4q-web/login",
				"userName=admin;password=admin", null, null, null, null);
		assertTrue(result.isSuccess());
	}

	@Test
	public void testGetWithActualBehavior() {
		PluginReturn result = this
				.getHttpPlugin()
				.get("http://133.133.12.2:8080/bench4q-web/index.jsp",
						"",
						"header=Accept|value=text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8|",
						null);
		assertTrue(result.isSuccess());
	}

	@Test
	public void testSessionAndCookie() throws MalformedCookieException,
			IllegalArgumentException {
		String loginPageUrl = "http://124.16.137.203:7080/cloudshare_web/login";
		HttpReturn indexReturn = this
				.getHttpPlugin()
				.get(loginPageUrl,
						"",
						"header=Accept|value=text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8|;",
						null);
		assertEquals(UnAuthorized, indexReturn.getStatusCode());
		assertNotNull(indexReturn.getHeaders());
		logTheCookie("first");

		HttpReturn loginReturn = this
				.getHttpPlugin()
				.post(loginPageUrl,
						null,
						"header=Content-Type|value=application/x-www-form-urlencoded|;header=Content-Length|value=63|;header=Accept|value=text/html,application/xhtml+xml,application/xml|;",
						"username=chentienan12%40otcaix.iscas.ac.cn&password=jingcai2008",
						null, null);
		assertEquals(302, loginReturn.getStatusCode());
		assertNotNull(loginReturn.getHeaders());
		logTheCookie("second");

		HttpReturn homeReturn = this
				.getHttpPlugin()
				.get("http://124.16.137.203:7080/cloudshare_web/home",
						null,
						"header=Accept|value=text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8|;",
						null);
		assertEquals(200, homeReturn.getStatusCode());
		logTheCookie("third");

		HttpReturn ticketReturn = this
				.getHttpPlugin()
				.post("http://124.16.137.203:7080/cloudshare_web/v2/file/ajax/uploadFiles/-1",
						null,
						"header=Accept|value=*/*|;"
								+ "header=Content-Length|value=0|;"
								+ "header=X-Requested-With|value=XMLHttpRequest|;",
						"",
						null,
						"varName=ticket|varExpression=\"ticket\":\\s*?\"\\w+\",|leftBoundry=\"ticket\"\": \"|rightBoundry=\",|;");
		assertEquals(200, ticketReturn.getStatusCode());
		assertNotNull(ticketReturn.getRunTimeParams());
		assertEquals(1, ticketReturn.getRunTimeParams().size());
		System.out.println(ticketReturn.getRunTimeParams().get("ticket"));
	}

	private void logTheCookie(String times) {
		for (Cookie cookie : this.getHttpPlugin().getHttpClient().getState()
				.getCookies()) {
			System.out.println("Cookie from httpState for " + times + " : "
					+ cookie.getName() + " : " + cookie.getValue());
		}
	}

	@Test
	public void testDoExtractResponseVariables() throws Exception {
		String respVarsToSave = "varName=ticket|varExpression=\"ticket\":\\s*?\"\\w+\",|leftBoundry=\"ticket\"\": \"|rightBoundry=\",|;";
		String responseBodyAsString = "{" + "\n"
				+ "\"ticket\": \"e55de793fc934001b156e97f94ff3eb6\"," + "\n"
				+ "\"parentId\": 3273," + "\n" + "\"expiresIn\": 300000" + "\n"
				+ "}";
		@SuppressWarnings("unchecked")
		Map<String, String> nameValueMap = (Map<String, String>) TestHelper
				.invokePrivate(this.getHttpPlugin(),
						"doExtractResponseVariables", new Class<?>[] {
								String.class, String.class }, new Object[] {
								respVarsToSave, responseBodyAsString });
		assertEquals(1, nameValueMap.size());
		assertTrue(nameValueMap.containsKey("ticket"));
		System.out.println(nameValueMap.get("ticket"));
		assertEquals("e55de793fc934001b156e97f94ff3eb6",
				nameValueMap.get("ticket"));
	}
}
