package org.bench4q.master.api;

import org.apache.log4j.Logger;
import org.bench4q.master.TestPlunginHelper;
import org.bench4q.master.domain.service.PluginService;
import org.bench4q.master.domain.service.UserService;
import org.bench4q.master.exception.Bench4QException;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.share.models.master.plugin.PluginGUI;
import org.bench4q.share.models.master.plugin.PluginModel;
import org.bench4q.share.models.master.plugin.PluginResponseModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping("/plugin")
public class PluginController extends BaseController {
	private Logger logger = Logger.getLogger(PluginController.class);
	private PluginService pluginService;

	public PluginService getPluginService() {
		return pluginService;
	}

	@Autowired
	public void setPluginService(PluginService pluginService) {
		this.pluginService = pluginService;
	}
	
	@RequestMapping("/add")
	@ResponseBody
	public void addPluginToDataBase() throws Bench4QException{
		PluginModel pluginModel=TestPlunginHelper.createOnePlugin();
		this.getPluginService().addPlugin(pluginModel);
	}
	@RequestMapping("/loadPluginList")
	@ResponseBody
	public PluginResponseModel loadPluginList() throws Bench4QException {
		if (!this.checkScope(UserService.NORAML_AUTHENTICATION)) {
			throw new Bench4QException(400 + "", "not permitted",
					"/loadPluginList");
		}
		PluginResponseModel pluginResponseModel = new PluginResponseModel();
		pluginResponseModel.setPluginList(this.getPluginService()
				.getPluginNameList());

		pluginResponseModel.setSuccess(true);
		return pluginResponseModel;
	}

	@RequestMapping(value = "loadMethodList/{pluginName}", method = {
			RequestMethod.GET, RequestMethod.POST })
	@ResponseBody
	public PluginResponseModel getMethodList(
			@PathVariable("pluginName") String pluginName)
			throws Bench4QException {
		if (!this.checkScope(UserService.NORAML_AUTHENTICATION)) {
			throw new Bench4QException(400 + "", "not permitted",
					"/loadMethodList/{pluginName}");
		}
		try {
			PluginResponseModel pluginResponseModel = new PluginResponseModel();
			pluginResponseModel.setMethodList(this.getPluginService()
					.getMethodNameInPlugin(pluginName));
			pluginResponseModel.setSuccess(true);
			return pluginResponseModel;
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
			return null;
		}
	}

	@RequestMapping(value = "loadMethodParams/{pluginName}/{methodName}", method = {
			RequestMethod.GET, RequestMethod.POST })
	@ResponseBody
	public PluginResponseModel getMethodParams(
			@PathVariable("pluginName") String pluginName,
			@PathVariable("methodName") String methodName)
			throws Bench4QException {
		if (!this.checkScope(UserService.SUPER_AUTHENTICATION)) {
			throw new Bench4QException(400 + "", "not permitted",
					"loadMethodParams/{pluginName}/{methodName}");
		}
		PluginResponseModel pluginResponseModel = new PluginResponseModel();
		pluginResponseModel.setMethosMethodParamModels(this.getPluginService()
				.getMethodParamModelsInPlugin(pluginName, methodName));
		pluginResponseModel.setSuccess(true);
		return pluginResponseModel;
	}

	@RequestMapping(value = "/addPlugin", method = { RequestMethod.PUT })
	@ResponseBody
	public PluginResponseModel addPlugin(@RequestBody PluginGUI pluginGUI)
			throws Bench4QException {
		if (!this.checkScope(UserService.SUPER_AUTHENTICATION)) {
			throw new Bench4QException(400 + "", "not permitted",
					"loadMethodParams/{pluginName}/{methodName}");
		}
		PluginResponseModel pluginResponseModel = new PluginResponseModel();
		pluginResponseModel.setSuccess(this.getPluginService().addPlugin(
				pluginGUI.getPlugin()));
		return pluginResponseModel;

	}

	@RequestMapping(value = "/deletePlugin/{pluginName}", method = {
			RequestMethod.GET, RequestMethod.POST })
	@ResponseBody
	public PluginResponseModel deletePlugin(
			@PathVariable("pluginName") String pluginName)
			throws Bench4QException {
		if (!this.checkScope(UserService.SUPER_AUTHENTICATION)) {
			throw new Bench4QException(400 + "", "not permitted",
					"loadMethodParams/{pluginName}/{methodName}");
		}
		PluginResponseModel pluginResponseModel = new PluginResponseModel();
		pluginResponseModel.setSuccess(this.getPluginService().deletePlugin(
				pluginName));
		return pluginResponseModel;

	}

}
