package org.bench4q.master.domain.entity;

import java.util.Observable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.testplan.TestMonitorSampler;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.helper.ApplicationContextHelper;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.monitor.MonitorMain;

@Entity
@Table(name = "monitor")
public class Monitor extends Observable {
	private int id;
	private String hostName;
	private int port;
	private TestPlan testPlan;
	private Set<MonitorResult> results;
	private Logger logger = Logger.getLogger(Monitor.class);

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id", nullable = false)
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	@Column(name = "hostName", nullable = false)
	public String getHostName() {
		return hostName;
	}

	public void setHostName(String hostName) {
		this.hostName = hostName;
	}

	@Column(name = "port", nullable = false)
	public int getPort() {
		return port;
	}

	public void setPort(int port) {
		this.port = port;
	}

	@ManyToOne
	@JoinColumn(name = "testPlanId")
	public TestPlan getTestPlan() {
		return testPlan;
	}

	public void setTestPlan(TestPlan testPlan) {
		this.testPlan = testPlan;
	}

	@OneToMany(mappedBy = "monitor", cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
	public Set<MonitorResult> getResults() {
		return results;
	}

	public void setResults(Set<MonitorResult> results) {
		this.results = results;
	}

	@Transient
	public TestMonitorSampler getTestMonitorSampler() {
		return ApplicationContextHelper.getContext().getBean(
				TestMonitorSampler.class);
	}

	public void doRunInit() {

	}

	public void doAfterRun() {
		try {
			logger.info("enter monitor do after run");
			MonitorMain monitorResult = getTestMonitorSampler()
					.getMonitorResult(hostName, port);
			logger.info(MarshalHelper.marshal(MonitorMain.class, monitorResult));
			collectResult(monitorResult);
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
		}
	}

	private void collectResult(MonitorMain monitorMain) {
		try {
			logger.info("set change");
			this.setChanged();
			this.notifyObservers(monitorMain);
			logger.info("entry notify:" + monitorMain);
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
		}

	}

}
