package org.bench4q.master.domain.entity;

import java.util.Date;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.IAggregate;
import org.bench4q.master.domain.testplan.LoadDistribute;
import org.bench4q.share.enums.master.TestPlanStatus;

@Entity
@Table(name = "testplan")
public class TestPlan implements IAggregate {
	private int id;
	private String name;
	private int sampleCycleInSeconds;
	private Date createDateTime;
	private Date lastRunningTime;
	private User user;
	private String testPlanRunId;
	private String currentStatus;
	private int failTimes;
	private int requiredLoad;
	private Set<TestPlanScript> testPlanScripts;
	private Set<Monitor> monitors;
	private LoadDistribute loadDistribute;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id", nullable = false)
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	@Column(name = "name", nullable = false)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "sampleCycleInSeconds", nullable = false)
	public int getSampleCycleInSeconds() {
		return sampleCycleInSeconds;
	}

	public void setSampleCycleInSeconds(int sampleCycleInSeconds) {
		this.sampleCycleInSeconds = sampleCycleInSeconds;
	}

	@Column(name = "testPlanRunId", columnDefinition = "TEXT", nullable = false)
	public String getTestPlanRunId() {
		return testPlanRunId;
	}

	public void setTestPlanRunId(String testPlanRunId) {
		this.testPlanRunId = testPlanRunId;
	}

	@Column(name = "createDateTime", nullable = false)
	public Date getCreateDateTime() {
		return createDateTime;
	}

	public void setCreateDateTime(Date createDateTime) {
		this.createDateTime = createDateTime;
	}

	@Column(name = "lastRunningTime", nullable = false)
	public Date getLastRunningTime() {
		return lastRunningTime;
	}

	public void setLastRunningTime(Date lastRunningTime) {
		this.lastRunningTime = lastRunningTime;
	}

	@ManyToOne
	@JoinColumn(name = "userId", nullable = false)
	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	@Column(name = "currentStatus", nullable = false)
	public String getCurrentStatus() {
		return currentStatus;
	}

	public void setCurrentStatus(String currentStatus) {
		this.currentStatus = currentStatus;
	}

	@Column(name = "failTimes", nullable = false)
	public int getFailTimes() {
		return failTimes;
	}

	public void setFailTimes(int failTimes) {
		this.failTimes = failTimes;
	}

	@Column(name = "requiredLoad", nullable = false)
	public int getRequiredLoad() {
		return requiredLoad;
	}

	public void setRequiredLoad(int requiredLoad) {
		this.requiredLoad = requiredLoad;
	}

	@OneToMany(mappedBy = "testPlan", cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
	public Set<TestPlanScript> getTestPlanScripts() {
		return testPlanScripts;
	}

	public void setTestPlanScripts(Set<TestPlanScript> testPlanScripts) {
		this.testPlanScripts = testPlanScripts;
	}

	@OneToMany(mappedBy = "testPlan", cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
	public Set<Monitor> getMonitors() {
		return monitors;
	}

	public void setMonitors(Set<Monitor> monitors) {
		this.monitors = monitors;
	}

	@Transient
	public LoadDistribute getLoadDistribute() {
		return loadDistribute;
	}

	public void setLoadDistribute(LoadDistribute loadDistribute) {
		this.loadDistribute = loadDistribute;
	}

	public TestPlanScript extracSpecifiedScript(int scriptId) {
		for (TestPlanScript testPlanScript : testPlanScripts) {
			if (testPlanScript.getScript().getId() == scriptId) {
				return testPlanScript;
			}
		}
		return null;
	}

	public Monitor extractSpecifiedMonitor(String hostName) {
		for (Monitor monitor : this.getMonitors()) {
			if (monitor.getHostName().equals(hostName))
				return monitor;
		}
		return null;
	}

	public TestPlanStatus run() {
		Logger.getLogger(TestPlan.class).info(this.getTestPlanRunId()+" start to run ");
		return this.getLoadDistribute().generateLoadForTestPlan(this);
	}

	public void doAfterRun() {
		try {
			ScheduledExecutorService sheScheduledExecutorService = Executors
					.newScheduledThreadPool(10);
			sheScheduledExecutorService.scheduleAtFixedRate(new Runnable() {
				public void run() {

					for (TestPlanScript testPlanScript : getTestPlanScripts()) {
						testPlanScript.doAfterRun();
					}
					if (getMonitors() != null) {
						for (Monitor monitor : getMonitors()) {
							monitor.doAfterRun();
						}
					}

				}
			}, 0, this.getSampleCycleInSeconds(), TimeUnit.SECONDS);
		} catch (Exception e) {
			Logger.getLogger(TestPlan.class).info(e, e.fillInStackTrace());

		}

	}

	@Transient
	public boolean isFinish() {
		for (TestPlanScript testPlanScript : this.getTestPlanScripts()) {
			if (!testPlanScript.isFinish()) {
				return false;
			}
		}
		return true;
	}

}
