package org.bench4q.master.domain.entity;

import java.util.Collections;
import java.util.List;
import java.util.Observable;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.RunningScriptInterface;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.domain.service.AgentService;
import org.bench4q.master.domain.service.TestPlanService;
import org.bench4q.master.domain.testplan.RunningScriptSampler;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.master.testplan.highavailable.AgentRunBlotter;
import org.bench4q.master.testplan.highavailable.faultolerence.StopAgentFault;
import org.bench4q.master.testplan.schedulscript.ExecutionOverTask;
import org.bench4q.master.testplan.schedulscript.WarmUpOverTask;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.agent.StopTestModel;
import org.bench4q.share.models.master.statistics.SampleModel;

@Entity
@Table(name = "TestPlanScript")
public class TestPlanScript extends Observable implements
		RunningScriptInterface {
	private int id;
	private Script script;
	private int requireLoad;
	private PlanedConfig planedConfig;
	private TestPlan testPlan;
	private Set<RunningAgentDB> runningAgents;
	private Set<TestPlanScriptResult> testPlanScriptResults;
	private boolean finish;

	private TestPlanFactory testPlanFactory;
	private AgentMessenger agentMessenger;
	private RunningScriptSampler sampler;
	private AgentService agentService;
	private Timer timer = new Timer();
	private Logger logger = Logger.getLogger(TestPlanScript.class);

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id", nullable = false)
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	@ManyToOne
	@JoinColumn(name = "scriptId", nullable = false)
	public Script getScript() {
		return script;
	}

	public void setScript(Script script) {
		this.script = script;
	}

	@Column(name = "requireLoad", nullable = false)
	public int getRequireLoad() {
		return requireLoad;
	}

	public void setRequireLoad(int requireLoad) {
		this.requireLoad = requireLoad;
	}

	@OneToOne(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
	@JoinColumn(name = "configId", nullable = false)
	public PlanedConfig getPlanedConfig() {
		return planedConfig;
	}

	public void setPlanedConfig(PlanedConfig planedConfig) {
		this.planedConfig = planedConfig;
	}

	@ManyToOne
	@JoinColumn(name = "testPlanId")
	public TestPlan getTestPlan() {
		return testPlan;
	}

	public void setTestPlan(TestPlan testPlan) {
		this.testPlan = testPlan;
	}

	@OneToMany(mappedBy = "testPlanScript", cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.LAZY)
	public Set<TestPlanScriptResult> getTestPlanScriptResults() {
		return testPlanScriptResults;
	}

	public void setTestPlanScriptResults(
			Set<TestPlanScriptResult> testPlanScriptResults) {
		this.testPlanScriptResults = testPlanScriptResults;
	}

	@OneToMany(mappedBy = "testPlanScript", cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
	public Set<RunningAgentDB> getRunningAgentsDB() {
		return runningAgents;
	}

	public void setRunningAgentsDB(Set<RunningAgentDB> runningAgents) {
		this.runningAgents = runningAgents;
	}

	@Column(name = "finish", nullable = false)
	public boolean isFinish() {
		return finish;
	}

	public void setFinish(boolean finish) {
		this.finish = finish;
	}

	@Transient
	public int getScriptId() {
		return this.getScript().getId();
	}

	@Transient
	public RunScenarioModel getScenario() {
		try {
			return (RunScenarioModel) MarshalHelper
					.unmarshal(RunScenarioModel.class, this.getScript()
							.getScriptContent());
		} catch (Exception e) {
			Logger.getLogger(TestPlanScript.class).error(
					ExceptionLog.getStackTrace(e));
			return null;
		}
	}

	@Transient
	public Set<RunningAgentDB> getRunningAgents() {
		return this.getRunningAgentsDB();
	}

	@Transient
	public UUID getTestPlanID() {
		return UUID.fromString(this.getTestPlan().getTestPlanRunId());
	}

	@Transient
	private TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	public void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	@Transient
	public RunningScriptSampler getSampler() {
		return sampler;
	}

	public void setSampler(RunningScriptSampler sampler) {
		this.sampler = sampler;
	}

	@Transient
	private Timer getTimer() {
		return timer;
	}

	@Transient
	private AgentMessenger getAgentMessenger() {
		return agentMessenger;
	}

	public void setAgentMessenger(AgentMessenger agentMessenger) {
		this.agentMessenger = agentMessenger;
	}

	@Transient
	private AgentService getAgentService() {
		return agentService;
	}

	public void setAgentService(AgentService agentService) {
		this.agentService = agentService;
	}

	public boolean scheduleAsConfig() {
		PlanedConfig testScriptConfig = this.getPlanedConfig();
		if (testScriptConfig == null) {
			return false;
		}
		Timer timer = new Timer();
		timer.schedule(new WarmUpOverTask(), testScriptConfig.getWarmUp()
				* TestPlanService.TIME_UNIT);

		ExecutionOverTask executionOverTask = new ExecutionOverTask(this);
		timer.schedule(
				executionOverTask,
				(testScriptConfig.getWarmUp() + testScriptConfig
						.getExecuteRange()) * TestPlanService.TIME_UNIT);

		timer.schedule(
				new TimerTask() {
					@Override
					public void run() {
						System.out.println("cool down over!");
					}
				},
				(testScriptConfig.getWarmUp()
						+ testScriptConfig.getExecuteRange() + testScriptConfig
							.getCoolDown()) * TestPlanService.TIME_UNIT);
		return true;
	}

	public void doAfterDistributeLoad(
			List<? extends RunningAgentInterface> agentsAfterDistribute) {
		this.getRunningAgentsDB().addAll(
				this.getTestPlanFactory().createRunningAgentDBsWithoutId(
						agentsAfterDistribute, this));
		this.setSampler(new RunningScriptSampler(Collections
				.unmodifiableCollection(this.getRunningAgentsDB())));
		this.scheduleAsConfig();
	}

	public void doAfterRun() {
		doPeriodicalBrief();
	}

	// private void doPeriodicalBrief() {
	// this.getTimer().scheduleAtFixedRate(new TimerTask() {
	// @Override
	// public void run() {
	// if (isFinish()) {
	// getTimer().cancel();
	// return;
	// }
	// notifyObserver(getSampler().getScriptBrief());
	// notifyObserver(getSampler().getPagesBrief());
	// notifyObserver(getSampler().getBehaviorsBrief());
	// }
	// }, 0, this.getTestPlan().getSampleCycleInSeconds() * 1000);
	// }
	private void doPeriodicalBrief() {
		try {
			this.setChanged();
			notifyObserver(getSampler().getScriptBrief());
			notifyObserver(getSampler().getPagesBrief());
			notifyObserver(getSampler().getBehaviorsBrief());
		} catch (Exception e) {
			logger.info(ExceptionLog.getStackTrace(e));
		}

	}

	private void notifyObserver(SampleModel sample) {

		this.notifyObservers(sample);

	}

	public TestPlanScript doForComplete() {
		for (RunningAgentInterface runningAgent : this.getRunningAgentsDB()) {
			if (runningAgent == null) {
				continue;
			}
			StopTestModel resultModel = this.getAgentMessenger().stop(
					runningAgent.getAgent(), runningAgent.getAgentRunId());
			if (resultModel == null || !resultModel.isSuccess()) {
				doFaultTolerance(runningAgent);
				continue;
			}
			backLoadAndCleanUp(runningAgent);
		}
		this.setFinish(true);
		this.getTimer().cancel();
		return this;
	}

	private void doFaultTolerance(RunningAgentInterface runningAgent) {
		logger.error("when stop agent with hostName "
				+ runningAgent.getAgent().getHostName() + " returns null!");
		StopAgentFault stopAgentFault = new StopAgentFault(
				AgentRunBlotter.buildAgentRunBlotter(this.getTestPlanID(),
						runningAgent));
		stopAgentFault.doTolerance();
	}

	private void backLoadAndCleanUp(RunningAgentInterface runningAgent) {
		String hostName = runningAgent.getAgent().getHostName();
		this.getAgentService().backLoadToAgent(hostName,
				runningAgent.getLoadInUse());
	}

	public RunningAgentDB extractSpecificRunningAgentDB(String hostName) {
		for (RunningAgentDB runningAgentDB : this.getRunningAgentsDB()) {
			if (runningAgentDB.getAgent().getHostName().equals(hostName)) {
				return runningAgentDB;
			}
		}
		return null;
	}
}
