package org.bench4q.master.domain.factory;

import java.util.HashSet;

import org.bench4q.master.domain.entity.plugin.Method;
import org.bench4q.master.domain.entity.plugin.MethodParam;
import org.bench4q.master.domain.entity.plugin.MethodParamProperty;
import org.bench4q.master.domain.entity.plugin.ParamType;
import org.bench4q.master.domain.entity.plugin.Plugin;
import org.bench4q.master.domain.repository.PluginRepository;
import org.bench4q.share.models.master.plugin.MethodModel;
import org.bench4q.share.models.master.plugin.MethodParamModel;
import org.bench4q.share.models.master.plugin.MethodParamPropertyModel;
import org.bench4q.share.models.master.plugin.ParamTypeEnum;
import org.bench4q.share.models.master.plugin.PluginModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
@Component
public class PluginFactory {

	private PluginRepository pluginRepository;

	public PluginRepository getPluginRepository() {
		return pluginRepository;
	}

	@Autowired
	public void setPluginRepository(PluginRepository pluginRepository) {
		this.pluginRepository = pluginRepository;
	}

	public Plugin createPluginEntity(PluginModel pluginModel) {
		Plugin plugin = new Plugin();
		plugin.setPluginMethods(new HashSet<Method>());
		if (pluginModel.getMethods() != null) {
			for (MethodModel methodModel : pluginModel.getMethods()) {
				plugin.getPluginMethods().add(
						createMethodEntity(methodModel, plugin));
			}
		}

		plugin.setName(pluginModel.getName());
		return plugin;

	}

	private Method createMethodEntity(MethodModel methodModel, Plugin plugin) {
		Method method = new Method();
		method.setMethodParams(new HashSet<MethodParam>());
		method.setName(methodModel.getName());
		if (methodModel.getMethodParams() != null) {
			for (MethodParamModel methodParamModel : methodModel
					.getMethodParams()) {
				method.getMethodParams().add(
						createMethodParamEntity(methodParamModel, method));
			}
		}
		method.setPlugin(plugin);

		return method;
	}

	private MethodParam createMethodParamEntity(
			MethodParamModel methodParamModel, Method method) {
		MethodParam methodParam = new MethodParam();
		methodParam.setLable(methodParamModel.getLable());
		methodParam.setName(methodParamModel.getName());
		methodParam.setMethod(method);
		methodParam.setParamType(createPraParamTypeWithOutId(methodParamModel
				.getParamType()));
		if (methodParamModel.getMethodParamProperties() != null) {
			methodParam
					.setMethodParamProperties(new HashSet<MethodParamProperty>());
			for (MethodParamPropertyModel methodParamPropertyModel : methodParamModel
					.getMethodParamProperties()) {
				methodParam.getMethodParamProperties().add(
						createMethodParamPropertyWithOutId(
								methodParamPropertyModel, methodParam));
			}
		}
		return methodParam;
	}

	private ParamType createPraParamTypeWithOutId(ParamTypeEnum paramTypeEnum) {
		return this.getPluginRepository().getParamTypeByType(paramTypeEnum);
	}

	private MethodParamProperty createMethodParamPropertyWithOutId(
			MethodParamPropertyModel methodParamPropertyModel,
			MethodParam methodParam) {
		MethodParamProperty methodParamProperty = new MethodParamProperty();
		methodParamProperty.setKey(methodParamPropertyModel.getKey());
		methodParamProperty.setValue(methodParamPropertyModel.getValue());
		methodParamProperty.setMethodParam(methodParam);
		return methodParamProperty;
	}

	public static MethodModel extractMethodModel(Method method) {
		MethodModel methodModel = new MethodModel();
		methodModel.setName(method.getName());
		if (method.getMethodParams() != null) {
			methodModel.setMethodParams(new HashSet<MethodParamModel>());
			for (MethodParam methodParam : method.getMethodParams()) {
				methodModel.getMethodParams().add(
						extractMethodParamModel(methodParam));
			}
		}
		return methodModel;
	}

	private static MethodParamModel extractMethodParamModel(
			MethodParam methodParam) {
		MethodParamModel methodParamModel = new MethodParamModel();
		methodParamModel.setLable(methodParam.getLable());
		methodParamModel.setName(methodParam.getName());
		methodParamModel.setParamType(methodParam.getParamType().getType());
		if (methodParam.getMethodParamProperties() != null) {
			methodParamModel
					.setMethodParamProperties(new HashSet<MethodParamPropertyModel>());
			for (MethodParamProperty methodParamProperty : methodParam
					.getMethodParamProperties()) {
				methodParamModel.getMethodParamProperties().add(
						extractMethodParamPropertyModel(methodParamProperty));
			}
		}
		return methodParamModel;
	}

	private static MethodParamPropertyModel extractMethodParamPropertyModel(
			MethodParamProperty methodParamProperty) {
		MethodParamPropertyModel methodParamPropertyModel = new MethodParamPropertyModel();
		methodParamPropertyModel.setKey(methodParamProperty.getKey());
		methodParamPropertyModel.setValue(methodParamProperty.getValue());
		return methodParamPropertyModel;
	}

}
