package org.bench4q.master.domain.factory;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.master.Main;
import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.entity.Monitor;
import org.bench4q.master.domain.entity.PlanedConfig;
import org.bench4q.master.domain.entity.RunningAgentDB;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.entity.TestPlanScriptResult;
import org.bench4q.master.domain.entity.User;
import org.bench4q.master.domain.repository.AgentRepository;
import org.bench4q.master.domain.service.AgentService;
import org.bench4q.master.domain.service.ScriptService;
import org.bench4q.master.domain.testplan.LoadDistribute;
import org.bench4q.master.domain.testplan.TestMoniorResultSave;
import org.bench4q.master.domain.testplan.TestScriptResultSave;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.exception.ExceptionUtils.IllegalParameterException;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.master.MonitorModel;
import org.bench4q.share.models.master.RunningScriptModel;
import org.bench4q.share.models.master.TestPlanModel;
import org.bench4q.share.models.master.TestScriptConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class TestPlanFactory {
	private Logger logger = Logger.getLogger(TestPlanFactory.class);
	private ScriptService scriptService;
	private AgentRepository agentRepository;

	private LoadDistribute loadDistribute;
	private AgentMessenger agentMessenger;
	private AgentService agentService;
	private TestScriptResultSave testScriptResultSave;
	private TestMoniorResultSave testMoniorResultSave;

	public LoadDistribute getLoadDistribute() {
		return loadDistribute;
	}

	@Autowired
	public void setLoadDistribute(LoadDistribute loadDistribute) {
		this.loadDistribute = loadDistribute;
	}

	public AgentMessenger getAgentMessenger() {
		return agentMessenger;
	}

	@Autowired
	public void setAgentMessenger(AgentMessenger agentMessenger) {
		this.agentMessenger = agentMessenger;
	}

	public AgentService getAgentService() {
		return agentService;
	}

	@Autowired
	public void setAgentService(AgentService agentService) {
		this.agentService = agentService;
	}

	public TestScriptResultSave getTestScriptResultSave() {
		return testScriptResultSave;
	}

	@Autowired
	public void setTestScriptResultSave(
			TestScriptResultSave testScriptResultSave) {
		this.testScriptResultSave = testScriptResultSave;
	}

	private ScriptService getScriptService() {
		return scriptService;
	}

	@Autowired
	private void setScriptService(ScriptService scriptService) {
		this.scriptService = scriptService;
	}

	private AgentRepository getAgentRepository() {
		return agentRepository;
	}

	@Autowired
	private void setAgentRepository(AgentRepository agentRepository) {
		this.agentRepository = agentRepository;
	}

	public TestMoniorResultSave getTestMoniorResultSave() {
		return testMoniorResultSave;
	}

	@Autowired
	public void setTestMoniorResultSave(
			TestMoniorResultSave testMoniorResultSave) {
		this.testMoniorResultSave = testMoniorResultSave;
	}

	public TestPlan createATestPlanWithoutIdentity(TestPlanModel testPlanModel,
			User user, UUID runId) throws IllegalParameterException {
		TestPlan result = new TestPlan();
		result.setCreateDateTime(new Date());
		result.setCurrentStatus(TestPlanStatus.NotStart.name());
		result.setFailTimes(0);
		result.setName(testPlanModel.getName());
		result.setSampleCycleInSeconds(getLargerOneBetweenUserOptionAndMinInSystem(testPlanModel));
		result.setLastRunningTime(new Date(0));
		result.setTestPlanRunId(runId.toString());
		result.setUser(user);
		Set<TestPlanScript> testPlanScripts = new HashSet<TestPlanScript>();
		int requiredLoad = 0;
		for (RunningScriptModel runningScriptModel : testPlanModel
				.getRunningScriptModels()) {
			if (runningScriptModel.getRequireLoad() <= 0) {
				throw new IllegalParameterException(
						"runningScriptModel's requireLoad is L.T. zero where scriptId is "
								+ runningScriptModel.getScriptId());
			}
			requiredLoad += runningScriptModel.getRequireLoad();
			testPlanScripts.add(createATestPlanScriptWithoutId(
					runningScriptModel.getRequireLoad(),
					runningScriptModel.getScriptId(),
					runningScriptModel.getConfig(), result));
		}
		result.setRequiredLoad(requiredLoad);
		result.setTestPlanScripts(testPlanScripts);

		Set<Monitor> monitors = new HashSet<Monitor>();
		for (MonitorModel monitorModel : testPlanModel.getMonitorModels()) {
			Monitor monitorInBusiness = BusinessModelMapFactory
					.toBusiness(monitorModel);
			monitorInBusiness.setTestPlan(result);
			monitors.add(monitorInBusiness);
		}
		result.setMonitors(monitors);
		return result;
	}

	private int getLargerOneBetweenUserOptionAndMinInSystem(
			TestPlanModel testPlanModel) {
		return testPlanModel.getSampleCycleInSeconds() <= Main.Min_Sample_Cycle_InSecond ? Main.Min_Sample_Cycle_InSecond
				: testPlanModel.getSampleCycleInSeconds();
	}

	public TestPlanScript createATestPlanScriptWithoutId(int requireLoad,
			int scriptId, TestScriptConfig config, TestPlan testPlanDB) {
		TestPlanScript testPlanScript = new TestPlanScript();
		testPlanScript.setRequireLoad(requireLoad);
		testPlanScript.setScript(this.getScriptService().getScript(scriptId));
		testPlanScript.setPlanedConfig(createAPlanedConfigWithoutId(config));
		testPlanScript.setTestPlan(testPlanDB);
		return testPlanScript;
	}

	private PlanedConfig createAPlanedConfigWithoutId(TestScriptConfig config) {
		PlanedConfig planedConfig = new PlanedConfig();
		planedConfig.setWarmUp(config.getWarmUp());
		planedConfig.setExecuteRange(config.getExecuteRange());
		planedConfig.setCoolDown(config.getCoolDown());
		return planedConfig;
	}

	public TestPlanScriptResult createScriptResultWithoutId(Object model,
			TestPlanScript testPlanScript, Date createDatetime) {
		TestPlanScriptResult result = new TestPlanScriptResult();
		result.setTestPlanScript(testPlanScript);
		try {
			result.setResultContent(MarshalHelper.marshal(model.getClass(),
					model));
			result.setResultType(model.getClass().getName());
		} catch (JAXBException e) {
			logger.error(ExceptionLog.getStackTrace(e));
			result.setResultContent("");
		}
		result.setCreateDatetime(createDatetime);
		return result;
	}

	public TestPlan convertToDomain(TestPlan testPlanInRepo) {
		if (testPlanInRepo == null) {
			return null;
		}
		/*
		 * testPlanInRepo.setLoadDistribute(ApplicationContextHelper.getContext()
		 * .getBean(LoadDistribute.class));
		 */
		testPlanInRepo.setLoadDistribute(this.getLoadDistribute());
		for (TestPlanScript testPlanScript : testPlanInRepo
				.getTestPlanScripts()) {
			converToDomain(testPlanScript);
		}
		if (testPlanInRepo.getMonitors() != null) {
			for (Monitor monitor : testPlanInRepo.getMonitors()) {
				convertToDomain(monitor);
			}
		}

		return testPlanInRepo;
	}

	private void converToDomain(TestPlanScript testPlanScript) {
		testPlanScript.setTestPlanFactory(this);
		// testPlanScript.setAgentMessenger(ApplicationContextHelper.getContext()
		// .getBean(AgentMessenger.class));
		// testPlanScript.setAgentService(ApplicationContextHelper.getContext()
		// .getBean(AgentService.class));
		// testPlanScript.addObserver(ApplicationContextHelper.getContext()
		// .getBean(TestScriptResultSave.class));
		testPlanScript.setAgentMessenger(this.getAgentMessenger());
		testPlanScript.setAgentService(this.getAgentService());
		testPlanScript.addObserver(this.getTestScriptResultSave());
	}

	private void convertToDomain(Monitor monitor) {
		/*
		 * monitor.addObserver(ApplicationContextHelper.getContext().getBean(
		 * TestMoniorResultSave.class));
		 */
		monitor.addObserver(this.getTestMoniorResultSave());
	}

	public List<RunningAgentDB> createRunningAgentDBsWithoutId(
			List<? extends RunningAgentInterface> runningAgents,
			TestPlanScript testPlanScript) {
		// TODO: see this kind of map RunningAgent to agent, maybe there's a
		// problem
		List<RunningAgentDB> runningAgentDBs = new ArrayList<RunningAgentDB>();
		for (RunningAgentInterface runningAgent : runningAgents) {
			RunningAgentDB runningAgentDB = new RunningAgentDB();
			runningAgentDB.setAgent(this.getAgentRepository().getAgentBy(
					runningAgent.getAgent().getHostName()));
			runningAgentDB.setAgentRunId(runningAgent.getAgentRunId());
			runningAgentDB.setLoadInUse(runningAgent.getLoadInUse());
			runningAgentDB.setScript(this.getScriptService().getScript(
					runningAgent.getScriptId()));
			runningAgentDB.setTestPlanScript(testPlanScript);
			runningAgentDBs.add(runningAgentDB);
		}
		return runningAgentDBs;
	}
}
