package org.bench4q.master.domain.repository;

import java.util.List;

import org.bench4q.master.domain.entity.plugin.ParamType;
import org.bench4q.master.domain.entity.plugin.Plugin;
import org.bench4q.master.exception.Bench4QException;
import org.bench4q.master.exception.ExceptionUtils.EntityUniqueAlReadyExistException;
import org.bench4q.share.models.master.plugin.ParamTypeEnum;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Component;

@Component
public class PluginRepository extends AbstractRepositoty {
	
	public ParamType getParamTypeByType(ParamTypeEnum paramTypeEnum) {
		ParamType paramType = null;
		Session session = this.getSessionHelper().openSession();
		paramType = (ParamType) session.createCriteria(Plugin.class)
				.add(Restrictions.eq("type", paramTypeEnum)).uniqueResult();
		releaseSession(session);
		return paramType;
	}

	public boolean attatch(Plugin plugin) throws Bench4QException {

		Session session = this.getSessionHelper().openSession();
		Transaction transaction = session.beginTransaction();
		Plugin pluginExist = null;
		try {
			pluginExist = (Plugin) session.createCriteria(Plugin.class)
					.add(Restrictions.eq("name", plugin.getName()))
					.uniqueResult();
			;
			if (pluginExist != null) {
				logger.info("the plugin exist");

				return false;
			}
			session.merge(plugin);
			transaction.commit();
			return true;
		} catch (Exception e) {
			transaction.rollback();
			logger.error(e, e.fillInStackTrace());
			throw new Bench4QException("", "add plugin fail", "");
		} finally {
			releaseSession(session);
		}
	}

	@Override
	protected void guardOtherUniqueConditionForEntity(
			String uniquePropertyName, String value)
			throws EntityUniqueAlReadyExistException {
		if (getPlugin(value) != null) {
			throw new EntityUniqueAlReadyExistException("User with the name "
					+ value + "already exists");
		}
	}

	public boolean detach(String pluginName) throws Bench4QException {
		Session session = this.getSessionHelper().openSession();
		Transaction transaction = session.beginTransaction();
		try {
			Plugin pluginExist = (Plugin) session.createCriteria(Plugin.class)
					.add(Restrictions.eq("name", pluginName)).uniqueResult();
			if (pluginExist == null) {
				logger.info("plugin not exist");
				System.out.println("not exist");
				return false;
			}
			session.delete(pluginExist);
			transaction.commit();
			return true;
		} catch (Exception e) {
			logger.error(e, e.fillInStackTrace());
			transaction.rollback();
			throw new Bench4QException("", "delete plugin fail", "");
		} finally {
			releaseSession(session);
		}
	}

	public List<Plugin> loadPlugins() throws Bench4QException {
		Session session = this.getSessionHelper().openSession();
		try {
			@SuppressWarnings("unchecked")
			List<Plugin> ret = session.createCriteria(Plugin.class)
					.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY).list();
			return ret;
		} catch (Exception e) {
			logger.error("load Plugins ");
			logger.error(e, e.fillInStackTrace());
			throw new Bench4QException("", "load plugin failed", "");
		} finally {
			releaseSession(session);
		}
	}

	public boolean isExist(Plugin plugin) {
		Session session = this.getSessionHelper().openSession();
		Plugin pluginAlreadyExist = (Plugin) session
				.createCriteria(Plugin.class)
				.add(Restrictions.eq("userName", plugin.getName()))
				.uniqueResult();
		releaseSession(session);
		return pluginAlreadyExist != null;
	}

	public Plugin getPlugin(String pluginName) {
		Plugin plugin = null;
		Session session = this.getSessionHelper().openSession();
		plugin = deGetPlugin(pluginName, session);
		plugin.getPluginMethods();
		releaseSession(session);
		return plugin;
	}

	private Plugin deGetPlugin(String pluginName, Session session) {
		return (Plugin) session.createCriteria(Plugin.class)
				.add(Restrictions.eq("name", pluginName)).uniqueResult();
	}
}
