package org.bench4q.master.domain.service;

import java.util.List;

import org.bench4q.master.domain.entity.Agent;
import org.bench4q.master.domain.repository.AgentRepository;
import org.bench4q.master.testplan.highavailable.HighAvailablePool;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class AgentService {
	public static int AGENT_STATUS_Idel = 1;
	public static int AGENT_STATUS_InRun = 2;
	public static int AGENT_STATUS_BackUp = 3;
	public static int AGENT_STATUS_BreakDown = 4;
	private AgentRepository agentRepository;
	private Object AGENT_LOCK;
	private HighAvailablePool highAvailablePool;

	public AgentService() {
		this.setAgentLock(new Object());
	}

	private HighAvailablePool getHighAvailablePool() {
		return highAvailablePool;
	}

	@Autowired
	private void setHighAvailablePool(HighAvailablePool highAvailablePool) {
		this.highAvailablePool = highAvailablePool;
	}

	private AgentRepository getAgentRepository() {
		return agentRepository;
	}

	@Autowired
	private void setAgentRepository(AgentRepository agentRepository) {
		this.agentRepository = agentRepository;
	}

	public Object getAgentLock() {
		return AGENT_LOCK;
	}

	public void setAgentLock(Object object) {
		this.AGENT_LOCK = object;
	}

	public boolean addAgentToPool(Agent agentWithoutId) {
		if (!this.getAgentRepository().attach(
				Agent.createAgentWithoutId(agentWithoutId.getHostName(),
						agentWithoutId.getPort()))) {
			return false;
		}
		Agent agent = this.getAgentRepository().getAgentBy(
				agentWithoutId.getHostName());
		if (agent == null) {
			return false;
		}
		this.getHighAvailablePool().getPool().put(agent.getHostName(), agent);
		return true;
	}

	public boolean removeAgentFromPool(int agentId) {
		return this.getAgentRepository().detach(agentId);
	}

	public List<Agent> loadAgentPoolFromDB() {
		return this.getAgentRepository().loadEntities();
	}

	public boolean backLoadToAgent(String hostName, int backLoad) {
		Agent agent = this.getAgentRepository().getAgentBy(hostName);
		if (agent == null) {
			return false;
		}
		if (!isValidBackLoad(agent, backLoad)) {
			return false;
		}
		agent.setRemainLoad(agent.getRemainLoad() + backLoad);
		return this.getAgentRepository().update(agent);
	}

	private boolean isValidBackLoad(Agent agentToOperate, int backLoad) {
		return backLoad >= 0
				&& agentToOperate.getMaxLoad() - agentToOperate.getRemainLoad() >= backLoad;
	}

	public boolean getLoadFromAgent(String hostName, int requireLoad) {
		Agent agent = this.getAgentRepository().getAgentBy(hostName);
		if (!isValidRequireLoad(agent, requireLoad) || agent == null) {
			return false;
		}
		agent.setRemainLoad(agent.getRemainLoad() - requireLoad);
		return this.getAgentRepository().update(agent);
	}

	private boolean isValidRequireLoad(Agent agentToOperate, int requireLoad) {
		return requireLoad >= 0
				&& agentToOperate.getRemainLoad() >= requireLoad;
	}

	public boolean resetAgent(Agent agentInParam) {
		Agent agentInRepo = this.getAgentRepository().getAgentBy(
				agentInParam.getHostName());
		if (agentInRepo == null) {
			return false;
		}
		agentInRepo.setRemainLoad(agentInRepo.getMaxLoad());
		return this.getAgentRepository().update(agentInRepo);
	}

	public boolean updateAgentStatus(int currentStatus, String hostName) {
		Agent agent = this.getAgentRepository().getAgentBy(hostName);
		if (agent == null) {
			return false;
		}
		agent.setCurrentStatus(currentStatus);
		return this.getAgentRepository().update(agent);
	}

}
