package org.bench4q.master.domain.service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.entity.plugin.MethodParam;
import org.bench4q.master.domain.entity.plugin.Plugin;
import org.bench4q.master.domain.repository.PluginRepository;
import org.bench4q.master.exception.Bench4QException;
import org.bench4q.master.domain.entity.plugin.Method;
import org.bench4q.master.domain.factory.PluginFactory;
import org.bench4q.share.models.master.plugin.MethodParamModel;
import org.bench4q.share.models.master.plugin.PluginModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class PluginService {
	private Set<Plugin> plugins;
	private List<String> pluginNameList;
	private PluginRepository pluginRepository;
	private PluginFactory pluginFactory;
	private Logger logger = Logger.getLogger(PluginService.class);

	public PluginRepository getPluginRepository() {
		return pluginRepository;
	}

	@Autowired
	public void setPluginRepository(PluginRepository pluginRepository) {
		this.pluginRepository = pluginRepository;
	}

	public PluginFactory getPluginFactory() {
		return pluginFactory;
	}

	@Autowired
	public void setPluginFactory(PluginFactory pluginFactory) {
		this.pluginFactory = pluginFactory;
	}

	private Set<Plugin> getPlugins() throws Bench4QException {
		List<Plugin> loadPluginList = this.getPluginRepository().loadPlugins();
		if (loadPluginList != null) {
			this.plugins = new HashSet<Plugin>();
			for (Plugin plugin : loadPluginList) {
				this.plugins.add(plugin);
			}
		}

		return this.plugins;
	}

	public boolean addPlugin(PluginModel pluginModel) throws Bench4QException {
		return this.getPluginRepository().attatch(
				this.getPluginFactory().createPluginEntity(pluginModel));
	}

	public boolean deletePlugin(String pluginName) throws Bench4QException {
		return this.getPluginRepository().detach(pluginName);
	}

	private Plugin getPluginByName(String pluginName) {
		return this.getPluginRepository().getPlugin(pluginName);
	}

	public List<String> getPluginNameList() throws Bench4QException {
		this.pluginNameList = new ArrayList<String>();
		this.getPlugins();
		if (this.plugins != null) {
			for (Plugin plugin : this.plugins) {
				this.pluginNameList.add(plugin.getName());
			}
		}
		return this.pluginNameList;
	}

	public List<String> getMethodNameInPlugin(String pluginName) {
		List<String> methodNameList = new ArrayList<String>();
		Set<Method> methods = this.getPluginByName(pluginName)
				.getPluginMethods();

		if (methods != null) {
			for (Method method : methods) {
				methodNameList.add(method.getName());
			}
		}
		return methodNameList;
	}

	private Method getMethodInPlugin(String pluginName, String methodName)
			throws Bench4QException {
		Set<Method> methods = this.getMethodInPlugin(pluginName);
		if (methods != null) {
			for (Method method : methods) {
				if (method.getName().equals(methodName))
					return method;
			}
		}
		logger.info("method is empty");

		throw new Bench4QException("", "no such method:" + methodName
				+ "in plugin:" + pluginName, "");

	}

	public Set<MethodParamModel> getMethodParamModelsInPlugin(
			String pluginName, String methodName) throws Bench4QException {
		return PluginFactory.extractMethodModel(
				this.getMethodInPlugin(pluginName, methodName))
				.getMethodParams();
	}

	private Set<Method> getMethodInPlugin(String pluginName) {

		return this.getPluginByName(pluginName).getPluginMethods();
	}

	public Set<MethodParam> getMethodParams(String pluginName, String methodName)
			throws Bench4QException {
		Method method = this.getMethodInPlugin(pluginName, methodName);
		return method.getMethodParams();
	}

}
