package org.bench4q.master.domain.service;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.entity.TestPlanScriptResult;
import org.bench4q.master.domain.repository.TestPlanRepository;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.helper.SessionHelper;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.master.ValueTimeModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptPagesBriefModel;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class TestPlanScriptResultService {
	private TestPlanScriptService testPlanScriptService;
	private TestPlanRepository testPlanRepository;
	private SessionHelper sessionHelper;
	private static Logger logger = Logger
			.getLogger(TestPlanScriptResultService.class);

	private TestPlanScriptService getTestPlanScriptService() {
		return testPlanScriptService;
	}

	@Autowired
	private void setTestPlanScriptService(
			TestPlanScriptService testPlanScriptService) {
		this.testPlanScriptService = testPlanScriptService;
	}

	public SessionHelper getSessionHelper() {
		return sessionHelper;
	}

	@Autowired
	public void setSessionHelper(SessionHelper sessionHelper) {
		this.sessionHelper = sessionHelper;
	}

	private TestPlanRepository getTestPlanRepository() {
		return testPlanRepository;
	}

	@Autowired
	private void setTestPlanRepository(TestPlanRepository testPlanRepository) {
		this.testPlanRepository = testPlanRepository;
	}

	/**
	 * Here I just use the Time that createDateTime in testplanscriptResult, If
	 * it's not accuracy, i just need to modify the way of pick createDateTime
	 * 
	 * @param testPlanId
	 * @param scriptId
	 * @return
	 */
	public List<ValueTimeModel> loadScriptBriefSpecificField(UUID testPlanId,
			int scriptId, String fieldName) {
		List<TestPlanScriptResult> scriptBriefresults = this
				.getTestPlanScriptService().queryScriptBriefResults(testPlanId,
						scriptId);
		List<ValueTimeModel> ret = new ArrayList<ValueTimeModel>();
		for (TestPlanScriptResult result : scriptBriefresults) {
			try {
				ScriptBriefResultModel briefModel = (ScriptBriefResultModel) MarshalHelper
						.unmarshal(ScriptBriefResultModel.class,
								result.getResultContent());
				ret.add(ValueTimeModel.buildValueTimeModel(
						getSpecificFieldValue(briefModel, fieldName),
						result.getCreateDatetime()));
			} catch (JAXBException e) {
				logger.info(ExceptionLog.getStackTrace(e)
						+ " When unmarshal the model from ResultCOntent");
				continue;
			} catch (NoSuchFieldException e) {
				logger.info(ExceptionLog.getStackTrace(e)
						+ " When get Field from the model from ResultCOntent");
				continue;
			}
		}
		return ret;
	}

	private long getSpecificFieldValue(ScriptBriefResultModel briefModel,
			String fieldName) throws NoSuchFieldException {
		try {
			Field field = getFieldAndSetAccessible(briefModel, fieldName);
			return (Long) field.get(briefModel);
		} catch (Exception e) {
			throw new NoSuchFieldException();
		}
	}

	private Field getFieldAndSetAccessible(ScriptBriefResultModel briefModel,
			String fieldName) throws NoSuchFieldException, SecurityException {
		Field field = briefModel.getClass().getDeclaredField(fieldName);
		field.setAccessible(true);
		return field;
	}

	public List<ScriptBriefResultModel> loadScriptBriefWithDuation(
			UUID testPlanId, int scriptId, long startTime) {
		Session session = this.getSessionHelper().openSession();
		List<ScriptBriefResultModel> result = new ArrayList<ScriptBriefResultModel>();
		try {
			List<TestPlanScriptResult> scriptResults = doGetTestPlanScriptResults(
					testPlanId, scriptId, startTime,
					ScriptBriefResultModel.class, session);
			for (TestPlanScriptResult testPlanScriptResult : scriptResults) {
				result.add((ScriptBriefResultModel) MarshalHelper.unmarshal(
						ScriptBriefResultModel.class,
						testPlanScriptResult.getResultContent()));
			}
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
		}
		return result;
	}

	private List<TestPlanScriptResult> doGetTestPlanScriptResults(
			UUID testPlanId, int scriptId, long startTime, Class<?> resultType,
			Session session) {
		TestPlanScript testPlanScript = this.getTestPlanRepository()
				.getTestPlanBy(testPlanId).extracSpecifiedScript(scriptId);
		@SuppressWarnings("unchecked")
		List<TestPlanScriptResult> results = (List<TestPlanScriptResult>) session
				.createCriteria(TestPlanScriptResult.class)
				.add(Restrictions.eq("testPlanScript", testPlanScript))
				.add(Restrictions.eq("resultType", resultType.getName()))
				.add(Restrictions.ge("createDatetime", new Date(startTime)))
				.list();
		return results;
	}

	public ScriptBehaviorsBriefModel getScriptBehaviorsBrief(UUID testPlanId,
			int scriptId) {
		Session session = this.getSessionHelper().openSession();
		try {
			TestPlanScriptResult testPlanScriptResult = doGetLastSampleResult(
					testPlanId, scriptId, ScriptBehaviorsBriefModel.class,
					session);
			if (testPlanScriptResult == null) {
				return new ScriptBehaviorsBriefModel();
			}
			return (ScriptBehaviorsBriefModel) MarshalHelper.unmarshal(
					ScriptBehaviorsBriefModel.class,
					testPlanScriptResult.getResultContent());
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
		}
		return new ScriptBehaviorsBriefModel();
	}

	private TestPlanScriptResult doGetLastSampleResult(UUID testPlanId,
			int scriptId, Class<?> resultType, Session session) {
		TestPlanScript testPlanScript = this.getTestPlanRepository()
				.getTestPlanBy(testPlanId).extracSpecifiedScript(scriptId);
		@SuppressWarnings("unchecked")
		List<TestPlanScriptResult> results = (List<TestPlanScriptResult>) session
				.createCriteria(TestPlanScriptResult.class)
				.add(Restrictions.eq("testPlanScript", testPlanScript))
				.add(Restrictions.eq("resultType", resultType.getName()))
				.addOrder(Order.desc("createDatetime")).setFetchSize(1).list();
		if (results.size() > 0) {
			return results.get(0);
		}
		return null;
	}

	public ScriptPagesBriefModel getScriptPagesBrief(UUID testPlanId,
			int scriptId) {
		Session session = this.getSessionHelper().openSession();
		ScriptPagesBriefModel result = new ScriptPagesBriefModel();
		try {
			TestPlanScriptResult testPlanScriptResult = doGetLastSampleResult(
					testPlanId, scriptId, ScriptPagesBriefModel.class, session);
			if (testPlanScriptResult == null) {
				return result;
			}
			result = (ScriptPagesBriefModel) MarshalHelper.unmarshal(
					ScriptPagesBriefModel.class,
					testPlanScriptResult.getResultContent());
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e));
		}
		return result;
	}
}
