package org.bench4q.master.domain.service;

import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.entity.Monitor;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.entity.User;
import org.bench4q.master.domain.factory.BusinessModelMapFactory;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.domain.repository.TestPlanRepository;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.exception.ExceptionUtils.IllegalParameterException;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.bench4q.share.models.master.MonitorModel;
import org.bench4q.share.models.master.TestPlanModel;
import org.bench4q.share.models.master.TestPlanDBModel;
import org.bench4q.share.models.master.TestPlanResultModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class TestPlanService {
	private BusinessModelMapFactory businessMapFactory;
	private TestPlanRepository testPlanRepository;
	private TestPlanFactory testPlanFactory;
	public static final long TIME_UNIT = 1000;

	private TestPlanRepository getTestPlanRepository() {
		return testPlanRepository;
	}

	@Autowired
	private void setTestPlanRepository(TestPlanRepository testPlanRepository) {
		this.testPlanRepository = testPlanRepository;
	}

	private BusinessModelMapFactory getBusinessMapFactory() {
		return businessMapFactory;
	}

	@Autowired
	private void setBusinessMapFactory(
			BusinessModelMapFactory businessMapFactory) {
		this.businessMapFactory = businessMapFactory;
	}

	private TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	@Autowired
	private void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	public boolean submitTestPlan(final TestPlanModel testPlanBusinessModel,
			final User user, final UUID testPlanRunId) {
		try {
			return this.getTestPlanRepository().attach(
					this.getTestPlanFactory().createATestPlanWithoutIdentity(
							testPlanBusinessModel, user, testPlanRunId));
		} catch (IllegalParameterException e) {
			// TODO: give back the message
			Logger.getLogger(TestPlanService.class).error(
					ExceptionLog.getStackTrace(e) + e.getMessage());
			return false;
		}
	}

	public List<TestPlan> loadTestPlans(User user) {
		return this.getTestPlanRepository().loadEntities(user);
	}

	public boolean removeTestPlanInDB(int testPlanId) {
		return this.getTestPlanRepository().detach(testPlanId);
	}

	public TestPlanStatus queryTestPlanStatus(UUID testPlanRunID) {
		return TestPlanStatus.valueOf(this.getTestPlanRepository()
				.getTestPlanBy(testPlanRunID).getCurrentStatus());
	}

	public TestPlanDBModel getTestPlanDBModel(UUID runId) {
		return this.getBusinessMapFactory().toModel(
				this.getTestPlanRepository().getTestPlanBy(runId));
	}

	public TestPlanResultModel buildResultModel(UUID runId) {
		TestPlanResultModel resultModel = new TestPlanResultModel();
		TestPlan testPlan = this.getTestPlanRepository().getTestPlanBy(runId);
		resultModel.setCurrentStatus(TestPlanStatus.valueOf(testPlan
				.getCurrentStatus()));
		resultModel.setTestPlanId(runId);
		List<Integer> scriptIds = new LinkedList<Integer>();
		for (TestPlanScript testPlanScript : testPlan.getTestPlanScripts()) {
			scriptIds.add(testPlanScript.getScript().getId());
		}
		resultModel.setScriptIds(scriptIds);
		List<MonitorModel> monitorModels = new LinkedList<MonitorModel>();
		for (Monitor monitor : testPlan.getMonitors()) {
			monitorModels.add(BusinessModelMapFactory.toModel(monitor));
		}
		resultModel.setMonitorModels(monitorModels);
		return resultModel;
	}
}
