package org.bench4q.master.domain.testplan;

import java.util.List;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.RunningScriptInterface;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.repository.TestPlanRepository;
import org.bench4q.master.testplan.highavailable.HighAvailablePool;
import org.bench4q.master.transaction.script.ScriptLoadCommand;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class LoadDistribute {

	private HighAvailablePool highAvailableAgentPool;
	private TestPlanRepository testPlanRepository;
	private static Logger logger = Logger.getLogger(LoadDistribute.class);

	public HighAvailablePool getHighAvailableAgentPool() {
		return highAvailableAgentPool;
	}

	@Autowired
	public void setHighAvailableAgentPool(
			HighAvailablePool highAvailableAgentPool) {
		this.highAvailableAgentPool = highAvailableAgentPool;
	}

	private TestPlanRepository getTestPlanRepository() {
		return testPlanRepository;
	}

	@Autowired
	private void setTestPlanRepository(TestPlanRepository testPlanRepository) {
		this.testPlanRepository = testPlanRepository;
	}

	private boolean hasEnoughCurrentLoad(int totalRequireLoad) {
		return this.highAvailableAgentPool.getCurrentAvailableLoad() >= totalRequireLoad;
	}

	private boolean hasEnoughMaxLoad(int totalRequireLoad) {
		return this.getHighAvailableAgentPool().getMaxAvailableLoad() >= totalRequireLoad;
	}

	public TestPlanStatus generateLoadForTestPlan(TestPlan testPlanInDomain) {
		synchronized (this.highAvailableAgentPool.getPool()) {
			if (!hasEnoughMaxLoad(testPlanInDomain.getRequiredLoad())) {
				logger.error("There is no enough max load in the pool!");
				return TestPlanStatus.PendingNoEnoughMaxLoad;
			}
			if (!hasEnoughCurrentLoad(testPlanInDomain.getRequiredLoad())) {
				logger.info("There is no enough current load in the pool!");
				return TestPlanStatus.PendingNoEnoughCurrentLoad;
			}
			for (RunningScriptInterface testPlanScript : testPlanInDomain
					.getTestPlanScripts()) {
				if (!distributeLoadForScript(testPlanScript)) {
					return TestPlanStatus.ErrorInDistributeLoadForScript;
				}
			}
			doAfterDistributeLoadForTestPlan(testPlanInDomain);
			return TestPlanStatus.InRunning;
		}
	}

	private void doAfterDistributeLoadForTestPlan(TestPlan testPlanInDomain) {
		this.getHighAvailableAgentPool().timerTask();
		testPlanInDomain.doAfterRun();
	}

	public boolean distributeLoadForScript(RunningScriptInterface runningScript) {
		ScriptLoadCommand scriptLoadCommand = new ScriptLoadCommand(
				runningScript, runningScript.getTestPlanID());
		List<? extends RunningAgentInterface> runningAgents = scriptLoadCommand
				.execute();
		if (runningAgents == null) {
			logger.error("distribute load error, the runningAgents is null");
			scriptLoadCommand.rollBack();
			return false;
		}
		doAfterDistributeLoadSuccessForScript(runningScript, runningAgents);
		return true;
	}

	private void doAfterDistributeLoadSuccessForScript(
			RunningScriptInterface scriptInput,
			List<? extends RunningAgentInterface> runningAgents) {
		scriptInput.doAfterDistributeLoad(runningAgents);
		if (!(scriptInput instanceof TestPlanScript)) {
			return;
		}
		this.getTestPlanRepository().updateEntity(
				((TestPlanScript) scriptInput).getTestPlan());
	}
}
