package org.bench4q.master.domain.testplan;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.helper.ApplicationContextHelper;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.master.testplan.datastatistics.BehaviorsBriefStatistics;
import org.bench4q.master.testplan.datastatistics.DataStatistics;
import org.bench4q.master.testplan.datastatistics.PagesBriefStatistics;
import org.bench4q.master.testplan.datastatistics.ScriptBriefStatistics;
import org.bench4q.master.testplan.datastatistics.ScriptStatistics;
import org.bench4q.share.models.agent.statistics.AgentBriefStatusModel;
import org.bench4q.share.models.agent.statistics.DataStatisticsModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptPagesBriefModel;

public class RunningScriptSampler {
	private List<DataStatistics> dataStatisticsList;
	private List<RunningAgentInterface> runningAgents;
	private AgentMessenger agentMessenger;

	private List<DataStatistics> getDataStatisticsList() {
		return dataStatisticsList;
	}

	private void setDataStatisticsList(List<DataStatistics> dataStatisticsList) {
		this.dataStatisticsList = dataStatisticsList;
	}

	public List<RunningAgentInterface> getRunningAgents() {
		return runningAgents;
	}

	private void setRunningAgents(List<RunningAgentInterface> runningAgents) {
		this.runningAgents = runningAgents;
	}

	private AgentMessenger getAgentMessenger() {
		return agentMessenger;
	}

	private void setAgentMessenger(AgentMessenger agentMessenger) {
		this.agentMessenger = agentMessenger;
	}

	private RunningScriptSampler() {
		this.setDataStatisticsList(new ArrayList<DataStatistics>());
		this.getDataStatisticsList().add(new ScriptBriefStatistics());
		this.getDataStatisticsList().add(new BehaviorsBriefStatistics());
		this.getDataStatisticsList().add(new PagesBriefStatistics());
		this.setAgentMessenger(ApplicationContextHelper.getContext().getBean(
				AgentMessenger.class));
		this.setRunningAgents(new LinkedList<RunningAgentInterface>());
	}

	public RunningScriptSampler(
			Collection<? extends RunningAgentInterface> runningAgents) {
		this();
		this.getRunningAgents().addAll(runningAgents);
	}

	public ScriptBriefResultModel getScriptBrief() {
		for (RunningAgentInterface runningAgent : getRunningAgents()) {
			if (runningAgent.isBreakDown()) {
				continue;
			}
			AgentBriefStatusModel briefStatusModel = this.getAgentMessenger()
					.brief(runningAgent.getAgent(),
							runningAgent.getAgentRunId());
			if (briefStatusModel == null) {
				continue;
			}
			getScriptBriefStatistics().add(briefStatusModel);
		}
		return (ScriptBriefResultModel) getScriptBriefStatistics()
				.getStatistics();
	}

	public ScriptBehaviorsBriefModel getBehaviorsBrief() {
		for (RunningAgentInterface runningAgent : getRunningAgents()) {
			getBehaviorsBriefStatistics().add(
					this.getAgentMessenger().behaviorsBrief(
							runningAgent.getAgent(),
							runningAgent.getAgentRunId()));
		}

		ScriptBehaviorsBriefModel result = (ScriptBehaviorsBriefModel) getBehaviorsBriefStatistics()
				.getStatistics();
		return result;
	}

	public ScriptPagesBriefModel getPagesBrief() {
		for (RunningAgentInterface runningAgent : getRunningAgents()) {
			this.getPagesBriefStatistics().add(
					this.getAgentMessenger().pagesBrief(
							runningAgent.getAgent(),
							runningAgent.getAgentRunId()));
		}
		ScriptPagesBriefModel result = (ScriptPagesBriefModel) this
				.getPagesBriefStatistics().getStatistics();
		return result;
	}

	private DataStatistics getScriptBriefStatistics() {
		return getSpecificDataStatistics(ScriptBriefStatistics.class);
	}

	private DataStatistics getBehaviorsBriefStatistics() {
		return getSpecificDataStatistics(BehaviorsBriefStatistics.class);
	}

	public DataStatistics getPagesBriefStatistics() {
		return getSpecificDataStatistics(PagesBriefStatistics.class);
	}

	private DataStatistics getSpecificDataStatistics(
			Class<? extends DataStatistics> dataStatisticsClass) {
		for (DataStatistics dataStatistics : this.getDataStatisticsList()) {
			if (dataStatistics.getClass().equals(dataStatisticsClass)) {
				return dataStatistics;
			}
		}
		return new ScriptStatistics() {
			public void add(DataStatisticsModel dataUnit) {
			}

			public Object getStatistics() {
				return null;
			}
		};
	}
}
