package org.bench4q.master.domain.testplan;

import java.lang.reflect.Field;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.UUID;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.entity.Monitor;
import org.bench4q.master.domain.entity.MonitorResult;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.repository.TestPlanRepository;
import org.bench4q.master.helper.SessionHelper;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.master.statistics.SampleModel;
import org.bench4q.share.models.monitor.MonitorMain;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class TestMoniorResultSave implements Observer {
	private Logger logger = Logger.getLogger(TestMoniorResultSave.class);
	private SessionHelper sessionHelper;
	private TestPlanRepository testPlanRepository;

	public SessionHelper getSessionHelper() {
		return sessionHelper;
	}

	@Autowired
	public void setSessionHelper(SessionHelper sessionHelper) {
		this.sessionHelper = sessionHelper;
	}

	private TestPlanRepository getTestPlanRepository() {
		return testPlanRepository;
	}

	@Autowired
	private void setTestPlanRepository(TestPlanRepository testPlanRepository) {
		this.testPlanRepository = testPlanRepository;
	}

	public void update(Observable o, Object arg) {
		// TODO Auto-generated method stub

		Monitor monitor = (Monitor) o;
		logger.info("enter monitor update:" + monitor.getHostName());
		this.saveMonitorResult(
				UUID.fromString(monitor.getTestPlan().getTestPlanRunId()),
				(MonitorMain) arg, ((SampleModel) arg).getSamplingTime(),
				monitor);
	}

	public boolean saveMonitorResult(UUID testPlanRunId, MonitorMain mainModel,
			Date createDatetime, Monitor monitor) {
		Session session = this.getSessionHelper().openSession();
		try {
			TestPlan testPlan = this.getTestPlanRepository().doGetTestPlanBy(
					session, testPlanRunId);
			if (testPlan == null) {
				return false;
			}
			monitor = testPlan.extractSpecifiedMonitor(monitor.getHostName());
			Set<MonitorResult> monitorResultToAdd = monitor.getResults();
			List<MonitorResult> monitorResults = this
					.extractMonitorResultFromMonitorModel(mainModel, testPlan,
							monitor, createDatetime);
			if (monitorResults == null)
				return false;
			if (monitorResults.size() == 0) {
				System.out.println("size is 0 ");
				return false;
			}

			for (MonitorResult monitorResult : monitorResults) {
				if (session == null)
					return false;
				monitorResultToAdd.add(monitorResult);
			}
			logger.info("monitor ressult size:" + monitorResultToAdd.size());
			return this.getTestPlanRepository().doUpdateEntity(session,
					testPlan);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			if (session.isConnected()) {
				session.close();
			}
		}

	}

	private List<MonitorResult> extractMonitorResultFromMonitorModel(
			MonitorMain monitorMain, TestPlan testPlan, Monitor monitor,
			Date createDatetime) throws IllegalArgumentException,
			IllegalAccessException, JAXBException {

		List<MonitorResult> monitorResults = new LinkedList<MonitorResult>();

		Field[] fields = monitorMain.getClass().getDeclaredFields();

		for (int i = 0; i < fields.length; i++) {
			MonitorResult monitorResult = new MonitorResult();
			monitorResult.setTestPlanDB(testPlan);
			monitorResult.setMonitor(monitor);
			monitorResult.setCreateDatetime(createDatetime);
			monitorResult.setHostNameUnderMonitor(monitor.getHostName());
			fields[i].setAccessible(true);
			monitorResult.setType(fields[i].getName());
			monitorResult.setContent(MarshalHelper.marshal(fields[i].getType(),
					fields[i].get(monitorMain)));
			monitorResults.add(monitorResult);
		}
		return monitorResults;

	}
}
