package org.bench4q.master.domain.testplan;

import java.util.Date;
import java.util.Observable;
import java.util.Observer;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.RunningScriptInterface;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.domain.repository.TestPlanRepository;
import org.bench4q.master.helper.SessionHelper;
import org.bench4q.share.models.master.statistics.SampleModel;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class TestScriptResultSave implements Observer {
	private TestPlanFactory testPlanFactory;
	private SessionHelper sessionHelper;
	private TestPlanRepository testPlanRepository;
	private Logger logger = Logger.getLogger(TestScriptResultSave.class);

	private TestPlanRepository getTestPlanRepository() {
		return testPlanRepository;
	}

	@Autowired
	private void setTestPlanRepository(TestPlanRepository testPlanRepository) {
		this.testPlanRepository = testPlanRepository;
	}

	private SessionHelper getSessionHelper() {
		return sessionHelper;
	}

	@Autowired
	private void setSessionHelper(SessionHelper sessionHelper) {
		this.sessionHelper = sessionHelper;
	}

	private TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	@Autowired
	private void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	public void update(Observable o, Object arg) {
		// TODO Auto-generated method stub
		RunningScriptInterface runningScript = (RunningScriptInterface) o;
		this.doSaveResult(runningScript.getTestPlanID(),
				runningScript.getScriptId(), arg,
				((SampleModel) arg).getSamplingTime());
	}

	public boolean doSaveResult(UUID testPlanRunId, int scriptId,
			Object resultModel, Date now) {
		Session session = this.getSessionHelper().openSession();
		try {
			TestPlan testPlan = this.getTestPlanRepository().doGetTestPlanBy(
					session, testPlanRunId);
			if (testPlan == null) {
				return false;
			}
			TestPlanScript specifiedScript = testPlan
					.extracSpecifiedScript(scriptId);
			specifiedScript.getTestPlanScriptResults().add(
					this.getTestPlanFactory().createScriptResultWithoutId(
							resultModel, specifiedScript, now));
			this.getTestPlanRepository().doUpdateEntity(session, testPlan);
			return true;
		} catch (Exception e) {
			logger.error(e, e.fillInStackTrace());
		} finally {
			if (session != null) {
				session.close();
			}
		}
		return false;
	}

}
