package org.bench4q.master.scriptrecord.httpcapture.generator;

import java.io.ByteArrayInputStream;
import java.nio.charset.Charset;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.bench4q.master.exception.ExceptionLog;

public class ResponseParser {
	private Logger logger = Logger.getLogger(ResponseParser.class);
	private byte[] response;
	private ResponseHeader responseHeader;
	private String responseBody;

	private byte[] getResponse() {
		return response;
	}

	private void setResponse(byte[] response) {
		this.response = response;
	}

	public ResponseHeader getResponseHeader() {
		return responseHeader;
	}

	private void setResponseHeader(ResponseHeader responseHeader) {
		this.responseHeader = responseHeader;
	}

	public String getResponseBody() {
		return responseBody;
	}

	private void setResponseBody(String responseBody) {
		this.responseBody = responseBody;
	}

	public ResponseParser(byte[] response) {
		this.setResponse(response);
		this.setResponseHeader(parseResponseHeader());
		if (this.getResponseHeader().isValidResponseHeader()
				&& this.getResponseHeader().isHtmlContentType()) {
			this.parseResponseBody();
		}
	}

	private ResponseHeader parseResponseHeader() {
		ResponseHeader result = new ResponseHeader();
		result.setCharset(parseCharset());
		result.setContentEncoding(parseContentEncoding());
		result.setContentLength(Integer.parseInt(parseContentLength()));
		result.setContentType(parseContentType());
		result.setRespCode(parseResponseCode());
		return result;
	}

	private String preprocess(String respString) {
		return respString.toLowerCase();
	}

	private String parseContentLength() {
		String respStr = preprocess(new String(this.getResponse()));
		int pos = respStr.indexOf("content-length:");
		if (pos > -1) {
			pos += 15;
			int end = respStr.indexOf("\r\n", pos);
			return respStr.substring(pos, end).trim();
		}
		return "-1";
	}

	private String parseContentEncoding() {
		String respStr = preprocess(new String(this.getResponse()));
		int pos = respStr.indexOf("content-encoding:");
		if (pos > -1) {
			pos += 18;
			int end = respStr.indexOf("\r\n", pos);
			return respStr.substring(pos, end);
		}
		return null;
	}

	private String parseCharset() {
		String response = preprocess(new String(this.getResponse()));
		String ret = null;
		int pos = response.indexOf("content-type:");
		if (pos > -1) {
			pos += 14;
			int end = response.indexOf("\r\n", pos);
			int middle = response.indexOf(";", pos);
			if (middle > -1 && middle < end) {
				ret = response.substring(middle + 1, end);
			}
		}
		if (ret != null) {
			int begin = ret.indexOf("charset=");
			ret = ret.substring(begin + 8);
		}
		return ret;
	}

	private String parseContentType() {
		String response = preprocess(new String(this.getResponse()));
		String contentType = null;
		int pos = response.indexOf("content-type:");
		if (pos > -1) {
			pos += 14;
			int end = response.indexOf("\r\n", pos);
			int end2 = response.indexOf(";", pos);
			if ((end2 > -1) && (end2 < end))
				end = end2;
			if (end > -1)
				contentType = response.substring(pos, end).trim();

			logger.debug("  Content-Type: " + contentType);
		} else {
			logger.debug("  No content-type header!  First few lines:");
			StringTokenizer st = new StringTokenizer(response, "\n");
			int i = 0;
			while ((st.hasMoreTokens()) && (i < 5)) {
				logger.debug(st.nextToken());
				++i;
			}
		}
		return contentType;
	}

	private String parseResponseCode() {
		String response = preprocess(new String(this.getResponse()));
		String respCode = null;
		int pos = response.indexOf(" ");
		if (pos != -1) {
			int end = response.indexOf(" ", pos + 1);
			int end2 = response.indexOf("\n", pos + 1);
			if ((end2 != -1) && (end2 < end))
				end = end2;
			if (end != -1)
				respCode = response.substring(pos + 1, end).trim();
		}
		logger.debug("HTTP response code: " + respCode);
		return respCode;
	}

	private void parseResponseBody() {
		ContentDecoder contentDecoder = ContentDecoder.createDecoder(this
				.getResponseHeader().getContentEncoding());
		byte[] contentBodyAfterDecoded = contentDecoder
				.decodeContent(new ByteArrayInputStream(this.getResponse(),
						this.getResponse().length
								- this.getResponseHeader().getContentLength(),
						this.getResponseHeader().getContentLength()));
		if (contentBodyAfterDecoded == null) {
			return;
		}
		Charset charset = null;
		try {
			charset = Charset.forName(this.getResponseHeader().getCharset());
		} catch (Exception e) {
			charset = Charset.forName("utf-8");
			logger.error(ExceptionLog.getStackTrace(e));
		}
		this.setResponseBody(new String(contentBodyAfterDecoded, charset));
	}
}
