package org.bench4q.master.testplan.highavailable;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.RunningScriptInterface;
import org.bench4q.master.domain.entity.Agent;
import org.bench4q.master.domain.service.AgentService;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.share.models.agent.ServerStatusModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

@Component
public class HighAvailablePool extends CurrentLoadSubject {
	private AgentService agentService;
	private AgentMessenger agentMessenger;
	private Map<String, Agent> pool;
	private Map<String, ServerStatusModel> agentStatusOfPreviousBeatMap;
	private Map<UUID, AgentRunBlotter> agentRunBlotters;
	private List<UUID> agentRunIdShouldBeSubstitute;
	private long maxAvailableLoad;
	private int currentAvailableLoad;
	static Logger logger = Logger.getLogger(HighAvailablePool.class);

	public Map<String, Agent> getPool() {
		return this.pool;
	}

	private void setPool(Map<String, Agent> pool) {
		this.pool = pool;
	}

	@Autowired
	public void setAgentService(AgentService agentService) {
		this.agentService = agentService;
	}

	private AgentMessenger getAgentMessenger() {
		return agentMessenger;
	}

	@Autowired
	private void setAgentMessenger(AgentMessenger agentMessenger) {
		this.agentMessenger = agentMessenger;
	}

	public Long getMaxAvailableLoad() {
		this.calculateHAPoolLoadStatusInMonopolize();
		return maxAvailableLoad;
	}

	private void setMaxAvailableLoad(Long maxAvailableLoad) {
		this.maxAvailableLoad = maxAvailableLoad;
	}

	public int getCurrentAvailableLoad() {
		this.calculateHAPoolLoadStatusInMonopolize();
		return currentAvailableLoad;
	}

	private void setCurrentAvailableLoad(int currentAvailableLoad) {
		this.currentAvailableLoad = currentAvailableLoad;
	}

	private void setAgentStatusOfPreviousBeatMap(
			Map<String, ServerStatusModel> agentStatusOfPreviousBeatMap) {
		this.agentStatusOfPreviousBeatMap = agentStatusOfPreviousBeatMap;
	}

	public Map<UUID, AgentRunBlotter> getAgentRunBlotters() {
		return agentRunBlotters;
	}

	private void setAgentRunBlotters(Map<UUID, AgentRunBlotter> agentRunningInfo) {
		this.agentRunBlotters = agentRunningInfo;
	}

	private List<UUID> getAgentRunIdShouldBeSubstitute() {
		return agentRunIdShouldBeSubstitute;
	}

	private void setAgentRunIdShouldBeSubstitute(
			List<UUID> agentRunIdShouldBeSubstitute) {
		this.agentRunIdShouldBeSubstitute = agentRunIdShouldBeSubstitute;
	}

	public HighAvailablePool() {
		this.setPool(new HashMap<String, Agent>());
		this.setAgentRunBlotters(new HashMap<UUID, AgentRunBlotter>());
		this.setAgentStatusOfPreviousBeatMap(new HashMap<String, ServerStatusModel>());
		this.setCurrentAvailableLoad(0);
		this.setAgentRunIdShouldBeSubstitute(new LinkedList<UUID>());
	}

	@Scheduled(cron = "0,30 */1 * * * *")
	public void timerTask() {
		synchronized (this.agentService.getAgentLock()) {
			synchronized (this.getPool()) {
				heartBeatsAndUpdateHAPool();
				doSubstituteIfRequired();
			}
		}
		logger.info("Rotation Over");
	}

	private void doSubstituteIfRequired() {
		for (UUID agentRunId : this.getAgentRunIdShouldBeSubstitute()) {
			AgentRunBlotter agentRunBlotter = this.getAgentRunBlotters().get(
					agentRunId);
			if (agentRunBlotter == null) {
				// TODO:Actually this is an error
				continue;
			}
			agentRunBlotter.substituteOnBoard(agentRunId);
		}
	}

	private void calculateHAPoolLoadStatusInMonopolize() {
		int maxLoad = 0, availableLoad = 0;
		for (Agent agent : this.getPool().values()) {
			if (agent.getCurrentStatus() == AgentService.AGENT_STATUS_BreakDown) {
				continue;
			} else if (agent.getCurrentStatus() == AgentService.AGENT_STATUS_Idel) {
				maxLoad += agent.getMaxLoad();
				availableLoad += agent.getRemainLoad();
			} else if (agent.getCurrentStatus() == AgentService.AGENT_STATUS_InRun) {
				maxLoad += agent.getMaxLoad();
			}
		}
		this.setMaxAvailableLoad((long) maxLoad);
		this.setCurrentAvailableLoad(availableLoad);
	}

	private void heartBeatsAndUpdateHAPool() {
		this.pool.clear();
		this.setCurrentAvailableLoad(0);
		this.setMaxAvailableLoad((long) 0);
		for (Agent agent : this.agentService.loadAgentPoolFromDB()) {
			System.out.println(System.currentTimeMillis());
			checkHeartBeat(agent);
			System.out.println(System.currentTimeMillis());
		}
	}

	public void checkHeartBeat(Agent agent) {
		ServerStatusModel model = queryAgentStatus(agent);
		if (model == null) {
			doForBreakDown(agent);
		} else {
			doForHealth(model, agent);
		}
		this.getPool().put(agent.getHostName(), agent);
	}

	public ServerStatusModel queryAgentStatus(Agent agent) {
		return this.getAgentMessenger().askLiving(agent.getHostName(),
				agent.getPort());
	}

	private void doForBreakDown(Agent agent) {
		updateAgentStatus(AgentService.AGENT_STATUS_BreakDown, agent);
		List<UUID> runIDs = queryUnfinishedTest(null, agent.getHostName());
		if (runIDs == null) {
			return;
		}
		this.getAgentRunIdShouldBeSubstitute().addAll(runIDs);
	}

	private void updateAgentStatus(int status, Agent agent) {
		agent.setCurrentStatus(status);
		// TODO: update this all together to db
		this.agentService.updateAgentStatus(status, agent.getHostName());
	}

	private void doForHealth(ServerStatusModel newModel, Agent agent) {
		List<UUID> agentUnfinishedRunIds = newModel.getRunningTests();
		if (agentUnfinishedRunIds == null || agentUnfinishedRunIds.size() == 0) {
			doForInIdle(agent);
			return;
		}
		doForInRunning(agent, newModel);
	}

	private void doForInRunning(Agent agent, ServerStatusModel newModel) {
		updateAgentStatus(AgentService.AGENT_STATUS_InRun, agent);
		this.agentStatusOfPreviousBeatMap.put(agent.getHostName(), newModel);
	}

	private void doForInIdle(Agent agent) {
		updateAgentStatus(AgentService.AGENT_STATUS_Idel, agent);
		this.agentService.resetAgent(agent);
	}

	private List<UUID> queryUnfinishedTest(ServerStatusModel newModel,
			String hostName) {
		if (newModel == null) {
			ServerStatusModel model = this.agentStatusOfPreviousBeatMap
					.get(hostName);
			return model == null ? null : model.getRunningTests();
		}
		return newModel.getRunningTests();
	}

	public void doForTestPlanComplete(
			final Collection<? extends RunningScriptInterface> runningScripts) {

		for (RunningScriptInterface runningScript : runningScripts) {
			for (RunningAgentInterface runningAgent : runningScript
					.getRunningAgents()) {
				this.getAgentRunBlotters().remove(runningAgent.getAgentRunId());
				logger.info("see the agentBlotter after run : "
						+ this.getAgentRunBlotters().get(
								runningAgent.getAgentRunId()));
			}
		}
	}
}
