package org.bench4q.master.transaction.script;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.RunningScriptInterface;
import org.bench4q.master.domain.RunningAgentInterface.AbstractRunningAgent;
import org.bench4q.master.domain.entity.Agent;
import org.bench4q.master.domain.service.AgentService;
import org.bench4q.master.exception.ExceptionLog;
import org.bench4q.master.helper.ApplicationContextHelper;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.master.testplan.highavailable.AgentRunBlotter;
import org.bench4q.master.testplan.highavailable.HighAvailablePool;
import org.bench4q.master.transaction.Transaction;
import org.bench4q.master.transaction.agent.AgentRunException;
import org.bench4q.master.transaction.agent.AgentTransaction;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.agent.RunScenarioResultModel;
import org.bench4q.share.models.agent.StopTestModel;

public abstract class ScriptLoadBase implements Transaction {
	private RunningScriptInterface runningScript;
	private UUID testPlanRunID;
	private List<RunningAgentInterface> agentListThisTime;
	private AgentMessenger runningAgentService;
	private HighAvailablePool highAvailableAgentPool;
	private static Logger logger = Logger.getLogger(ScriptLoadCommand.class);

	protected RunningScriptInterface getRunningScript() {
		return runningScript;
	}

	protected void setRunningScript(RunningScriptInterface runningScript) {
		this.runningScript = runningScript;
	}

	private UUID getTestPlanRunID() {
		return testPlanRunID;
	}

	protected void setTestPlanRunID(UUID testPlanRunID) {
		this.testPlanRunID = testPlanRunID;
	}

	private List<RunningAgentInterface> getAgentListThisTime() {
		return agentListThisTime;
	}

	private void setAgentListThisTime(
			List<RunningAgentInterface> agentListThisTime) {
		this.agentListThisTime = agentListThisTime;
	}

	private AgentMessenger getRunningAgentService() {
		return runningAgentService;
	}

	private void setRunningAgentService(AgentMessenger runningAgentService) {
		this.runningAgentService = runningAgentService;
	}

	private HighAvailablePool getHighAvailableAgentPool() {
		return highAvailableAgentPool;
	}

	private void setHighAvailableAgentPool(
			HighAvailablePool highAvailableAgentPool) {
		this.highAvailableAgentPool = highAvailableAgentPool;
	}

	public static Logger getLogger() {
		return logger;
	}

	public static void setLogger(Logger logger) {
		ScriptLoadBase.logger = logger;
	}

	public ScriptLoadBase(RunningScriptInterface runningScript, UUID testPlanID) {
		initialize(runningScript, testPlanID);
	}

	private void initialize(RunningScriptInterface runningScript,
			UUID testPlanID) {
		this.setRunningScript(runningScript);
		this.setTestPlanRunID(testPlanID);
		this.setRunningAgentService(ApplicationContextHelper.getContext()
				.getBean(AgentMessenger.class));
		this.setHighAvailableAgentPool(ApplicationContextHelper.getContext()
				.getBean(HighAvailablePool.class));
		this.setAgentListThisTime(new ArrayList<RunningAgentInterface>());
	}

	public List<RunningAgentInterface> execute() {
		logger.info(this.getRunningScript().getScriptId());
		RunScenarioModel runScenarioModel = this.getRunningScript()
				.getScenario();
		if (runScenarioModel == null) {
			logger.error("runScenarioModel is null!");
			return null;
		}
		try {
			this.runAgentsWithScenario(runScenarioModel, getRequireLoad(), this
					.getRunningScript().getScriptId(), this.getTestPlanRunID());

			if (this.getAgentListThisTime() == null) {
				logger.error("runAgentsWithScenario fails where scriptId = "
						+ this.getRunningScript().getScriptId());
				return null;
			}
			if (isNotValidScript(this.getRunningScript())) {
				logger.error("The running agents of runningScriptModel is null, whose scriptId is"
						+ this.getRunningScript().getScriptId());
				return null;
			}
			return this.getAgentListThisTime();
		} catch (Exception e) {
			logger.error(ExceptionLog.getStackTrace(e).toString()
					+ " whose scriptID is "
					+ this.getRunningScript().getScriptId()
					+ " and TestPlanID is "
					+ this.getTestPlanRunID().toString());
			return null;
		}
	}

	private boolean isNotValidScript(RunningScriptInterface input) {
		return input == null || input.getRunningAgents() == null;
	}

	protected abstract int getRequireLoad();

	private void runAgentsWithScenario(RunScenarioModel runScenarioModel,
			int totalRequireLoad, int scriptId, UUID testPlanId)
			throws JAXBException, ScriptLoadDistributeException {
		RunScenarioResultModel runScenarioResultModel = new RunScenarioResultModel();
		int loadForRunCurrent;

		if (totalRequireLoad >= this.getHighAvailableAgentPool()
				.getCurrentAvailableLoad()) {
			logger.info("currentAvailableLoad not enough for substitute");
			return;
		}

		for (Agent agent : this.getHighAvailableAgentPool().getPool().values()) {
			if (allocationFinish(totalRequireLoad)) {
				break;
			}
			if (this.getHighAvailableAgentPool().queryAgentStatus(agent) == null
					|| inUse(agent)) {
				continue;
			}
			loadForRunCurrent = getMin(totalRequireLoad, agent.getRemainLoad());
			runScenarioModel.setPoolSize(loadForRunCurrent);
			AgentTransaction agentTransaction = new AgentTransaction(
					agent.getHostName(), agent.getPort(), runScenarioModel);
			try {
				runScenarioResultModel = (RunScenarioResultModel) agentTransaction
						.execute();
			} catch (AgentRunException e) {
				logger.info(ExceptionLog.getStackTrace(e));
				agentTransaction.rollBack();
				continue;
			}
			agent.setCurrentStatus(AgentService.AGENT_STATUS_InRun);
			agent.setRemainLoad(agent.getRemainLoad() - loadForRunCurrent);
			RunningAgentInterface runningAgent = AbstractRunningAgent
					.buildDegendRunningAgentDB(agent, loadForRunCurrent,
							scriptId, runScenarioResultModel.getRunId());
			this.getAgentListThisTime().add(runningAgent);
			updateAgentBlotterInHA(testPlanId, runningAgent);
			totalRequireLoad -= loadForRunCurrent;
		}
		if (!allocationFinish(totalRequireLoad)) {
			logger.error("allocationUnfinished, remain " + totalRequireLoad
					+ " unallocated");
			throw new ScriptLoadDistributeException();
		}
	}

	private boolean allocationFinish(int requireLoad) {
		return requireLoad <= 0;
	}

	private boolean inUse(Agent agent) {
		return agent.getRemainLoad() < agent.getMaxLoad();
	}

	private int getMin(int requireLoad, int remainLoadByStart) {
		return remainLoadByStart >= requireLoad ? requireLoad
				: remainLoadByStart;
	}

	private void updateAgentBlotterInHA(UUID testPlanId,
			RunningAgentInterface runningAgent) {
		this.getHighAvailableAgentPool()
				.getAgentRunBlotters()
				.put(runningAgent.getAgentRunId(),
						AgentRunBlotter.buildAgentRunBlotter(testPlanId,
								runningAgent));
	}

	public void rollBack() {
		stopAgentsRunningThisScript();
	}

	private void stopAgentsRunningThisScript() {
		if (this.getAgentListThisTime() == null) {
			return;
		}
		for (RunningAgentInterface runningAgent : this.getAgentListThisTime()) {
			// TODO: there's a problem
			AgentRunBlotter agentRunBlotter = this.getHighAvailableAgentPool()
					.getAgentRunBlotters().get(runningAgent.getAgentRunId());
			if (agentRunBlotter.getRunningAgent().isBreakDown()) {
			} else if (stopAgentSuccess(runningAgent)) {
			}
			this.getAgentListThisTime().remove(runningAgent);
		}
	}

	private boolean stopAgentSuccess(RunningAgentInterface runningAgent) {
		StopTestModel resultModel = this.getRunningAgentService().stop(
				runningAgent.getAgent(), runningAgent.getAgentRunId());
		return (resultModel != null) && resultModel.isSuccess();
	}
}
