package org.bench4q.master.test.domain;

import static org.junit.Assert.assertTrue;

import java.util.UUID;

import org.bench4q.master.domain.entity.Monitor;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.domain.testplan.TestMoniorResultSave;
import org.bench4q.master.domain.testplan.TestMonitorSampler;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.share.models.monitor.MonitorMain;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_TestMonitorResultSave extends TestBase_MakeUpTestPlan {

	private TestMoniorResultSave testMoniorResultSave;
	private UUID testPlanRunId;
	private TestMonitorSampler testMonitorSampler;
	private TestPlanFactory testPlanFactory;

	public TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	@Autowired
	public void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	public TestMoniorResultSave getTestMoniorResultSave() {
		return testMoniorResultSave;
	}

	@Autowired
	public void setTestMoniorResultSave(
			TestMoniorResultSave testMoniorResultSave) {
		this.testMoniorResultSave = testMoniorResultSave;
	}

	public TestMonitorSampler getTestMonitorSampler() {
		return testMonitorSampler;
	}

	@Autowired
	public void setTestMonitorSampler(TestMonitorSampler testMonitorSampler) {
		this.testMonitorSampler = testMonitorSampler;
	}

	@Before
	public void setUp() {
		prepareForTestPlanRunning();
		this.submitATestPlanWithOneScript();
		TestPlan testPlan = fetchTestPlan();
		this.testPlanRunId = UUID.fromString(testPlan.getTestPlanRunId());
		testPlan.run();
		this.getTestPlanRepository().updateEntity(testPlan);
	}

	@Test
	public void testDoSaveResult() {
		TestPlan testPlan = fetchTestPlan();
		Monitor monitor = testPlan.getMonitors().iterator().next();
		int beforeMonitorResultSize = monitor.getResults().size();
		MonitorMain monitorMain = monitor.getTestMonitorSampler()
				.getMonitorResult(monitor.getHostName(), monitor.getPort());
		Assert.assertNotNull(monitorMain);

		assertTrue(this.getTestMoniorResultSave().saveMonitorResult(
				this.testPlanRunId, monitorMain, monitorMain.getSamplingTime(),
				monitor));
		testPlan = fetchTestPlan();
		monitor = testPlan.getMonitors().iterator().next();
		Assert.assertEquals(beforeMonitorResultSize + 5, monitor.getResults()
				.size());

	}

	@After
	public void clean() {
		cleanUpForTestPlanRunning();
	}

	private TestPlan fetchTestPlan() {
		return this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
	}

}
