package org.bench4q.master.test.domain;

import static org.junit.Assert.*;

import java.util.List;
import java.util.Set;

import org.bench4q.master.domain.entity.RunningAgentDB;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.bench4q.share.models.master.ValueTimeModel;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_TestPlan extends TestBase_MakeUpTestPlan {
	private TestPlanFactory testPlanFactory;

	private TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	@Autowired
	private void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	@Before
	public void prepare() {
		prepareForTestPlanRunning();
		submitATestPlanWithOneScript();
	}

	@After
	public void cleanUp() {
		cleanUpForTestPlanRunning();
	}

	@Test
	public void testDoAfter() {
		
	}
	
	@Test
	public void testRun() throws InterruptedException {
		assertTrue(this.getHaPool().getCurrentAvailableLoad() >= 500);
		TestPlan testPlanFromRepo = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		TestPlan testPlanInDomain = this.getTestPlanFactory().convertToDomain(
				testPlanFromRepo);
		TestPlanStatus returnStatus = testPlanInDomain.run();
		assertEquals(TestPlanStatus.InRunning, returnStatus);
		
		
		assertTrue(this.getTestPlanRepository().updateEntity(testPlanInDomain));
		TestPlan testPlanAfterRun = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		testPlanAfterRun = this.getTestPlanFactory().convertToDomain(
				testPlanAfterRun);
		Set<TestPlanScript> testPlanScripts = testPlanAfterRun
				.getTestPlanScripts();
		assertNotNull(testPlanScripts);
		assertEquals(1, testPlanScripts.size());
		TestPlanScript testPlanScript = testPlanAfterRun
				.extracSpecifiedScript(getScriptId());
		assertNotNull(testPlanScript);
		assertEquals(EACH_SCRIPT_LOAD_SMALLSCALE,
				testPlanScript.getRequireLoad());
		assertNotNull(testPlanScript.getRunningAgentsDB());
		assertEquals(1, testPlanScript.getRunningAgentsDB().size());
		RunningAgentDB runningAgentDB = testPlanScript
				.extractSpecificRunningAgentDB(Test_AGENT_HOSTNAME);
		assertNotNull(runningAgentDB);
		assertEquals(EACH_SCRIPT_LOAD_SMALLSCALE, runningAgentDB.getLoadInUse());
		Thread.sleep(10000);
		List<ValueTimeModel> valueTimeModels = this
				.getTestPlanScriptResultService().loadScriptBriefSpecificField(
						getTestPlanRunIdUuid(), getScriptId(),
						"averageResponseTime");
		assertNotNull(valueTimeModels);
		assertTrue(valueTimeModels.size() > 0);
		while (!TestPlanStatus.valueOf(
				this.getTestPlanRepository()
						.getTestPlanBy(getTestPlanRunIdUuid())
						.getCurrentStatus()).equals(TestPlanStatus.Complete)) {
			System.out.println("sleep for not complete");
			Thread.sleep(10000);
		}
	}
}
