package org.bench4q.master.test.repository;

import static org.junit.Assert.*;

import org.bench4q.master.TestPlunginHelper;
import org.bench4q.master.domain.entity.plugin.MethodParam;
import org.bench4q.master.domain.entity.plugin.Plugin;
import org.bench4q.master.domain.factory.PluginFactory;
import org.bench4q.master.domain.repository.PluginRepository;
import org.bench4q.master.exception.Bench4QException;
import org.bench4q.share.models.master.plugin.PluginModel;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_PluginRepository {
	private PluginRepository pluginRepository;
	private String pluginName;
	private PluginFactory pluginFactory;

	public PluginRepository getPluginRepository() {
		return pluginRepository;
	}

	@Autowired
	public void setPluginRepository(PluginRepository pluginRepository) {
		this.pluginRepository = pluginRepository;
	}

	public PluginFactory getPluginFactory() {
		return pluginFactory;
	}

	@Autowired
	public void setPluginFactory(PluginFactory pluginFactory) {
		this.pluginFactory = pluginFactory;
	}

	@Before
	public void setUp() throws Bench4QException {
		pluginName = addPlugin();
	}

	@Test
	public void testGetPlugin() {
		assertNotNull(this.getPluginRepository().getPlugin(pluginName));
	}

	@Test
	public void TestLoadMethodParam() {
		Plugin plugin = this.getPluginRepository().getPlugin(pluginName);
		MethodParam methodParam = plugin.getPluginMethods().iterator().next()
				.getMethodParams().iterator().next();

		System.out.println(methodParam.getParamType().getClass());
	}

	@Test
	public void testGetMethodList() throws Bench4QException {
		Plugin plugin = this.getPluginRepository().getPlugin(pluginName);
		assertNotNull(plugin);
		assertEquals(1, plugin.getPluginMethods().size());
	}

	@Test
	public void testLoadPlugin() throws Bench4QException {

		int countBeforeInsert = this.getPluginRepository().loadPlugins().size();
		String insertPlugin = addPlugin();
		assertEquals(countBeforeInsert + 1, this.getPluginRepository()
				.loadPlugins().size());
		this.getPluginRepository().detach(insertPlugin);

	}

	@After
	public void clear() throws Bench4QException {
		/* this.getPluginRepository().detach(pluginName); */
	}

	private String addPlugin() throws Bench4QException {
		PluginModel pluginModel = TestPlunginHelper.createOnePlugin();
		this.getPluginRepository().attatch(
				this.getPluginFactory().createPluginEntity(pluginModel));
		return pluginModel.getName();

	}
}
