package org.bench4q.master.test.service;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

import javax.xml.bind.JAXBException;

import org.apache.commons.io.FileUtils;
import org.bench4q.master.domain.service.PluginService;
import org.bench4q.master.exception.Bench4QException;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.master.plugin.PluginModel;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import TestHelper.Test_PlunginHelper;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_PluginService {

	private PluginService pluginService;

	public PluginService getPluginService() {
		return pluginService;
	}

	private String nameForPlugin;

	@Autowired
	public void setPluginService(PluginService pluginService) {
		this.pluginService = pluginService;
	}

	@Before
	public void setUp() throws Bench4QException {
		nameForPlugin = addPlugin();
	}

	@Test
	public void testPluginModel() throws JAXBException, IOException {
		File file = new File(
				buildSavePath()
						+ System.getProperty("file.separator")
						+ new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss")
								.format(new Date()) + "plugin.xml");
		FileUtils.writeStringToFile(
				file,
				MarshalHelper.marshal(PluginModel.class,
						Test_PlunginHelper.createOnePlugin()));

	}

	@Test
	public void testGetMethodList() throws Bench4QException {
		assertTrue(this.getPluginService().getPluginNameList().size() > 0);
		int methodCount = this.getPluginService()
				.getMethodNameInPlugin(nameForPlugin).size();
		assertEquals(1, methodCount);
	}

	@Test
	public void testLoadPluginList() throws Bench4QException {
		int pluginCountBefore = this.getPluginService().getPluginNameList()
				.size();
		String pluginName = addPlugin();
		System.out.println(this.getPluginService().getPluginNameList().size()
				- pluginCountBefore);
		Assert.assertTrue(this.getPluginService().getPluginNameList().size() == (pluginCountBefore + 1));

		Assert.assertTrue(this.getPluginService().deletePlugin(pluginName));
	}

	@Test
	public void getMethodParam() throws Bench4QException {
		String methodName = this.getPluginService()
				.getMethodNameInPlugin(nameForPlugin).get(0);
		assertTrue(this.getPluginService().getMethodParamModelsInPlugin(nameForPlugin,
				methodName).size()==2);
	}

	@Test
	public void testDeletePlugin() throws Bench4QException {
		String pluginName = addPlugin();
		Assert.assertTrue(this.getPluginService().deletePlugin(pluginName));

	}

	private String addPlugin() throws Bench4QException {
		PluginModel plugin = Test_PlunginHelper.createOnePlugin();
		plugin.setName("test" + UUID.randomUUID());
		Assert.assertTrue(this.getPluginService().addPlugin(plugin));
		return plugin.getName();
	}

	@After
	public void clear() throws Bench4QException {
		this.getPluginService().deletePlugin(nameForPlugin);
	}

	private String buildSavePath() {
		String dirString = "GUI" + System.getProperty("file.separator")
				+ System.getProperty("file.separator");
		File dirFile = new File(dirString);
		if (!dirFile.exists()) {
			dirFile.mkdirs();
		}
		return dirString;
	}

}
