package org.bench4q.master.test.service;

import java.util.Date;

import org.bench4q.master.Main;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.entity.User;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.bench4q.share.helper.TestHelper;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import static org.junit.Assert.*;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_TestPlanEngine extends TestBase_MakeUpTestPlan {

	@Before
	public void prepare() {

	}

	@After
	public void cleanUp() {
		cleanUpForTestPlanRunning();
	}

	@Test
	public void testCommintComplete() throws Exception {
		TestPlanStatus status = TestPlanStatus.Complete;
		testForStatus(status);
	}

	@Test
	public void testInRunning() throws Exception {
		testForStatus(TestPlanStatus.InRunning);
	}

	@Test
	public void testRunTestPlanRightly() throws InterruptedException {
		assertNotNull(this.getAgentMessenger().askLiving(Test_AGENT_HOSTNAME,
				TEST_PORT));
		User user = this.getUserRepository().getUser("admin");
		int scriptId = getUserFirstScript(user);
		this.getHaPool().timerTask();
		Date dateBeforeRun = new Date();
		this.setTestPlanRunIdUuid(this.getTestPlanEngine().runWith(
				createATestPlanWithOneScript(scriptId), user));
		assertNotNull(getTestPlanRunIdUuid());
		Thread.sleep(10000);
		TestPlan testPlan = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		assertEquals(TestPlanStatus.InRunning,
				TestPlanStatus.valueOf(testPlan.getCurrentStatus()));
		assertTrue(testPlan.getLastRunningTime().after(dateBeforeRun));
		assertEquals(TestPlanStatus.InRunning,
				TestPlanStatus.valueOf(getTestPlanRepository()
						.getRunningTestPlanBy(getTestPlanRunIdUuid())
						.getCurrentStatus()));

	}

	@Test
	public void testRunWithWronglyWhenNoEnoughLoadInPool()
			throws InterruptedException {
		System.out.println("This need each agent not in running");
		User user = this.getUserRepository().getUser("admin");
		int scriptId = getUserFirstScript(user);
		this.getHaPool().getPool().clear();
		this.setTestPlanRunIdUuid(this.getTestPlanEngine().runWith(
				createATestPlanWithOneScript(scriptId), user));
		Thread.sleep(10000);
		TestPlan testPlan = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		assertNotEquals(TestPlanStatus.InRunning,
				TestPlanStatus.valueOf(testPlan.getCurrentStatus()));
		assertNull(this.getTestPlanRepository().getRunningTestPlanBy(
				getTestPlanRunIdUuid()));
		deleteTestPlan();
	}

	@Test
	public void testPickATestPlan() {
		this.getHaPool().timerTask();
		Main.getPortToServe();
		assertNotEquals(0, this.getHaPool().getCurrentAvailableLoad());
		System.out.println(this.getHaPool().getCurrentAvailableLoad());
		TestPlan testPlanPicked = this.getTestPlanEngine().pickATestPlan();
		assertNotNull(testPlanPicked);
		assertTrue(testPlanPicked.getRequiredLoad() <= this.getHaPool()
				.getCurrentAvailableLoad());
		assertTrue(testPlanPicked.getFailTimes() <= Main.MAX_FAIL_TIMES);
		assertTrue(System.currentTimeMillis()
				- testPlanPicked.getLastRunningTime().getTime() > Main.MIN_EXECUTE_INTERVAL_IN_SECONDS * 1000);
		assertNotEquals(TestPlanStatus.Complete,
				TestPlanStatus.valueOf(testPlanPicked.getCurrentStatus()));
	}

	@Test
	public void testPickAndRunByCycle() throws InterruptedException {
		this.getHaPool().timerTask();
		Main.getPortToServe();
		this.submitATestPlanWithOneScript();
		Thread.sleep(71000);
		assertEquals(
				TestPlanStatus.InRunning,
				TestPlanStatus.valueOf(this.getTestPlanRepository()
						.getTestPlanBy(getTestPlanRunIdUuid())
						.getCurrentStatus()));
		deleteTestPlan();
	}

	private void testForStatus(TestPlanStatus status) throws Exception {
		this.submitATestPlanWithOneScript();
		TestPlan testPlan = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		testPlan.run();
		this.getTestPlanRepository().attachRunningTestPlan(testPlan);
		TestHelper.invokePrivate(this.getTestPlanEngine(),
				"commit" + status.name(), new Class[] { TestPlan.class },
				new Object[] { this.getTestPlanRepository()
						.getRunningTestPlanBy(getTestPlanRunIdUuid()) });
		assertEquals(status,
				TestPlanStatus.valueOf(testPlan.getCurrentStatus()));
		assertEquals(
				status,
				TestPlanStatus.valueOf(this.getTestPlanRepository()
						.getRunningTestPlanBy(getTestPlanRunIdUuid())
						.getCurrentStatus()));
		deleteTestPlan();
	}
}
