package org.bench4q.master.test.service;

import static org.junit.Assert.*;

import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.bench4q.master.domain.entity.User;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.share.helper.TestHelper;
import org.bench4q.share.models.master.TestPlanModel;
import org.bench4q.share.models.master.ValueTimeModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptPageBriefModel;
import org.bench4q.share.models.master.statistics.ScriptPagesBriefModel;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_TestPlanScriptResultService extends TestBase_MakeUpTestPlan {
	@Test
	public void testGetSpecficFieldValue() throws Exception {
		long averageResponseTime = (Long) TestHelper.invokePrivate(
				this.getTestPlanScriptResultService(), "getSpecificFieldValue",
				new Class[] { ScriptBriefResultModel.class, String.class },
				new Object[] { buildScriptBriefResultModel(0),
						"averageResponseTime" });
		assertEquals(0, averageResponseTime);
	}

	@Test
	public void testLoadAvgFromScriptBriefResultWithWrongTestPlanID()
			throws NoSuchFieldException, SecurityException {
		List<ValueTimeModel> results = this
				.getTestPlanScriptResultService()
				.loadScriptBriefSpecificField(
						UUID.fromString("af67ccd7-ca7b-4dbf-9544-8ffe81b3af2d"),
						61,
						ScriptBriefResultModel.class.getDeclaredField(
								"averageResponseTime").getName());
		assertEquals(0, results.size());
	}

	@Test
	public void testLoadAvgFromScriptBriefResult() throws NoSuchFieldException,
			SecurityException, InterruptedException {
		Thread.sleep(5000);
		List<ValueTimeModel> results = this.getTestPlanScriptResultService()
				.loadScriptBriefSpecificField(
						this.getTestPlanRunIdUuid(),
						this.getScriptId(),
						ScriptBriefResultModel.class.getDeclaredField(
								"averageResponseTime").getName());
		System.out.println(results.size());
		assertTrue(results.size() > 0);
		assertTrue(results.get(1).getValue() % 100 == 0);
	}

	@Test
	public void testLoadScriptBriefWithWrongTestPlanId() {
		List<ScriptBriefResultModel> results = this
				.getTestPlanScriptResultService()
				.loadScriptBriefWithDuation(
						UUID.fromString("fdfe0e55-c65f-4829-ae7c-b1d3a90f62d5"),
						this.getScriptId(), 0);
		ScriptPagesBriefModel scriptPagesBriefModel = this
				.getTestPlanScriptResultService()
				.getScriptPagesBrief(
						UUID.fromString("fdfe0e55-c65f-4829-ae7c-b1d3a90f62d5"),
						this.getScriptId());
		ScriptBehaviorsBriefModel behavirosResults = this
				.getTestPlanScriptResultService()
				.getScriptBehaviorsBrief(
						UUID.fromString("fdfe0e55-c65f-4829-ae7c-b1d3a90f62d5"),
						this.getScriptId());
		assertEquals(0, results.size());
		assertNotNull(scriptPagesBriefModel);
		assertEquals(0, scriptPagesBriefModel.getScriptPageBriefModels().size());
		assertNotNull(behavirosResults);
		assertNotNull(behavirosResults.isFinished());
	}

	@Test
	public void testLoadScriptBriefWithDuation() throws InterruptedException {
		Thread.sleep(5000);
		List<ScriptBriefResultModel> results = this
				.getTestPlanScriptResultService().loadScriptBriefWithDuation(
						this.getTestPlanRunIdUuid(), this.getScriptId(), 0);
		ScriptPagesBriefModel scriptPagesBriefModel = this
				.getTestPlanScriptResultService().getScriptPagesBrief(
						this.getTestPlanRunIdUuid(), this.getScriptId());
		ScriptBehaviorsBriefModel behavirosResults = this
				.getTestPlanScriptResultService().getScriptBehaviorsBrief(
						this.getTestPlanRunIdUuid(), this.getScriptId());
		assertEquals(3, results.size());
		assertNotNull(scriptPagesBriefModel);
		assertTrue(scriptPagesBriefModel.getScriptPageBriefModels().size() > 0);
		assertTrue(scriptPagesBriefModel.getScriptPageBriefModels().get(0)
				.getMaxResponseTimeFromBegin() >= 200);
		assertNotNull(behavirosResults);
		assertNotNull(behavirosResults.isFinished());
	}

	@Before
	public void makeUpTestPlanAndTestPlanScriptAndTestPlanScriptResult() {
		UUID testPlanRunId = UUID.randomUUID();
		User user = this.getUserRepository().getUser("admin");
		this.setScriptId(this.getUserFirstScript(user));
		TestPlanModel model = createATestPlanWithOneScript(this.getScriptId());
		this.getTestPlanService().submitTestPlan(model, user, testPlanRunId);
		this.setTestPlanRunIdUuid(testPlanRunId);
		for (int i = 0; i < 3; i++) {
			this.getTestScriptResultSave().doSaveResult(testPlanRunId,
					this.getScriptId(), buildScriptBriefResultModel(i),
					new Date());

			this.getTestScriptResultSave().doSaveResult(testPlanRunId,
					this.getScriptId(), new ScriptBehaviorsBriefModel(),
					new Date());
			this.getTestScriptResultSave().doSaveResult(testPlanRunId,
					this.getScriptId(), makeUpScriptPagesBriefModel(i),
					new Date());
		}

	}

	private ScriptPagesBriefModel makeUpScriptPagesBriefModel(int i) {
		ScriptPagesBriefModel scriptPagesBriefModel = new ScriptPagesBriefModel();
		ScriptPageBriefModel scriptPageBriefModel = new ScriptPageBriefModel();
		scriptPageBriefModel.setMaxResponseTimeFromBegin(200 + 10 * i);
		scriptPageBriefModel.setMinResponseTimeFromBegin(200 - 10 * i);
		scriptPagesBriefModel.getScriptPageBriefModels().add(
				scriptPageBriefModel);
		return scriptPagesBriefModel;
	}

	@After
	public void cleanUp() {
		this.getTestPlanService().removeTestPlanInDB(
				this.getTestPlanRepository()
						.getTestPlanBy(this.getTestPlanRunIdUuid()).getId());
	}
}
