package org.bench4q.master.test.testplan;

import static org.junit.Assert.*;

import java.util.Set;

import org.bench4q.master.domain.entity.RunningAgentDB;
import org.bench4q.master.domain.entity.TestPlan;
import org.bench4q.master.domain.factory.TestPlanFactory;
import org.bench4q.master.domain.testplan.LoadDistribute;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.share.enums.master.TestPlanStatus;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_LoadDistribute extends TestBase_MakeUpTestPlan {
	private LoadDistribute loadDistribute;
	private TestPlanFactory testPlanFactory;

	private LoadDistribute getLoadDistribute() {
		return loadDistribute;
	}

	@Autowired
	private void setLoadDistribute(LoadDistribute loadDistribute) {
		this.loadDistribute = loadDistribute;
	}

	private TestPlanFactory getTestPlanFactory() {
		return testPlanFactory;
	}

	@Autowired
	private void setTestPlanFactory(TestPlanFactory testPlanFactory) {
		this.testPlanFactory = testPlanFactory;
	}

	@Before
	public void prepare() {
		prepareForTestPlanRunning();
		submitATestPlanWithOneScript();
	}

	@After
	public void cleanUp() {
		for (RunningAgentDB runningAgentDB : this.getTestPlanRepository()
				.getTestPlanBy(getTestPlanRunIdUuid())
				.extracSpecifiedScript(getScriptId()).getRunningAgentsDB()) {
			this.getAgentMessenger().stop(runningAgentDB.getAgent(),
					runningAgentDB.getAgentRunId());
		}
		this.getTestPlanRepository().detach(
				getTestPlanRepository().getTestPlanBy(getTestPlanRunIdUuid())
						.getId());
	}

	/**
	 * This need a agent could be found
	 */
	@Test
	public void testGenerateLoadForTestPlan() {
		TestPlan testPlanInDomain = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());

		testPlanInDomain = this.getTestPlanFactory().convertToDomain(
				testPlanInDomain);
		assertEquals(TestPlanStatus.InRunning, this.getLoadDistribute()
				.generateLoadForTestPlan(testPlanInDomain));
		assertEquals(EACH_SCRIPT_LOAD_SMALLSCALE,
				testPlanInDomain.getRequiredLoad());
		testPlanInDomain = this.getTestPlanRepository().getTestPlanBy(
				getTestPlanRunIdUuid());
		Set<RunningAgentDB> runningAgentsAfterRefetch = testPlanInDomain
				.extracSpecifiedScript(getScriptId()).getRunningAgentsDB();
		assertTrue(runningAgentsAfterRefetch.size() > 0);
		for (RunningAgentDB runningAgentDB : runningAgentsAfterRefetch) {
			assertNotNull(runningAgentDB);
		}
	}

}
