package org.bench4q.master.test.transaction;

import static org.junit.Assert.*;

import java.util.List;
import java.util.UUID;

import org.bench4q.master.domain.RunningAgentInterface;
import org.bench4q.master.domain.entity.Agent;
import org.bench4q.master.domain.entity.TestPlanScript;
import org.bench4q.master.test.TestBase_MakeUpTestPlan;
import org.bench4q.master.transaction.script.ScriptLoadCommand;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath:service-test-context.xml" })
public class Test_ScriptLoadCommand extends TestBase_MakeUpTestPlan {
	private static final String Test_AGENT_HOSTNAME = "127.0.0.1";
	private UUID agentRunId;

	private UUID getAgentRunId() {
		return agentRunId;
	}

	private void setAgentRunId(UUID agentRunId) {
		this.agentRunId = agentRunId;
	}

	@Before
	public void prepare() {
	}

	@After
	public void cleanUp() {
		Agent agent = this.getAgentRepository().getAgentBy(Test_AGENT_HOSTNAME);
		this.getAgentMessenger().stop(agent, this.getAgentRunId());
		agent.setRemainLoad(agent.getMaxLoad());
		this.getAgentRepository().update(agent);
	}

	/**
	 * This need a agent in running
	 */

	@Test
	public void testExecuteWithTestPlanScript() {
		submitATestPlanWithOneScript();
		this.getHaPool().timerTask();
		TestPlanScript testPlanScript = this.getTestPlanRepository()
				.getTestPlanBy(getTestPlanRunIdUuid())
				.extracSpecifiedScript(this.getScriptId());
		ScriptLoadCommand scriptLoadCommand = new ScriptLoadCommand(
				testPlanScript, testPlanScript.getTestPlanID());
		List<? extends RunningAgentInterface> runningAgents = null;
		synchronized (this.getHaPool().getPool()) {
			runningAgents = scriptLoadCommand.execute();
		}

		assertNotNull(runningAgents);
		assertNotNull(runningAgents.get(0));
		this.setAgentRunId(runningAgents.get(0).getAgentRunId());
		assertEquals(EACH_SCRIPT_LOAD_SMALLSCALE, runningAgents.get(0)
				.getLoadInUse());
	}
}
