package org.bench4q.web.TestPlan;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;

import org.bench4q.share.models.master.MonitorModel;
import org.bench4q.share.models.master.RunningScriptModel;
import org.bench4q.share.models.master.TestPlanModel;
import org.bench4q.share.models.master.TestPlanResultModel;
import org.bench4q.share.models.master.TestScriptConfig;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.WebScriptModel;
import org.bench4q.web.model.TestPlanRequestModel;
import org.bench4q.web.model.TestPlanTaskModel;
import org.bench4q.web.service.BaseService;
import org.springframework.stereotype.Component;
import org.springframework.ui.ModelMap;

@Component
public class RunTestPlan extends BaseService {
	public void runTestPlan(String accessToken,
			TestPlanRequestModel testPlanRequestModel, ModelMap model)
			throws CustomGenericException {
		String url = "testPlan" + "/runTestPlanWithTestPlanModel";
		String caller = "runTestPlan";
		try {
			TestPlanResultModel testPlanResultModel = (TestPlanResultModel) communicateWithMaster
					.getRespnseObjectByPostXml(accessToken, url,
							TestPlanResultModel.class, ObjectXmlExchange.toXml(
									TestPlanModel.class,
									createTestPlan(testPlanRequestModel,
											monitorPort)), caller);
			TestPlanTaskModel testPlanTaskModel = this
					.extractTestPlanTaskModel(testPlanRequestModel,
							testPlanResultModel);
			addToTestPlanTaskList(testPlanTaskModel, model);
		} catch (JAXBException e) {
			throw new CustomGenericException("0", e.getMessage(), caller);
		}

	}

	private TestPlanModel createTestPlan(TestPlanRequestModel testPlan,
			int port) {
		TestPlanModel testPlanModel = new TestPlanModel();
		if (testPlan.getScriptList() != null
				&& testPlan.getScriptList().size() > 0) {
			List<RunningScriptModel> scriptList = createRunningScriptModel(testPlan
					.getScriptList());
			testPlanModel.setRunningScriptModels(scriptList);
		}
		if (testPlan.getIpList() != null && testPlan.getIpList().size() > 0) {
			List<MonitorModel> ipList = createMonitorModel(
					testPlan.getIpList(), port);
			testPlanModel.setMonitorModels(ipList);
		} else
			testPlanModel.setMonitorModels(new ArrayList<MonitorModel>());
		return testPlanModel;
	}

	private List<RunningScriptModel> createRunningScriptModel(
			List<WebScriptModel> scriptParam) {

		List<RunningScriptModel> scriptList = new ArrayList<RunningScriptModel>();

		for (WebScriptModel scriptModel : scriptParam) {
			RunningScriptModel runnningScriptModel = new RunningScriptModel();
			runnningScriptModel.setScriptId(scriptModel.getId());
			runnningScriptModel.setRequireLoad(scriptModel.getLoad());
			TestScriptConfig testScriptConfig = new TestScriptConfig();
			testScriptConfig.setCoolDown(scriptModel.getCooldown());
			testScriptConfig.setExecuteRange(scriptModel.getExecuteRange());
			testScriptConfig.setWarmUp(scriptModel.getWarmup());
			runnningScriptModel.setConfig(testScriptConfig);
			scriptList.add(runnningScriptModel);
		}
		return scriptList;
	}

	private List<MonitorModel> createMonitorModel(List<String> ipParam, int port) {
		List<MonitorModel> ipList = new ArrayList<MonitorModel>();
		MonitorModel monitorModel = new MonitorModel();
		for (String ip : ipParam) {
			monitorModel.setHostName(ip);
			monitorModel.setPort(port);
			ipList.add(monitorModel);
		}
		return ipList;
	}

	private TestPlanTaskModel extractTestPlanTaskModel(
			TestPlanRequestModel testPlanRequestModel,
			TestPlanResultModel testPlanResultModel) {
		TestPlanTaskModel testPlanTaskModel;
		if (testPlanResultModel.getTestPlanId() != null)
			testPlanTaskModel = new TestPlanTaskModel(
					testPlanRequestModel.getTestPlanName(), testPlanResultModel
							.getTestPlanId().toString(), testPlanResultModel
							.getCurrentStatus().toString());
		else
			testPlanTaskModel = new TestPlanTaskModel(
					testPlanRequestModel.getTestPlanName(), "-1",
					testPlanResultModel.getCurrentStatus().toString());
		return testPlanTaskModel;
	}

	@SuppressWarnings("unchecked")
	private void addToTestPlanTaskList(TestPlanTaskModel testPlanTaskModel,
			ModelMap model) {
		if (model.get("testPlanTaskList") == null) {
			List<TestPlanTaskModel> testPlanTaskModels = new ArrayList<TestPlanTaskModel>();
			testPlanTaskModels.add(testPlanTaskModel);
			model.put("testPlanTaskList", testPlanTaskModels);
			System.out.println("add the test task!");
			System.out.println(model.get("testPlanTaskList"));
		} else {
			((List<TestPlanTaskModel>) model.get("testPlanTaskList"))
					.add(testPlanTaskModel);
		}
	}

}
