package org.bench4q.web.api;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.bench4q.share.communication.HttpRequester;
import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.models.master.AgentModel;
import org.bench4q.share.models.master.AgentResponseModel;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.AgentListModel;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes("accessToken")
public class AgentActionController extends BaseControllerService {
	private final String baseUrl =this.masterIp+ "agentManage";
	public static int IN_IDLE = 1;

	private String getBaseUrl() {
		return baseUrl;
	}

	@RequestMapping("addAgentToPool")
	@ResponseBody
	// change to true or false
	public Boolean addAgenttoPool(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String hostName, @RequestParam String maxLoad,
			@RequestParam String port) {
		if (!validateInput(hostName))
			return false;
		if (!validateInput(maxLoad))
			return false;
		if (!validateInput(port))
			return false;
		String content;
		HttpResponse httpResponse;
		try {
			content = ObjectXmlExchange.toXml(AgentModel.class,
					buildAgent(hostName, maxLoad, port, IN_IDLE));
			System.out.println(content);
			httpResponse = this.getHttpRequester().sendPostXml(
					this.getBaseUrl() + "/addAgentToPool", content,
					makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println(httpResponseNull);
				return false;
			}
			System.out.println(httpResponse.getContent());
			AgentResponseModel agentResponseModel = extractAgentResponseModel(httpResponse
					.getContent());
			if (agentResponseModel.isSuccess()) {
				System.out.println(SUCCESS);

				return true;
			} else {
				System.out.println(FAIL);
				System.err.println(agentResponseModel.getFailCauseString());
				return false;
			}
		} catch (JAXBException e) {
			System.err.println(FAIL);
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

	}

	private AgentModel buildAgent(String hostName, String maxLoad, String port,
			int currentStatus) {
		AgentModel agent = new AgentModel();
		agent.setHostName(hostName);
		agent.setPort(Integer.parseInt(port));
		agent.setCurrentStatus(currentStatus);
		agent.setMaxLoad(Integer.parseInt(maxLoad));
		agent.setRemainLoad(Integer.parseInt(maxLoad));
		return agent;
	}

	@RequestMapping("removeAgentFromPool")
	@ResponseBody
	public Boolean removeAgentfromPool(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String hostName, @RequestParam String id) {
		System.out.println("enter remove agent");
		if (!validateInput(hostName))
			return false;
		if (!validateInput(id))
			return false;
		Map<String, String> params = this.makeParamsMap("agentId", id);
		params.put("hostName", hostName);
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendGet(
					this.getBaseUrl() + "/removeAgentFromPool", params,
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println(httpResponseNull);
				return false;
			}
			System.out.println(httpResponse.getContent());
			AgentResponseModel agentResponseModel = extractAgentResponseModel(httpResponse
					.getContent());
			if (agentResponseModel.isSuccess()) {
				System.out.println(SUCCESS);
				return true;
			} else {
				System.out.println(FAIL);
				System.out.println(agentResponseModel.getFailCauseString());
				return false;
			}
		} catch (Exception e) {
			System.err.println(FAIL);
			e.printStackTrace();
			return false;
		}

	}

	@RequestMapping(value = "loadAgents", method = { RequestMethod.POST,
			RequestMethod.GET })
	@ResponseBody
	public AgentListModel loadAgentsFromPool(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken) {
		System.out.println("enter load Agent");
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendPost(
					this.getBaseUrl() + "/queryAgentList", null,
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println("reponse is null");
				return null;
			}
			System.out.println(httpResponse.getContent());
			AgentResponseModel agentResponseModel = this
					.extractAgentResponseModel(httpResponse.getContent());
			if (agentResponseModel.isSuccess()) {
				System.out.println(SUCCESS);
				return buildAgentListModel(agentResponseModel.getAgents());
			} else {
				System.out.println(FAIL);
				System.out.println(agentResponseModel.getFailCauseString());
				return null;
			}
		} catch (Exception e) {
			System.out.println(FAIL);
			e.printStackTrace();
			return null;

		}

	}

	private AgentListModel buildAgentListModel(List<AgentModel> agents) {
		AgentListModel listModel = new AgentListModel();
		listModel.setList(agents);
		return listModel;
	}

	private AgentResponseModel extractAgentResponseModel(String content)
			throws JAXBException {
		AgentResponseModel resultModel = new AgentResponseModel();
		Unmarshaller ummarshaller = JAXBContext.newInstance(
				resultModel.getClass()).createUnmarshaller();
		resultModel = (AgentResponseModel) ummarshaller
				.unmarshal(new ByteArrayInputStream(content.getBytes()));
		return resultModel;
	}

}
