package org.bench4q.web.api;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;
import org.apache.log4j.Logger;
import org.bench4q.share.communication.HttpRequester;
import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.models.master.AuthorizeResponseModel;
import org.bench4q.share.models.master.RegisterResponseModel;
import org.bench4q.share.models.master.UserModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.BaseResponseModel;
import org.bench4q.web.model.TestPlanTaskModel;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes({ "accessToken", "username", "testPlanTaskList" })
public class AuthorizeActionController extends BaseControllerService {
	private Logger logger = Logger.getLogger(AuthorizeActionController.class);
	private String baseUrl =  this.masterIp+"user";
	private String INVALIDATE_INPUT = "invalidate input";

	@RequestMapping("login")
	public @ResponseBody
	BaseResponseModel login(UserModel user, ModelMap model)
			throws CustomGenericException {
		isValidate(user);
		AuthorizeResponseModel authorizeResponseModel = authorize(baseUrl
				+ "/normalAuthorize", user);
		return extractAuthorizeResponseModel(authorizeResponseModel, user,
				model);
	}

	@RequestMapping("adminLogin")
	public @ResponseBody
	BaseResponseModel adminLogin(UserModel user, ModelMap model)
			throws CustomGenericException {
		isValidate(user);

		AuthorizeResponseModel authorizeResponseModel = authorize(baseUrl
				+ "/adminAuthorize", user);
		return extractAuthorizeResponseModel(authorizeResponseModel, user,
				model);
	}

	@RequestMapping("register")
	@ResponseBody
	public BaseResponseModel register(UserModel user)
			throws CustomGenericException {
		System.out.println("enter register!!");
		isValidate(user);
		Map<String, String> params = this.makeParamsMap("userName",
				user.getUserName());
		params.put("password", user.getPassword());
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendGet(
					baseUrl + "/register", params, null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				throw new CustomGenericException("0",
						"invalidate httpResponse",
						"AuthorizeActionController_register");
			}
			RegisterResponseModel registerResponseModel = (RegisterResponseModel) ObjectXmlExchange
					.fromXml(RegisterResponseModel.class,
							httpResponse.getContent());
			return extractRegisterModel(registerResponseModel);
		} catch (IOException e) {
			logger.info(e,e.fillInStackTrace());
			throw new CustomGenericException("1",
					"invalidate httpresponse:io exception", this.baseUrl
							+ "/register");
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			logger.info(e,e.fillInStackTrace());
			throw new CustomGenericException("1",
					"invalidate httpresponse:JAXBException", this.baseUrl
							+ "/register");
		}

	}

	private AuthorizeResponseModel authorize(String url, UserModel user)
			throws CustomGenericException {
		try {
			Map<String, String> params = this.makeParamsMap("userName",
					user.getUserName());
			params.put("password", user.getPassword());
			HttpResponse httpResponse = this.getHttpRequester().sendGet(url,
					params, null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				throw new CustomGenericException("1",
						"network error：",
						"AuthorizeActionController_authorize");
			}

			return (AuthorizeResponseModel) ObjectXmlExchange.fromXml(
					AuthorizeResponseModel.class, httpResponse.getContent());
		} catch (IOException e) {
			logger.info(e,e.fillInStackTrace());
			throw new CustomGenericException("1",
					"invalidate httpResponse:IOException",
					"AuthorizeActionController_authorize");
		} catch (JAXBException e) {
			logger.info(e,e.fillInStackTrace());
			throw new CustomGenericException("1",
					"invalidate httpResponse:JAXBException",
					"AuthorizeActionController_authorize");
		}
	}

	private BaseResponseModel extractAuthorizeResponseModel(
			AuthorizeResponseModel authorizeResponseModel, UserModel user,
			ModelMap model) {

		if (!authorizeResponseModel.isSuccess()) {
			logger.info("user name:" + user.getUserName());
			logger.info("user password:" + user.getPassword());
			logger.info("password or name wrong");
			return new BaseResponseModel(false, "password or name wrong!");
		} else {
			model.addAttribute("accessToken",
					authorizeResponseModel.getAccessToken());
			model.addAttribute("username", user.getUserName());
			List<TestPlanTaskModel> testPlanTaskModels = new ArrayList<TestPlanTaskModel>();
			model.addAttribute("testPlanTaskList", testPlanTaskModels);
			return new BaseResponseModel(true);
		}

	}

	private BaseResponseModel extractRegisterModel(
			RegisterResponseModel registerResponseModel) {

		if (registerResponseModel.isSuccess())
			return new BaseResponseModel(true);

		else
			return new BaseResponseModel(false, "");
	}

	private void isValidate(UserModel user) throws CustomGenericException {
		if (user.getPassword() == null || user.getUserName() == null
				|| user.getPassword().equals("")
				|| user.getUserName().equals(""))
			throw new CustomGenericException("0", INVALIDATE_INPUT, "");
	}

}
