package org.bench4q.web.api;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.bench4q.share.models.master.MonitorMemoryResponseModel;
import org.bench4q.share.models.master.MonitorProcessorResponseModel;
import org.bench4q.share.models.monitor.MemoryModel;
import org.bench4q.share.models.monitor.ProcessorModel;
import org.bench4q.share.models.monitor.ProcessorModelChild;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.model.BaseResponseModel;
import org.bench4q.web.model.ResultModel;
import org.bench4q.web.service.ChangeToResultModel;
import org.bench4q.web.service.MonitorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes({ "accessToken" })
public class MonitorController {
	private Logger logger = Logger.getLogger(MonitorService.class);
	private MonitorService monitorService;

	public MonitorService getMonitorService() {
		return monitorService;
	}

	@Autowired
	public void setMonitorService(MonitorService monitorService) {
		this.monitorService = monitorService;
	}

	@RequestMapping("/{testPlanId}/SUT/{ip}/{port}/cpu/{cpuInstance}/{fieldName}/{startTime}")
	@ResponseBody
	public BaseResponseModel getCpuResult(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable("testPlanId") String testplanID,
			@PathVariable String ip, @PathVariable String port,
			@PathVariable String cpuInstance, @PathVariable String fieldName,
			@PathVariable String startTime) throws CustomGenericException {
		MonitorProcessorResponseModel monitorProcessorResponseModel = this
				.getMonitorService().getCpus(accessToken, testplanID, ip, port,
						startTime);
		String source = "MonitorController:getCpuResult";
		try {
			List<ResultModel> resultModels = extractCpuResultModelList(
					monitorProcessorResponseModel.getProcessorModels(),
					cpuInstance, fieldName);
			if (resultModels == null)
				return new BaseResponseModel(false, "empty data");
			else {
				return new BaseResponseModel(true, resultModels);
			}
		} catch (NoSuchFieldException e) {
			logger.info(e.getStackTrace());
			logger.info("get cpu info error :no such field:" + e.getCause());
			logger.info("monitorController:getCpu_" + fieldName);
			throw new CustomGenericException("5",
					"get cpu info error :no such field:" + fieldName,
					"monitorController:getCpu_" + fieldName);
		} catch (IllegalArgumentException e) {

			throw new CustomGenericException("5", e.getMessage(), source);
		}
	}

	@RequestMapping("/testPlanId/SUT/{ip}/{port}/memory/{fieldName}/{startTime}")
	@ResponseBody
	public BaseResponseModel geMemoryStatus(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable("testPlanId") String testPlanId,
			@RequestParam String ip, @RequestParam String port,
			@PathVariable String fieldName, @PathVariable String startTime)
			throws CustomGenericException {

		String source = "MonitorController:getMemoryStatus";
		try {

			MonitorMemoryResponseModel monitorMemoryResponseModel = this
					.getMonitorService().getMemoryResponseModel(accessToken,
							testPlanId, ip, port, startTime);
			List<ResultModel> resultModels = extractResultModelList(
					monitorMemoryResponseModel.getMemoryModels(), fieldName);
			if (resultModels == null)
				return new BaseResponseModel(false, "empty data");
			else
				return new BaseResponseModel(true, resultModels);
		} catch (NoSuchFieldException e) {
			logger.info(e.getStackTrace());
			logger.info("get Memory info error :no such field:" + e.getCause());
			logger.info(source + "_" + fieldName);
			throw new CustomGenericException("5",
					"get memory info error :no such field:" + fieldName, source
							+ "_" + fieldName);
		}
	}

	// @RequestMapping("/testPlanIds/SUT/{ip}/{port}/network/{instance}/{fieldName}/{startTime}")
	// @ResponseBody
	// public BaseResponseModel getNetwor(
	// @ModelAttribute("accessToken") String accessToken,
	// @PathVariable("testPlanId") String testplanID,
	// @PathVariable String ip, @PathVariable String port,
	// @PathVariable String instance, @PathVariable String fieldName,
	// @PathVariable String startTime) throws CustomGenericException{
	// String source = "MonitorController:getNetwork";
	// try {
	// MonitorNetworkReponseModel monitorResponseModel = this
	// .getMonitorService().getNetworks(accessToken, testplanID,
	// ip, port, startTime);
	//
	// List<ResultModel> resultModels = extractNetworkResultModelList(
	// monitorResponseModel.getModels(), instance, fieldName);
	// if (resultModels == null)
	// return new BaseResponseModel(false, "empty data");
	// else
	// return new BaseResponseModel(true, resultModels);
	// } catch (NoSuchFieldException e) {
	// logger.info("get network info error :no such field:" + e.getCause());
	// logger.info(source + "_" + fieldName);
	// throw new CustomGenericException("5",
	// "get network info error :no such field:" + fieldName,
	// source + "_" + fieldName);
	// } catch (IllegalArgumentException e) {
	// throw new CustomGenericException("5", e.getMessage(), source);
	// }
	//
	// }

	private List<ResultModel> extractCpuResultModelList(
			List<ProcessorModel> processorModels, String instance,
			String fieldName) throws NoSuchFieldException,
			IllegalArgumentException {
		if (processorModels == null)
			return null;
		List<ResultModel> resultModels = new ArrayList<ResultModel>();
		for (ProcessorModel processorModel : processorModels) {
			if (processorModel != null) {
				ResultModel resultModel = extractResultModel(processorModel,
						instance, fieldName);
				if (resultModel != null)
					resultModels.add(resultModel);
			}

		}
		return resultModels;
	}

	private List<ResultModel> extractResultModelList(
			List<MemoryModel> memoryModels, String fieldName)
			throws NoSuchFieldException {
		if (memoryModels == null)
			return null;
		List<ResultModel> resultModels = new ArrayList<ResultModel>();
		List<String> fieldNames = new ArrayList<String>();
		fieldNames.add(fieldName);
		for (MemoryModel memoryModel : memoryModels) {
			if (memoryModel != null) {
				ResultModel resultModel = ChangeToResultModel.getResultModel(
						MemoryModel.class, fieldNames);
				resultModel.setTime(memoryModel.getSamplingTime());
				resultModels.add(resultModel);
			}
		}
		return resultModels;
	}

	// private List<ResultModel> extractNetworkResultModelList(
	// List<NetworkInterfaceModel> networkInterfaceModels,
	// String instance, String fieldName) throws NoSuchFieldException,
	// IllegalArgumentException {
	// if (networkInterfaceModels == null)
	// return null;
	// List<ResultModel> resultModels = new ArrayList<ResultModel>();
	//
	// for (NetworkInterfaceModel networkInterfaceModel :
	// networkInterfaceModels) {
	// if (networkInterfaceModel != null) {
	// ResultModel resultModel = extractResultModel(
	// networkInterfaceModel, instance, fieldName);
	// if (resultModel != null)
	// resultModels.add(resultModel);
	// }
	// }
	// return resultModels;
	// }

	private ResultModel extractResultModel(ProcessorModel processorModel,
			String instance, String fieldName) throws NoSuchFieldException,
			IllegalArgumentException {
		String source = "MonitorController:extractResultModel";
		List<String> fieldNames = new ArrayList<String>();
		fieldNames.add(fieldName);

		for (ProcessorModelChild processorModelChild : processorModel
				.getProcessorModelList()) {

			if (processorModelChild.getInstance().equalsIgnoreCase(instance)) {
				ResultModel resultModel = ChangeToResultModel.getResultModel(
						processorModelChild, fieldNames);
				resultModel.setFinished(false);
				resultModel.setTime(processorModel.getSamplingTime());
				return resultModel;
			}
		}
		logger.info("no such cpu instance:" + instance + "_" + source);
		throw new IllegalArgumentException("no such cpu instance:" + instance);
	}

	// private ResultModel extractResultModel(
	// NetworkInterfaceModel networkInterfaceModel, String instance,
	// String fieldName) throws NoSuchFieldException {
	//
	// String source = "MonitorController:extractResultModel";
	// List<String> fieldNames = new ArrayList<String>();
	// fieldNames.add(fieldName);
	//
	// for (NetworkInterfaceModelChild networkInterfaceModelChild :
	// networkInterfaceModel
	// .getNetworkList()) {
	//
	// if (networkInterfaceModelChild.getInstance().equalsIgnoreCase(
	// instance)) {
	// ResultModel resultModel = ChangeToResultModel.getResultModel(
	// networkInterfaceModelChild, fieldNames);
	// resultModel.setFinished(false);
	// resultModel.setTime(networkInterfaceModel.getSamplingTime());
	// return resultModel;
	// }
	// }
	// logger.info("no such network instance:" + instance + "_" + source);
	// throw new IllegalArgumentException("no such network instance:"
	// + instance);
	// }

}
