package org.bench4q.web.api;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.master.plugin.MethodParamModel;
import org.bench4q.share.models.master.plugin.PluginResponseModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.model.BaseResponseModel;
import org.bench4q.web.model.UploadBehaviorRequestModel;
import org.bench4q.web.service.BaseService;
import org.bench4q.web.service.CommunicateWithMaster;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes({"accessToken","loadMethod"})
public class PluginActionController {

	private final String baseUrl = "plugin";
	private final String BASECALLER = "PluginActionController:";
	private CommunicateWithMaster communicateWithMaster;

	public CommunicateWithMaster getCommunicateWithMaster() {
		return communicateWithMaster;
	}

	@Autowired
	public void setCommunicateWithMaster(
			CommunicateWithMaster communicateWithMaster) {
		this.communicateWithMaster = communicateWithMaster;
	}

	public String getBaseUrl() {
		return baseUrl;
	}

	public String getBASECALLER() {
		return BASECALLER;
	}

	@RequestMapping("loadPlugin")
	public @ResponseBody
	BaseResponseModel loadPlugin(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken)
			throws CustomGenericException {
		System.out.println("loadPluginList!!");
		String caller = new String("PluginActionController:loadPlugin");
		String url = this.getBaseUrl() + "/loadPluginList";
		PluginResponseModel pluginResponseModel = (PluginResponseModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken, url,
						PluginResponseModel.class, null, caller);
		try {
			Logger.getLogger(PluginActionController.class).info(MarshalHelper.marshal(PluginResponseModel.class, pluginResponseModel));
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		if (pluginResponseModel.isSuccess()) {
			System.out.println("pluginResponseModel has been used");
			List<String> pluginList = pluginResponseModel.getPluginList();
			//System.out.println(pluginList.size());
			if (pluginList == null) {
				pluginList = new ArrayList<String>();
				System.out.println("pluginList is empty");
			}
			return new BaseResponseModel(true, pluginList);
		} else {
			return new BaseResponseModel(false,
					pluginResponseModel.getFailMessage());
		}
	}

	@RequestMapping("loadMethod")
	public @ResponseBody
	BaseResponseModel loadMethod(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String pluginName) throws CustomGenericException {

		System.out.println("loadMethodList!!");
		String caller = new String("PluginActionController:loadMethodList");
		String url = this.getBaseUrl() + "/loadMethodList/" + pluginName;
		Map<String, String> params = BaseService.makeParamsMap("pluginName",
				pluginName);
		PluginResponseModel pluginResponseModel = (PluginResponseModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken, url,
						PluginResponseModel.class, params, caller);

		try {
			Logger.getLogger(PluginActionController.class).info(MarshalHelper.marshal(PluginResponseModel.class, pluginResponseModel));
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
		if (pluginResponseModel.isSuccess()) {
			System.out.println("pluginResponseModel has been used");
			List<String> methodList = pluginResponseModel.getMethodList();
			if (methodList == null) {
				methodList = new ArrayList<String>();
				System.out.println("methodList is empty");
			}
			return new BaseResponseModel(true, methodList);
		} else {
			return new BaseResponseModel(false,
					pluginResponseModel.getFailMessage());
		}
	}

	@RequestMapping("loadMethodParameters")
	public @ResponseBody
	BaseResponseModel loadMethodParameters(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String pluginName, @RequestParam String methodName)
			throws CustomGenericException {

		System.out.println("loadMethodParameters!!");
		String caller = new String(
				"PluginActionController:loadMethodParameters");
		String url = this.getBaseUrl() + "/loadMethodParams/" + pluginName
				+ "/" + methodName;

		Map<String, String> params = BaseService.makeParamsMap("pluginName",
				pluginName);
		params.put("methodName", methodName);

		PluginResponseModel pluginResponseModel = (PluginResponseModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken, url,
						PluginResponseModel.class, params, caller);

		if (pluginResponseModel.isSuccess()) {
			System.out.println("pluginResponseModel has been used");
			Set<MethodParamModel> methodParamSet = pluginResponseModel
					.getMethosMethodParamModels();
			if (methodParamSet == null) {
				methodParamSet = new HashSet<MethodParamModel>();
			}
			return new BaseResponseModel(true, methodParamSet);
		} else {
			return new BaseResponseModel(false,
					pluginResponseModel.getFailMessage());
		}

	}
	
	@RequestMapping(value = "uploadBehavior", method = RequestMethod.POST)
	@ResponseBody
	public BaseResponseModel runTestPlan(
			@ModelAttribute("accessToken") String accessToken, ModelMap model,
			@RequestBody UploadBehaviorRequestModel uploadBehaviorRequestModel)
			throws CustomGenericException {
		
		
		return new BaseResponseModel(true);
	}

}
