package org.bench4q.web.api;

import java.io.ByteArrayInputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.bench4q.share.communication.HttpRequester;
import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.models.master.OrganizeRecordPortResponseModel;
import org.bench4q.share.models.master.PortModel;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.PortListModel;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes("accessToken")
public class RecordPortActionController extends BaseControllerService {
	private String baseUrlString =  this.masterIp+"RecordPort";
	
	public String getBaseUrlString() {
		return baseUrlString;
	}

	public void setBaseUrlString(String baseUrlString) {
		this.baseUrlString = baseUrlString;
	}

	@RequestMapping("addPortToPool")
	@ResponseBody
	public  boolean addPorttoPortPool(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String port) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("port", port);
		HttpResponse httpResponse;
		try {
			httpResponse = this.getHttpRequester().sendPost(
					this.getBaseUrlString() + "/addPortToPortPool", params,
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println("fail");
				return false;
			}
			OrganizeRecordPortResponseModel organizeRecordPortResponseModel = extractOrganizeRecordPortResponseModel(httpResponse
					.getContent());

			if (organizeRecordPortResponseModel.isSuccess()) {
				System.out.println("success");
				return true;
			} else {
				System.out.println("failure");
				System.out.println(organizeRecordPortResponseModel
						.getFailCauseString());
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	@RequestMapping("removePortFromPool")
	@ResponseBody
	public Boolean removePortfromPortPool(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken) {
		System.out.println("enter remove poot");
		String id = request.getParameter("id");
		System.out.println(id);
		Map<String, String> Params = new HashMap<String, String>();
		Params.put("portId", id);

		HttpResponse httpResponse;
		try {
			httpResponse = this.getHttpRequester().sendPost(
					this.getBaseUrlString() + "/removePortFromPool", Params,
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println("fail");
				return false;
			}
			OrganizeRecordPortResponseModel organizeRecordPortResponseModel = extractOrganizeRecordPortResponseModel(httpResponse
					.getContent());
			if (organizeRecordPortResponseModel.isSuccess()) {
				System.out.println("success");
				return true;
			} else {
				System.out.println("failure");
				System.out.println(organizeRecordPortResponseModel
						.getFailCauseString());
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	private OrganizeRecordPortResponseModel extractOrganizeRecordPortResponseModel(
			String content) throws JAXBException {
		OrganizeRecordPortResponseModel resultModel = new OrganizeRecordPortResponseModel();
		Unmarshaller ummarshaller = JAXBContext.newInstance(
				resultModel.getClass()).createUnmarshaller();
		resultModel = (OrganizeRecordPortResponseModel) ummarshaller
				.unmarshal(new ByteArrayInputStream(content.getBytes()));
		return resultModel;

	}

	@RequestMapping("loadPorts")
	@ResponseBody
	public PortListModel loadPorts(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken) {
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendPost(
					this.getBaseUrlString() + "/loadPortList", null,
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println(FAIL);
				return null;
			}
			OrganizeRecordPortResponseModel responseModel = extractOrganizeRecordPortResponseModel(httpResponse
					.getContent());
			if (responseModel.isSuccess()) {
				System.out.println("success");
				return buildPortListModel(responseModel.getPortModels());
			} else {
				System.out.println(FAIL);
				return null;
			}
		} catch (Exception e) {
			System.out.println(FAIL_WITH_EXCEPTION);
			e.printStackTrace();
			return null;
		}
	}

	private PortListModel buildPortListModel(List<PortModel> ports) {
		PortListModel result = new PortListModel();
		result.setList(ports);
		return result;
	}

	@RequestMapping("editPort")
	@ResponseBody
	public Boolean editPort(@ModelAttribute("accessToken") String accessToken,
			@RequestParam PortModel newPort) {
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendPost(
					this.getBaseUrlString() + "/editPort",
					makeParamsMap("port",
							ObjectXmlExchange.toXml(PortModel.class, newPort)),
					this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				System.out.println(FAIL);
				return false;
			}
			OrganizeRecordPortResponseModel responseModel = extractOrganizeRecordPortResponseModel(httpResponse
					.getContent());
			if (responseModel.isSuccess()) {
				System.out.println(SUCCESS);
				return true;
			} else {
				System.out.println(FAIL);
				return false;
			}
		} catch (Exception e) {
			System.out.println(FAIL_WITH_EXCEPTION);
			e.printStackTrace();
			return false;
		}
	}

}
