package org.bench4q.web.api;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.master.OperateScriptServerResponseModel;
import org.bench4q.share.models.master.ScriptModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.BaseResponseModel;
import org.bench4q.web.service.BaseService;
import org.bench4q.web.service.ScriptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import org.xml.sax.SAXException;

import com.google.gson.Gson;

@Controller
@SessionAttributes("accessToken")
public class ScriptActionController  {

	private final String baseUrl =  "RecordScript";
	private ScriptService scriptService;
	private final String BASECALLER = "ScriptActionController:";

	private String getBaseUrl() {
		return baseUrl;
	}

	private ScriptService getScriptService() {
		return scriptService;
	}

	@Autowired
	private void setScriptService(ScriptService scriptService) {
		this.scriptService = scriptService;
	}

	@RequestMapping("loadScript")
	public @ResponseBody
	BaseResponseModel loadScript(
			@ModelAttribute("accessToken") String accessToken)
			throws CustomGenericException {
		String caller = "ScriptActionController:loadScripts";
		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken,
						this.getBaseUrl() + "/loadScriptList",
						OperateScriptServerResponseModel.class, null, caller);

		if (operateScriptServerResponseModel.isSuccess()) {
			List<ScriptModel> scripts = operateScriptServerResponseModel
					.getScriptModels();
			if(scripts==null)
				scripts=new ArrayList<ScriptModel>();
				    return new BaseResponseModel(true, scripts);
		} else {

			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());
		}

	}

	@RequestMapping("deleteScript")
	public @ResponseBody
	BaseResponseModel deletescript(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String scriptId) throws CustomGenericException {
		String caller = new String("ScriptActionComntroller:deleteScript");
		String url = this.getBaseUrl() + "/deleteScript";
		Map<String, String> params = BaseService.makeParamsMap("scriptId", scriptId);
		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, params, caller);
		if (operateScriptServerResponseModel.isSuccess()) {
			return new BaseResponseModel(true);
		} else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());
	}

	@RequestMapping("editScript")
	public @ResponseBody
	BaseResponseModel editScript(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String scriptId) throws CustomGenericException {
		String caller = this.BASECALLER + "editScript";
		String url = this.getBaseUrl() + "/queryScriptById";
		Map<String, String> params = BaseService.makeParamsMap("scriptId", scriptId);
		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, params, caller);

		if (operateScriptServerResponseModel.isSuccess()) {
			List<ScriptModel> scripts = operateScriptServerResponseModel
					.getScriptModels();
			if (scripts == null || scripts.isEmpty())
				return new BaseResponseModel(false, "data empty");
			else {
				Iterator<ScriptModel> it = scripts.iterator();
				ScriptModel script = it.next();
				if (script.getScriptContent() == null
						|| script.getScriptContent().isEmpty())
					return new BaseResponseModel(false, "data empty");
				else {
					try {
						RunScenarioModel result = (RunScenarioModel) ObjectXmlExchange
								.fromXml(RunScenarioModel.class,
										script.getScriptContent());
						Gson gson = new Gson();
						String scriptContent = null;
						scriptContent = gson.toJson(result);
						return new BaseResponseModel(true,
								(Object) scriptContent);
					} catch (JAXBException e) {
						throw new CustomGenericException("1",
								"script content is invalidate", caller);
					}
				}
			}
		} else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());

	}

	@RequestMapping("saveEditScript")
	public @ResponseBody
	BaseResponseModel saveScript(
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String scriptId, @RequestParam String script)
			throws CustomGenericException {
		String url = this.getBaseUrl() + "/updateScript";
		String caller = this.BASECALLER + "saveScript";
		Gson gson = new Gson();
		RunScenarioModel scriptModel = gson.fromJson(script,
				RunScenarioModel.class);
		try{
		String scriptContent = ObjectXmlExchange.toXml(RunScenarioModel.class,
				scriptModel);
		Map<String, String> params = BaseService.makeParamsMap("scriptId", scriptId);
		params.put("content", scriptContent);

		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, params, caller);
		if (operateScriptServerResponseModel.isSuccess()) {
			return new BaseResponseModel(true);
		} else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());
		}
		catch(JAXBException e){
			return new BaseResponseModel(false,e.getMessage());
		}
	}

	@RequestMapping("startRecordServer")
	public @ResponseBody
	BaseResponseModel startRecordServer(
			@ModelAttribute("accessToken") String accessToken)
			throws JAXBException, CustomGenericException {
		String url = this.getBaseUrl() + "/startScriptRecordServer";
		String caller = BASECALLER + "startRecordServer";
		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, null, caller);

		if (operateScriptServerResponseModel.isSuccess())
			return new BaseResponseModel(true,
					operateScriptServerResponseModel);
		else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());
	}

	@RequestMapping("stopRecordServer")
	public @ResponseBody
	BaseResponseModel stopRecordServer(
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String port, @RequestParam String fileNameUUID)
			throws CustomGenericException {
		String caller = BASECALLER + "stopeRecordServer";
		String url = this.getBaseUrl() + "/stopScriptRecordServer";
		Map<String, String> params = BaseService.makeParamsMap("port", port);
		params.put("fileNameUUID", fileNameUUID);
		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, params, caller);
		if (operateScriptServerResponseModel.isSuccess())
			return new BaseResponseModel(true);
		else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());
	}

	@RequestMapping("saveRecordScript")
	public @ResponseBody
	BaseResponseModel saveScriptToDB(HttpServletRequest request,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String port, @RequestParam String scriptName,
			@RequestParam String fileNameUUID) throws CustomGenericException {
		String caller = BASECALLER + "saveRecordScript";
		String url = this.getBaseUrl() + "/saveScriptToDB";
		Map<String, String> params = BaseService.makeParamsMap("scriptName",
				scriptName);
		params.put("port", port);
		params.put("fileNameUUID", fileNameUUID);

		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getScriptService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						OperateScriptServerResponseModel.class, params, caller);
		if (operateScriptServerResponseModel.isSuccess())
			return new BaseResponseModel(true);
		else
			return new BaseResponseModel(false,
					operateScriptServerResponseModel.getFailCauseString());

	}

	@RequestMapping("uploadScript")
	@ResponseBody
	public BaseResponseModel uploadScript(
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam("script") CommonsMultipartFile script,
			@RequestParam String scriptName) throws CustomGenericException {
		String caller=BASECALLER+"uploadScript";
		String url=this.getBaseUrl()+"/uploadScript"+"/"+scriptName;
		if (script.isEmpty()) {
			return new BaseResponseModel(false,"empty file");
		}
		if (scriptName == null || scriptName.equals(""))
			return new BaseResponseModel(false,"empty file name");
		try{
		String scenarioModel = new String(script.getBytes());
			ObjectXmlExchange.fromXml(RunScenarioModel.class, scenarioModel);
			OperateScriptServerResponseModel operateScriptServerResponseModel=
					(OperateScriptServerResponseModel)this.getScriptService().
					getCommunicateWithMaster().getResponseModelByPut(accessToken, url, scenarioModel, OperateScriptServerResponseModel.class, caller);
			if (operateScriptServerResponseModel.isSuccess())
				return new BaseResponseModel(true,(Object)new String("upload script:" + scriptName + " success!"));
			
			else
				return new BaseResponseModel(false,operateScriptServerResponseModel.getFailCauseString());
		} catch (JAXBException e) {
			return new BaseResponseModel(false,"Failed:invalidated script file!");
		}

	}

	@RequestMapping("validateScript")
	public @ResponseBody
	String validateScript(@ModelAttribute("accessToken") String accessToken,
			@RequestParam String content) {
		String result;
		String message = "";
		String schemaFileName = "src/main/webapp/WEB-INF/validateScript.xsd";
		content = content.replace("runScenario", "runscenario")
				.replace("poolSize", "poolsize")
				.replace("usePlugin", "useplugin")
				.replace("userBehavior", "userbehavior")
				.replace("timerBehavior", "timerbehavior")
				.replace("userbehaviors", "behaviors");
		try {
			result = validateXmlWithSchema(schemaFileName, content);
			if (result == "true") {
				message = "Success";
			} else {
				message = URLEncoder.encode(result, "UTF-8").replace("+", "");
			}
		} catch (Exception e) {
			System.out.println("analysis of failure:" + e.toString());
			message = "failed to read schema file";
		}
		// System.out.println(message);
		return message;
	}

	private String validateXmlWithSchema(String schemaFileName, String xmlScript)
			throws SAXException, IOException {
		String validationResult = "";
		SchemaFactory schemaFactory = SchemaFactory
				.newInstance("http://www.w3.org/2001/XMLSchema");
		File schemaFile = new File(schemaFileName);
		Schema schema = schemaFactory.newSchema(schemaFile);
		Validator validator = schema.newValidator();
		InputStream script = new ByteArrayInputStream(xmlScript.getBytes());
		Source source = new StreamSource(script);
		try {
			// System.out.println("enter validateXmlWithSchema");
			validator.validate(source);
			validationResult = "true";
		} catch (Exception ex) {
			validationResult = ex.toString();
		}
		return validationResult;
	}

	@RequestMapping("readSchemaContent")
	public @ResponseBody
	String readSchemaContent() throws Exception {
		String schema = "";
		FileReader fr = new FileReader("src/main/webapp/WEB-INF/schema.json");
		BufferedReader br = new BufferedReader(fr);
		StringBuilder sb = new StringBuilder();
		while ((schema = br.readLine()) != null) {
			sb.append(schema);
		}
		br.close();
		String str = sb.toString();
		return str;
	}

}
