package org.bench4q.web.api;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXBException;

import org.bench4q.share.models.agent.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.master.MonitorModel;
import org.bench4q.share.models.master.ScriptModel;
import org.bench4q.share.models.master.TestPlanScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptPagesBriefModel;
import org.bench4q.web.TestPlan.RunTestPlan;
import org.bench4q.web.TestPlan.TestPlanFeature;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.model.*;
import org.bench4q.web.service.CommunicateWithMaster;
import org.bench4q.web.service.MonitorDescriptionInfoService;
import org.bench4q.web.service.ScriptService;
import org.bench4q.web.service.TestPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes({ "accessToken", "testPlanTaskList" })
public class TestPlanActionController {
	private final String baseUrl = "testPlan";
	private final String BASECALLER = "TestPlanActionController:";
	private TestPlanService testPlanService;
	private CommunicateWithMaster communicateWithMaster;
	private MonitorDescriptionInfoService monitorDescriptionInfoService;
	private RunTestPlan runTestPlan;
	private TestPlanFeature testPlanInfo;

	private ScriptService scriptService;

	public ScriptService getScriptService() {
		return scriptService;
	}

	@Autowired
	public void setScriptService(ScriptService scriptService) {
		this.scriptService = scriptService;
	}

	private TestPlanFeature getTestPlanInfo() {
		return testPlanInfo;
	}

	@Autowired
	private void setTestPlanInfo(TestPlanFeature testPlanInfo) {
		this.testPlanInfo = testPlanInfo;
	}

	private RunTestPlan getRunTestPlan() {
		return runTestPlan;
	}

	@Autowired
	private void setRunTestPlan(RunTestPlan runTestPlan) {
		this.runTestPlan = runTestPlan;
	}

	private CommunicateWithMaster getCommunicateWithMaster() {
		return this.communicateWithMaster;
	}

	@Autowired
	private void setCommunicateWithMaster(
			CommunicateWithMaster communicateWithMaster) {
		this.communicateWithMaster = communicateWithMaster;
	}

	private String getBaseUrl() {
		return baseUrl;
	}

	private TestPlanService getTestPlanService() {
		return testPlanService;
	}

	@Autowired
	private void setTestPlanService(TestPlanService testPlanService) {
		this.testPlanService = testPlanService;
	}

	@Autowired
	private void setMonitorDescriptionInfoService(
			MonitorDescriptionInfoService monitorDescriptionInfoService) {
		this.monitorDescriptionInfoService = monitorDescriptionInfoService;
	}

	@RequestMapping(value = "runTestPlan", method = RequestMethod.POST)
	@ResponseBody
	public BaseResponseModel runTestPlan(
			@ModelAttribute("accessToken") String accessToken, ModelMap model,
			@RequestBody TestPlanRequestModel testPlanRequestModel)
			throws CustomGenericException {
		String caller = this.BASECALLER + "runTestPlan";
		if (this.getTestPlanService().isValidateTestRunModel(
				testPlanRequestModel)) {
			throw new CustomGenericException("0",
					"run testplan request model is invalidate", caller);
		}
		this.getRunTestPlan().runTestPlan(accessToken, testPlanRequestModel,
				model);
		return new BaseResponseModel(true);
	}

	@RequestMapping(value = "testPlanTaskList", method = RequestMethod.POST)
	@ResponseBody
	public BaseResponseModel getTestPlanTaskList(
			@ModelAttribute("accessToken") String accessToken,
			@ModelAttribute("testPlanTaskList") List<TestPlanTaskModel> testPlanTaskModels,
			ModelMap model) throws CustomGenericException, JAXBException {
		String caller = this.BASECALLER + "testPlanTaskList";
		Iterator<TestPlanTaskModel> iterator = testPlanTaskModels.iterator();
		while (iterator.hasNext()) {
			TestPlanTaskModel testPlanTaskModel = iterator.next();
			WebTestPlanResultModel testPlanResultModel = this.getTestPlanInfo()
					.getTestPlanInfo(accessToken,
							testPlanTaskModel.getId().toString(), caller);
			if (testPlanResultModel.getCurrentStatus() == null)
				testPlanTaskModel.setStatus("-1");
			else
				testPlanTaskModel.setStatus(testPlanResultModel
						.getCurrentStatus().toString());
		}
		return new BaseResponseModel(true, testPlanTaskModels);
	}

	@RequestMapping("/{testPlanId}/runningInfo")
	@ResponseBody
	public BaseResponseModel getRunningInfo(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException {
		String caller = this.BASECALLER + "getRunningInfo";
		return new BaseResponseModel(true, (Object) this.getTestPlanInfo()
				.getRunningTestPlanModel(accessToken, testPlanId, caller));

	}

	// get the test plan run info (running,finished)
	@RequestMapping(value = "/{testPlanId}/queryTestPlan")
	@ResponseBody
	public BaseResponseModel queryTestPlan(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		String caller = "TestPlanActionController:queryTestPlan";
		if (!BaseControllerService.validateInput(testPlanId))
			throw new CustomGenericException("0", "invalidate testplan id",
					caller);
		return new BaseResponseModel(true, this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller));
	}

	@RequestMapping("/{testPlanId}/runningScriptInfo")
	@ResponseBody
	public BaseResponseModel getTestPlanScriptInfo(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		String caller = this.BASECALLER + "getTestPlanScriptInfo";
		List<RunningScriptInfoModel> runningScriptInfoModels = new ArrayList<RunningScriptInfoModel>();
		WebTestPlanResultModel testPlanResultModel = this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller);
		Set<Integer> scriptIdList = testPlanResultModel.getScriptIdNameMap()
				.keySet();
		if (scriptIdList == null) {
			throw new CustomGenericException("5",
					"empty runningScript in test plan:" + testPlanId, caller);
		}
		for (Integer scriptId : scriptIdList) {
			RunningScriptInfoModel runningScriptInfoModel = new RunningScriptInfoModel();
			runningScriptInfoModel.setScriptId(scriptId);
			String scriptName = this.getScriptService()
					.queryScript(scriptId, accessToken).getName();
			runningScriptInfoModel.setScriptName(scriptName);
			runningScriptInfoModel
					.setChildResults(new ArrayList<ResultInfoModel>());
			runningScriptInfoModel.getChildResults().add(
					this.getTestPlanService().getScriptBriefStatusResultInfo());
			runningScriptInfoModels.add(runningScriptInfoModel);
		}
		return new BaseResponseModel(true, runningScriptInfoModels);
	}

	@RequestMapping("/{testPlanId}/mapInfo")
	@ResponseBody
	public BaseResponseModel getReplaceList(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		String caller = this.BASECALLER + "getReplaceList";
		Map<String, Integer> map = new HashMap<String, Integer>();
		WebTestPlanResultModel testPlanResultModel = this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller);
		if (testPlanResultModel.getScriptIds() != null) {
			List<ScriptModel> scriptModels = this.getScriptService()
					.queryScripts(testPlanResultModel.getScriptIds(),
							accessToken);
			Map<String, Integer> scriptNameAndIdMap = this.getScriptService()
					.getScriptNameIdMap(scriptModels);
			map.putAll(scriptNameAndIdMap);
			Map<String, Integer> pageNameAndIdMap = this.getScriptService()
					.getPageNameAndIdMap(scriptModels);
			map.putAll(pageNameAndIdMap);
			Map<String, Integer> behaviorMap = this.getScriptService()
					.getBehaviorNameAndIdMap(scriptModels);
			map.putAll(behaviorMap);
			return new BaseResponseModel(true, (Object) map);
		} else
			return new BaseResponseModel(false, "empty script in testPlan:"
					+ testPlanId);
	}

	@RequestMapping("/{testPlanId}/sutInfo")
	@ResponseBody
	public BaseResponseModel getSutInfo(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		String caller = this.BASECALLER + "getSutInfo";
		WebTestPlanResultModel testPlanResultModel = this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller);
		List<MonitorModel> monitorModels = testPlanResultModel
				.getMonitorModels();
		if (monitorModels != null) {
			List<SutInfo> sutInfos = new ArrayList<SutInfo>();
			for (MonitorModel monitorModel : monitorModels) {
				SutInfo sutInfo = this.monitorDescriptionInfoService
						.getSutInfo(monitorModel.getHostName(), new Integer(
								monitorModel.getPort()).toString(),
								accessToken, testPlanId);
				sutInfos.add(sutInfo);
			}
			return new BaseResponseModel(true, sutInfos);
		} else
			return new BaseResponseModel(false, caller
					+ ":monitor info in testPlan is empty:" + testPlanId);

	}

	@RequestMapping("/{testPlanId}/script/{scriptId}/briefStatus/{fieldName}/{startTime}")
	@ResponseBody
	public BaseResponseModel getScriptBriefStatus(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId, @PathVariable String scriptId,
			@PathVariable String fieldName, @PathVariable String startTime)
			throws CustomGenericException, NoSuchFieldException {
		String caller = "TestPlanActionController:getScriptBriefStatus";
		if (!BaseControllerService.validateInput(testPlanId)
				|| !BaseControllerService.validateInput(scriptId)) {
			return new BaseResponseModel(false, caller
					+ "invalidate scriptid or testplanid");
		}
		String url = this.getBaseUrl() + "/scriptBrief" + "/" + testPlanId
				+ "/" + scriptId + "/" + startTime;
		TestPlanScriptBriefResultModel testPlanScriptBriefResultModel = (TestPlanScriptBriefResultModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken, url,
						TestPlanScriptBriefResultModel.class, null, caller);
		return new BaseResponseModel(true, this.getTestPlanService()
				.extractResultModelList(testPlanScriptBriefResultModel,
						fieldName));
	}

	@RequestMapping("/{testPlanId}/script/{scriptId}/behaviors")
	@ResponseBody
	public BaseResponseModel getScriptBehaviorBriefModel(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId, @PathVariable String scriptId)
			throws CustomGenericException {
		String caller = this.BASECALLER + "getScriptBehaviorBriefModel";
		String url = this.getBaseUrl() + "/getBehaviorsBrief" + "/"
				+ testPlanId + "/" + scriptId;
		if (!BaseControllerService.validateInput(testPlanId)
				|| !BaseControllerService.validateInput(scriptId)) {
			return new BaseResponseModel(false, caller
					+ ":invalidate scriptid or testplanid:");
		}
		ScriptBehaviorsBriefModel scriptBehaviorsBriefModel = (ScriptBehaviorsBriefModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken, url,
						ScriptBehaviorsBriefModel.class, null, caller);
		return new BaseResponseModel(true, scriptBehaviorsBriefModel);
	}

	@RequestMapping("/{testPlanId}/script/{scriptId}/page")
	@ResponseBody
	public BaseResponseModel getScriptPageBriefModel(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId, @PathVariable String scriptId)
			throws CustomGenericException {
		String caller = this.BASECALLER + "getScriptPageBriefModel";
		String url = this.getBaseUrl() + "/pagesBrief" + "/" + testPlanId + "/"
				+ scriptId;
		ScriptPagesBriefModel scriptPagesBriefModel = (ScriptPagesBriefModel) this
				.getTestPlanService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						ScriptPagesBriefModel.class, null, caller);
		return new BaseResponseModel(true, scriptPagesBriefModel);
	}

	@SuppressWarnings("unchecked")
	@RequestMapping("/{testPlanId}/urlContentTypeDistribution")
	@ResponseBody
	public BaseResponseModel getUrlContentTypeDistribution(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		List<UrlContentTypeDistributionModel> urlContentTypeDistributionModels = new ArrayList<UrlContentTypeDistributionModel>();
		Map<String, Long> urlContentTypeCountMap = (Map<String, Long>) getUrlContentTypeCount(
				accessToken, testPlanId).getData();
		long totalCount = this.getTestPlanService().getTotalCount(
				urlContentTypeCountMap);
		Map<String, Double> urlContentTypePercentMap = this
				.getTestPlanService().getUrlContentTypePercent(
						urlContentTypeCountMap, totalCount);
		Set<String> keys = urlContentTypeCountMap.keySet();
		Iterator<String> iterator = keys.iterator();
		while (iterator.hasNext()) {
			String key = iterator.next();
			urlContentTypeDistributionModels
					.add(new UrlContentTypeDistributionModel(key,
							urlContentTypeCountMap.get(key),
							urlContentTypePercentMap.get(key)));
		}
		return new BaseResponseModel(true, urlContentTypeDistributionModels);

	}

	@SuppressWarnings("unchecked")
	@RequestMapping("/{testPlanId}/urlContentTypePercent")
	@ResponseBody
	public BaseResponseModel getUrlContentTypePercent(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		Map<String, Long> urlContentTypeCountMap = (Map<String, Long>) getUrlContentTypeCount(
				accessToken, testPlanId).getData();
		long totalCount = this.getTestPlanService().getTotalCount(
				urlContentTypeCountMap);

		return new BaseResponseModel(true, this.getTestPlanService()
				.getUrlContentTypePercent(urlContentTypeCountMap, totalCount));
	}

	@RequestMapping("/{testPlanId}/urlContentTypeCount")
	@ResponseBody
	public BaseResponseModel getUrlContentTypeCount(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId) throws CustomGenericException,
			JAXBException {
		Map<String, Long> urlContentCountMap = new HashMap<String, Long>();

		for (BehaviorStatusCodeResultModel behaviorStatusCodeResultModel : getBehaviorStatusCodeResultModels(
				accessToken, testPlanId)) {
			if (urlContentCountMap.get(behaviorStatusCodeResultModel
					.getContentType()) == null)
				urlContentCountMap.put(
						behaviorStatusCodeResultModel.getContentType(),
						behaviorStatusCodeResultModel.getCount());
			else {
				long originalValue = urlContentCountMap
						.get(behaviorStatusCodeResultModel.getContentType());
				urlContentCountMap.put(
						behaviorStatusCodeResultModel.getContentType(),
						originalValue
								+ behaviorStatusCodeResultModel.getCount());
			}

		}
		return new BaseResponseModel(true, urlContentCountMap);

	}

	@RequestMapping("/{testPlanId}/testBriefStatus/{startTime}")
	@ResponseBody
	public BaseResponseModel getTotalRequest(
			@ModelAttribute("accessToken") String accessToken,
			@PathVariable String testPlanId, @PathVariable String startTime)
			throws CustomGenericException, JAXBException {
		String caller = this.BASECALLER + "getTotalRequest";
		if (!BaseControllerService.validateInput(testPlanId)) {
			return new BaseResponseModel(false, caller
					+ "  :testplanId is invalidate:" + testPlanId);
		}
		TestBriefStatusModel testBriefStatusModel = new TestBriefStatusModel();
		long vus = 0;
		long requests = 0;
		Date time = new Date(0);
		if (this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller)
				.getScriptIds() == null)
			throw new CustomGenericException("5", caller
					+ " :scripts in testPlan is empty", caller);
		for (Integer scriptId : this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller)
				.getScriptIds()) {
			ScriptBriefResultModel lastScriptBriefResultModel = getLastScriptBriefModel(
					accessToken, testPlanId, startTime, scriptId);
			requests += lastScriptBriefResultModel.getTotalFailCountFromBegin()
					+ lastScriptBriefResultModel
							.getTotalSuccessCountFromBegin();
			vus += lastScriptBriefResultModel.getvUserCount();
			if (lastScriptBriefResultModel.getSamplingTime().after(time))
				time = lastScriptBriefResultModel.getSamplingTime();
		}
		testBriefStatusModel.setRequests(requests);
		testBriefStatusModel.setVus(vus);
		testBriefStatusModel.setTime(time);
		return new BaseResponseModel(true, testBriefStatusModel);

	}

	private ScriptBriefResultModel getLastScriptBriefModel(String accessToken,
			String testPlanId, String startTime, Integer scriptId)
			throws CustomGenericException {
		String caller = "TestPlanActionController:getLastScriptBriefModel";
		String[] paths = new String[] { "scriptBrief", testPlanId,
				new Integer(scriptId).toString(), startTime };
		String url = generateUrl(this.getBaseUrl(), paths);
		TestPlanScriptBriefResultModel testPlanScriptBriefResultModel = (TestPlanScriptBriefResultModel) this
				.getTestPlanService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						TestPlanScriptBriefResultModel.class, null, caller);
		if (testPlanScriptBriefResultModel.getScriptBriefResultModels() == null)
			throw new CustomGenericException("5", "script result empty", caller);
		int lastScriptBriefResultIndex = testPlanScriptBriefResultModel
				.getScriptBriefResultModels().size();
		if (lastScriptBriefResultIndex == 0)
			throw new CustomGenericException("5", "script result empty", caller);
		ScriptBriefResultModel lastScriptBriefResultModel = testPlanScriptBriefResultModel
				.getScriptBriefResultModels().get(
						lastScriptBriefResultIndex - 1);
		return lastScriptBriefResultModel;

	}

	private String generateUrl(String baseUrl, String[] paths) {
		String url = baseUrl;
		for (String path : paths) {
			url += "/" + path;
		}
		return url;
	}

	private List<BehaviorStatusCodeResultModel> getBehaviorStatusCodeResultModels(
			String accessToken, String testPlanId)
			throws CustomGenericException, JAXBException {
		String caller = "getBehaviorStatusCodeResultModels";
		List<BehaviorStatusCodeResultModel> behaviorStatusCodeResultModels = new ArrayList<BehaviorStatusCodeResultModel>();
		List<Integer> scriptIdList = this.getTestPlanInfo()
				.getTestPlanInfo(accessToken, testPlanId, caller)
				.getScriptIds();
		if (scriptIdList == null)
			throw new CustomGenericException("5",
					"scripts in testPlan is empty", caller);
		for (Integer scriptId : scriptIdList) {
			ScriptBehaviorsBriefModel scriptBehaviorBriefModel = (ScriptBehaviorsBriefModel) getScriptBehaviorBriefModel(
					accessToken, testPlanId, new Integer(scriptId).toString())
					.getData();
			behaviorStatusCodeResultModels
					.addAll(this.getTestPlanService()
							.getBehaviorStatusCodeResultModels(
									scriptBehaviorBriefModel));

		}
		return behaviorStatusCodeResultModels;

	}
}
