package org.bench4q.web.api;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBException;

import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.models.ErrorResponseModel;
import org.bench4q.share.models.master.TestPlanDBModel;
import org.bench4q.share.models.master.TestPlanResponseModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.bench4q.web.model.BaseResponseModel;
import org.bench4q.web.model.TestPlanListModel;
import org.bench4q.web.service.CommunicateWithMaster;
import org.bench4q.web.service.TestPlanService;
import org.eclipse.jetty.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes({ "accessToken", "testPlanTaskList" })
public class TestPlanHistoryResult extends BaseControllerService {
	private String baseUrl = CommunicateWithMaster.getMasterAddress()+"testPlan";
	private TestPlanService testPlanService;
	private CommunicateWithMaster communicateWithMaster;
	private CommunicateWithMaster getCommunicateWithMaster() {
		return communicateWithMaster;
	}
@Autowired
	private void setCommunicateWithMaster(CommunicateWithMaster communicateWithMaster) {
		this.communicateWithMaster = communicateWithMaster;
	}

	public String getBaseUrl() {
		return baseUrl;
	}

	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}

	public TestPlanService getTestPlanService() {
		return testPlanService;
	}

	@Autowired
	public void setTestPlanService(TestPlanService testPlanService) {
		this.testPlanService = testPlanService;
	}

	@RequestMapping(value = "loadTestPlans", method = { RequestMethod.GET,
			RequestMethod.POST })
	@ResponseBody
	public BaseResponseModel
	 loadTestPlans(
			@ModelAttribute("accessToken") String accessToken)
			throws CustomGenericException {
		String caller = "TestPlanHistoryResult:loadTestPlans";
		String url = "testPlan"+ "/loadTestPlans";
		TestPlanResponseModel testPlanResponseModel = (TestPlanResponseModel) this
				.getTestPlanService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						TestPlanResponseModel.class, null, caller);
		if (testPlanResponseModel.isSuccess()) {
			return new BaseResponseModel(true,buildTestPlanListModel(testPlanResponseModel
					.getTestPlanDBModels()));
		} else {
			return new BaseResponseModel(false,testPlanResponseModel.getFailCause());
		}
	}

	private TestPlanListModel buildTestPlanListModel(List<TestPlanDBModel> list) {
		TestPlanListModel result = new TestPlanListModel();
		result.setList(list);
		return result;
	}

	@RequestMapping(value = "downloadReport", method = { RequestMethod.GET,
			RequestMethod.POST })
	public BaseResponseModel  downloadReport(HttpServletResponse response,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String testPlanId) throws CustomGenericException {
		String caller="TestPlanHistory:downloadReport";
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendGet(
					this.getBaseUrl() + "/getTestPlanReport",
					makeParamsMap("testPlanRunID", testPlanId),
					this.makeAccessTockenMap(accessToken));
			if (httpResponse == null ) {
				
				throw new CustomGenericException("1", "invalidate http response", caller);
			}
			if(httpResponse.getContent().isEmpty())
				throw new CustomGenericException("5", "report data empty","");
			/*if(httpResponse.getContent().isEmpty())
				throw new CustomGenericException(errCode, errMsg, source)*/
			if (httpResponse.getCode() == HttpStatus.BAD_REQUEST_400) {
				ErrorResponseModel errorResponseModel = (ErrorResponseModel) ObjectXmlExchange
						.fromXml(ErrorResponseModel.class,
								httpResponse.getContent());
				this.getCommunicateWithMaster().logException(errorResponseModel);
				throw new CustomGenericException("2", FAIL_WITH_EXCEPTION + "_"
						+ errorResponseModel.getMessage(), caller + "_"
						+ errorResponseModel.getResource());
			}
			byte[] pdfBuffer = httpResponse.getContent().getBytes();
			ServletOutputStream outputStream = response.getOutputStream();
			response.reset();
			response.setHeader("Content-disposition", "attachment; filename=\""
					+ testPlanId.toString() + ".pdf\"");
			response.addHeader("Content-Length", "" + pdfBuffer.length);
			response.setContentType("application/pdf");
			System.out.println("set response contentType:"
					+ response.getContentType());
			outputStream.write(pdfBuffer);
			outputStream.flush();
			outputStream.close();
			return new BaseResponseModel(true);
		} catch (IOException e) {
			throw new CustomGenericException("1", "exception:io exception", caller);
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			throw new CustomGenericException("1", "exception:io exception", caller);
		}
	}

	@RequestMapping(value = "deleteTest", method = { RequestMethod.GET,
			RequestMethod.POST })
	@ResponseBody
	public BaseResponseModel  deleteTest(HttpServletResponse response,
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam String testPlanId) throws CustomGenericException {
		String url = "testPlan" + "/removeTestPlanFromPool";
		String caller = "TestPlanHistoryController:deleteTest";
		Map<String, String> params = this.makeParamsMap("testPlanId",
				testPlanId);
		TestPlanResponseModel testPlanResponseModel = (TestPlanResponseModel) this
				.getTestPlanService()
				.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						TestPlanResponseModel.class, params, caller);
		
		if (testPlanResponseModel.isSuccess()) {

			return new BaseResponseModel(true);
		} else {
			
			return new BaseResponseModel(false,testPlanResponseModel.getFailCause());
		}

	}
}
