package org.bench4q.web.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.bench4q.share.communication.HttpRequester;
import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.models.ErrorResponseModel;
import org.bench4q.web.api.BaseControllerService;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.eclipse.jetty.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class CommunicateWithMaster {
	private Logger logger = Logger.getLogger(CommunicateWithMaster.class);
	private HttpRequester httpRequester;
	private final String HTTPRSEPONSE_INVALIDATE = "network error!";
	public final String FAIL_WITH_EXCEPTION = "fail with exception from master";
	private final String AUTH_HEADER_PROPERTY = "Authorization";
	private final String ACCES_TOCKEN_STARTER = "Bearer ";
	private final String IOEXCEPTION = "io exception: get httpresponse from master";
	private final String JAXBEXCEPTION = "JAXBException exception :get httpresponse from master";
	private final String BADHTTPRESPONSE = "bad response";
	private static final Properties prop = new Properties();
	private static final String CONFIG_FILE_NAME = "ipConfig.properties";
	private static String masterAddress = "";

	private HttpRequester getHttpRequester() {
		return httpRequester;
	}

	@Autowired
	private void setHttpRequester(HttpRequester httpRequester) {
		this.httpRequester = httpRequester;
	}

	public static String getMasterAddress() {
		return masterAddress;
	}

	private String addMaseterAddressToUrl(String url) {
		return masterAddress + url;
	}

	static {
		String masterIpFilePath;
		try {
			masterIpFilePath = BaseControllerService.class.getClassLoader()
					.getResource("").toURI().getPath()
					+ CONFIG_FILE_NAME;
			FileInputStream inputStream = new FileInputStream(new File(
					masterIpFilePath));
			prop.load(inputStream);
			inputStream.close();
			masterAddress = prop.getProperty("masterAddress");
		} catch (Exception e1) {
			masterAddress = "133.133.12.1:7979";
		}

	}

	public Object getRespnseObjectByPostXml(String accessToken, String url,
			Class<?> objectClass, String content, String caller)
			throws CustomGenericException {
		try {
			url = this.addMaseterAddressToUrl(url);
			HttpResponse httpResponse = this.getHttpRequester().sendPostXml(
					url, content, this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				logger.info(HTTPRSEPONSE_INVALIDATE);
				throw new CustomGenericException("1", HTTPRSEPONSE_INVALIDATE,
						caller);
			}
			if (httpResponse.getCode() == HttpStatus.BAD_REQUEST_400) {
				ErrorResponseModel errorResponseModel = (ErrorResponseModel) ObjectXmlExchange
						.fromXml(ErrorResponseModel.class,
								httpResponse.getContent());
				logException(errorResponseModel);
				throw new CustomGenericException("2", FAIL_WITH_EXCEPTION + "_"
						+ errorResponseModel.getMessage(), caller + "_"
						+ errorResponseModel.getResource());
			}
			if (httpResponse.getCode() != 200) {
				logger.info(caller + " get httpResponse code is not 200:");
				logger.info(httpResponse.getContent());
				throw new CustomGenericException("1", BADHTTPRESPONSE, caller);
			}
			Object responseObject = ObjectXmlExchange.fromXml(objectClass,
					httpResponse.getContent());
			return responseObject;
		} catch (IOException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.IOEXCEPTION, caller);
		} catch (JAXBException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.JAXBEXCEPTION,
					"get networks_" + caller);
		}
	}

	public Object getResponseModel(String accessToken, String url,
			Class<?> objectClass, Map<String, String> params, String caller)
			throws CustomGenericException {
		try {
			url = this.addMaseterAddressToUrl(url);
			HttpResponse httpResponse = this.getHttpRequester().sendGet(url,
					params, this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				logger.info(HTTPRSEPONSE_INVALIDATE);
				throw new CustomGenericException("1", HTTPRSEPONSE_INVALIDATE,
						caller);
			}
			if (httpResponse.getCode() == HttpStatus.BAD_REQUEST_400) {
				ErrorResponseModel errorResponseModel = (ErrorResponseModel) ObjectXmlExchange
						.fromXml(ErrorResponseModel.class,
								httpResponse.getContent());
				logException(errorResponseModel);
				throw new CustomGenericException("2", FAIL_WITH_EXCEPTION + "_"
						+ errorResponseModel.getMessage(), caller + "_"
						+ errorResponseModel.getResource());
			}
			if (httpResponse.getCode() != 200) {
				logger.info(caller + " get httpResponse code is not 200:");
				logger.info(httpResponse.getContent());
				throw new CustomGenericException("1", BADHTTPRESPONSE, caller);
			}
			Object responseObject = ObjectXmlExchange.fromXml(objectClass,
					httpResponse.getContent());
			return responseObject;
		} catch (IOException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.IOEXCEPTION, caller);
		} catch (JAXBException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.JAXBEXCEPTION,
					"get networks_" + caller);
		}
	}

	public Object getResponseModelByPut(String accessToken, String url,
			String content, Class<?> objectClass, String caller)
			throws CustomGenericException {
		try {
			url = this.addMaseterAddressToUrl(url);
			HttpResponse httpResponse = this.getHttpRequester().sendPutXml(url,
					content, this.makeAccessTockenMap(accessToken));
			if (HttpRequester.isInvalidResponse(httpResponse)) {

				throw new CustomGenericException("1", HTTPRSEPONSE_INVALIDATE,
						caller);
			}
			System.out.println(httpResponse.getContent());
			if (httpResponse.getCode() == HttpStatus.BAD_REQUEST_400) {
				ErrorResponseModel errorResponseModel = (ErrorResponseModel) ObjectXmlExchange
						.fromXml(ErrorResponseModel.class,
								httpResponse.getContent());
				logException(errorResponseModel);
				throw new CustomGenericException("2", FAIL_WITH_EXCEPTION + "_"
						+ errorResponseModel.getMessage(), caller + "_"
						+ errorResponseModel.getResource());
			}
			if (httpResponse.getCode() != 200) {
				logger.info(caller + " get httpResponse code is not 200:");
				logger.info(httpResponse.getContent());
				throw new CustomGenericException("1", BADHTTPRESPONSE, caller);
			}
			Object responseObject = ObjectXmlExchange.fromXml(objectClass,
					httpResponse.getContent());

			return responseObject;
		} catch (IOException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.IOEXCEPTION, caller);
		} catch (JAXBException e) {
			logger.error(e, e.fillInStackTrace());
			throw new CustomGenericException("1", this.JAXBEXCEPTION,
					"get networks_" + caller);
		}

	}

	private Map<String, String> makeAccessTockenMap(String accessToken) {
		Map<String, String> properties = new HashMap<String, String>();
		properties
				.put(AUTH_HEADER_PROPERTY, ACCES_TOCKEN_STARTER + accessToken);
		return properties;
	}

	public void logException(ErrorResponseModel errorResponseModel) {
		logger.info(FAIL_WITH_EXCEPTION);
		logger.info("Exception code:" + errorResponseModel.getCode());
		logger.info("Exception message:" + errorResponseModel.getMessage());
		logger.info("Exception resource:" + errorResponseModel.getResource());
	}

}
