package org.bench4q.web.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.bench4q.share.models.master.MonitorMemoryResponseModel;
import org.bench4q.share.models.master.MonitorNetworkReponseModel;
import org.bench4q.share.models.master.MonitorProcessorResponseModel;
import org.bench4q.share.models.monitor.ProcessorModel;
import org.bench4q.share.models.monitor.ProcessorModelChild;
import org.bench4q.web.exception.CustomGenericException;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestParam;

@Component
public class MonitorService extends BaseService {
	private String baseUrl = "monitorController";
	private Logger logger = Logger.getLogger(MonitorService.class);
	public String getBaseUrl() {
		return baseUrl;
	}

	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}

	public List<String> getCpuList(
			@ModelAttribute("accessToken") String accessToken,
			@RequestParam("testPlanId") String testPlanId,
			@RequestParam String ip, String port) throws CustomGenericException {
		String scource = "MonitorService:getCpuList";
		try {
			MonitorProcessorResponseModel monitorProcessorResponseModel = getCpus(
					accessToken, testPlanId, ip, port, "0");
			ProcessorModel processorModel = monitorProcessorResponseModel
					.getProcessorModels().get(0);
			List<String> cpuList = new ArrayList<String>();
			for (ProcessorModelChild processorModelChild : processorModel
					.getProcessorModelList()) {
				cpuList.add(processorModelChild.getInstance());
			}
			return cpuList;
		} catch (NullPointerException e) {
			logger.info(e.getStackTrace());
			throw new CustomGenericException("1", "error cpu list info"
					+ e.getCause(), scource);
		}
	}

	public MonitorMemoryResponseModel getMemoryResponseModel(
			@ModelAttribute("accessToken") String accessToken,
			String testPlanId, String ip, String port, String startTime)
			throws CustomGenericException {
		String url = this.getBaseUrl() + "/memorySUTInfo";
		String caller = "MonitorService:getMemoryResponse";
		Map<String, String> params = makeParamsMap(ip, port, testPlanId,
				startTime);

		return (MonitorMemoryResponseModel)this.getCommunicateWithMaster() 
				.getResponseModel(accessToken, url,
						MonitorMemoryResponseModel.class, params, caller);
	}

	public MonitorProcessorResponseModel getCpus(
			@ModelAttribute("accessToken") String accessToken,
			String testPlanId, String ip, String port, String startTime)
			throws CustomGenericException {
		Map<String, String> params = makeParamsMap(ip, port, testPlanId,
				startTime);
		String url = this.getBaseUrl() + "/processorSUTInfo";
		String caller = "MonitorService:getCpus";

		return (MonitorProcessorResponseModel) this.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						MonitorProcessorResponseModel.class, params, caller);

	}

//	public List<String> getNetworkList(String accessToken, String testplanId,
//			String ip, String port) throws CustomGenericException {
//		String scource = "MonitorService:getNetworkList";
//		try {
//			MonitorNetworkReponseModel monitorNetworkReponseModel = getNetworks(
//					accessToken, testplanId, ip, port, "0");
//			NetworkInterfaceModel networkInterfaceModel = monitorNetworkReponseModel
//					.getModels().get(0);
//			List<String> networkList = new ArrayList<String>();
//			for (NetworkInterfaceModelChild networkInterfaceModelChild : networkInterfaceModel
//					.getNetworkList()) {
//				networkList.add(networkInterfaceModelChild.getInstance());
//			}
//			return networkList;
//		} catch (NullPointerException e) {
//			logger.info(e.getStackTrace());
//			throw new CustomGenericException("1", "error network list info"
//					+ e.getCause(), scource);
//		}
//	}

	public MonitorNetworkReponseModel getNetworks(String accessToken,
			String testplanId, String ip, String port, String startTime)
			throws CustomGenericException {

		Map<String, String> params = makeParamsMap(ip, port, testplanId,
				startTime);
		String url = this.getBaseUrl() + "/networkInfo";
		String caller = "MonitorService:getNetworks";

		return (MonitorNetworkReponseModel) this.getCommunicateWithMaster()
				.getResponseModel(accessToken, url,
						MonitorNetworkReponseModel.class, params, caller);

	}

	private Map<String, String> makeParamsMap(String ip, String port,
			String testPlanId, String startTime) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("hostName", ip);
		params.put("port", port);
		params.put("testPlanRunId", testPlanId);
		params.put("duationBegin", startTime);
		return params;
	}

}
