package org.bench4q.web.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;

import org.bench4q.share.models.agent.ParameterModel;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.agent.scriptrecord.BatchModel;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.PageModel;
import org.bench4q.share.models.master.OperateScriptServerResponseModel;
import org.bench4q.share.models.master.ScriptModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.extractObjectFromXml.ObjectXmlExchange;
import org.springframework.stereotype.Component;

@Component
public class ScriptService extends BaseService {
	private String baseUrl = "RecordScript";

	public String getBaseUrl() {
		return baseUrl;
	}

	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}

	public ScriptModel queryScript(int id, String accessToken)
			throws CustomGenericException {
		// check script id and return throw exception
		String caller = "ScriptService:queryScript";

		OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
				.getCommunicateWithMaster().getResponseModel(accessToken,
						this.getBaseUrl() + "/queryScriptById",
						OperateScriptServerResponseModel.class,
						makeParamsMap("scriptId", String.valueOf(id)), caller);
		if (operateScriptServerResponseModel.isSuccess())
			return operateScriptServerResponseModel.getScriptModels().get(0);
		else
			throw new CustomGenericException("5", "query script failed", caller
					+ ":" + id);
	}

	public List<ScriptModel> queryScripts(List<Integer> scriptIds,
			String accessToken) throws CustomGenericException {
		String caller = "ScriptService:queryScript";
		List<ScriptModel> scriptModels = new ArrayList<ScriptModel>();
		for (Integer scriptId : scriptIds) {
			OperateScriptServerResponseModel operateScriptServerResponseModel = (OperateScriptServerResponseModel) this
					.getCommunicateWithMaster()
					.getResponseModel(
							accessToken,
							this.getBaseUrl() + "/queryScriptById",
							OperateScriptServerResponseModel.class,
							makeParamsMap("scriptId", String.valueOf(scriptId)),
							caller);
			if (operateScriptServerResponseModel.isSuccess())
				scriptModels.add(operateScriptServerResponseModel
						.getScriptModels().get(0));
			else
				throw new CustomGenericException("5", "query script failed",
						caller + ":" + scriptId);
		}
		return scriptModels;
	}

	public Map<String, Integer> getScriptNameIdMap(
			List<ScriptModel> scriptModels) throws CustomGenericException {
		Map<String, Integer> map = new HashMap<String, Integer>();
		for (ScriptModel scriptModel : scriptModels) {
			map.put(scriptModel.getName(), scriptModel.getId());
		}
		return map;
	}

	public Map<Integer, String> getScriptIdNameMap(
			List<ScriptModel> scriptModels) throws CustomGenericException {
		Map<Integer, String> map = new HashMap<Integer, String>();
		for (ScriptModel scriptModel : scriptModels) {
			map.put(scriptModel.getId(), scriptModel.getName());
		}
		return map;
	}

	public Map<String, Integer> getPageNameAndIdMap(
			List<ScriptModel> scriptModels) throws JAXBException {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (ScriptModel scriptModel : scriptModels) {
				Map<String, Integer> pageMap = getPageNameAndIdMap((RunScenarioModel)
						ObjectXmlExchange.fromXml(RunScenarioModel.class,
						scriptModel.getScriptContent()));
				if (pageMap != null)
					map.putAll(pageMap);
			}
			return map;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private Map<String, Integer> getPageNameAndIdMap(
			RunScenarioModel runScenarioModel) {

		List<PageModel> pageModels = runScenarioModel.getPages();
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (int i = 0; i < pageModels.size(); i++) {
				map.put("page_" + i, i);
			}
			return map;
		} catch (NullPointerException e) {
			return null;
		}

	}

	public Map<String, Integer> getBehaviorNameAndIdMap(
			List<ScriptModel> scriptModels) throws JAXBException {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (ScriptModel scriptModel: scriptModels) {
				Map<String, Integer> behaviorMap = getBehaviorNameAndIdMap((RunScenarioModel)
						ObjectXmlExchange.fromXml(RunScenarioModel.class,
						scriptModel.getScriptContent()));
				if (behaviorMap != null)
					map.putAll(behaviorMap);
			}
			return map;
		} catch (NullPointerException e) {
			return null;
		}
	}

	public List<String> getScriptPages(RunScenarioModel runScenarioModel) {
		try {
			List<String> scriptPages = new ArrayList<String>();
			for (int i = 1; i < scriptPages.size(); i++) {
				scriptPages.add("page_" + i);
			}
			return scriptPages;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	public List<String> getScriptBehaviorUrl(RunScenarioModel runScenarioModel) {

		List<BehaviorModel> behaviorModelsWithUrl = getBehaviorModelsHasUrl(runScenarioModel);
		if (behaviorModelsWithUrl == null)
			return null;
		List<String> urList = new ArrayList<String>();
		for (BehaviorModel behaviorModelWithUrl : behaviorModelsWithUrl) {
			String url = generateUrl(behaviorModelWithUrl.getParameters());
			urList.add(url);
		}
		return urList;
	}

	public Map<String, Integer> getBehaviorNameAndIdMap(
			RunScenarioModel runScenarioModel) {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			List<BehaviorModel> behaviorModels = this
					.getBehaviorModelsHasUrl(runScenarioModel);
			if (behaviorModels == null)
				return null;
			for (BehaviorModel behaviorModel : behaviorModels) {
				String url = this.generateUrl(behaviorModel.getParameters());
				if (url != null)
					map.put(url, behaviorModel.getId());
			}
			return map;

		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private String generateUrl(List<ParameterModel> parameterModels) {
		String url = new String("");
		int i = 0;
		for (ParameterModel parameterModel : parameterModels) {
			if (i == 1 && parameterModel.getValue() != null)
				url += "?" + parameterModel.getValue();
			if (i > 1 && parameterModel.getValue() != null)
				url += "&" + parameterModel.getValue();
			i++;
		}
		if (url.length() <= 0)
			return null;
		return url;
	}

	private List<BehaviorModel> getBehaviorModelsHasUrl(
			RunScenarioModel runScenarioModel) {
		try {

			List<BehaviorModel> behaviorModels = getBehaviorModels(runScenarioModel);
			if (behaviorModels == null)
				return null;
			List<BehaviorModel> behaviorModelsWithUrl = new ArrayList<BehaviorModel>();
			for (BehaviorModel behaviorModel : behaviorModels) {
				List<ParameterModel> parameterModels = new ArrayList<ParameterModel>();
				parameterModels.addAll(behaviorModel.getParameters());
				for (ParameterModel parameterModel : parameterModels) {
					if (parameterModel.getKey().equals("url"))
						behaviorModelsWithUrl.add(behaviorModel);
					break;
				}
			}

			return behaviorModelsWithUrl;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private List<BehaviorModel> getBehaviorModels(
			RunScenarioModel runScenarioModel) {
		try {
			List<BatchModel> batchModels = new ArrayList<BatchModel>();
			for (PageModel pageModel : runScenarioModel.getPages())
				batchModels.addAll(pageModel.getBatches());
			List<BehaviorModel> behaviorModels = new ArrayList<BehaviorModel>();
			for (BatchModel batchModel : batchModels)
				behaviorModels.addAll(batchModel.getBehaviors());
			return behaviorModels;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

}
