package org.bench4q.web.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.bench4q.share.models.agent.BehaviorBriefModel;
import org.bench4q.share.models.agent.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.master.TestPlanScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.model.ResultInfoModel;
import org.bench4q.web.model.ResultModel;
import org.bench4q.web.model.TestPlanRequestModel;
import org.bench4q.web.model.WebScriptModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.SessionAttributes;

@Component
@SessionAttributes({ "accessToken", "testPlanTaskList" })
public class TestPlanService extends BaseService {
	private ScriptService scriptService;

	public ScriptService getScriptService() {
		return scriptService;
	}

	@Autowired
	public void setScriptService(ScriptService scriptService) {
		this.scriptService = scriptService;
	}

	public TestPlanService() {
		// TODO Auto-generated constructor stub
	}

	public ResultInfoModel getScriptBriefStatusResultInfo() {
		String result = "briefStatus";

		String[] fields = new String[] { "averageResponseTime",
				"maxResponseTime", "minResponseTime", "averageElapsedTime",
				"successThroughputThisTime", "failThroughputThisTime",
				"failRateThisTime", "totalFailCountFromBegin",
				"responseTimeDeviationThisTime" };
		return getResultInfoModel(result, fields);
	}

	public ResultInfoModel getScriptBehaviorBriefResultInfo(
			RunScenarioModel runScenarioModel) {
		String behaviroResult = "behaviors";

		ResultInfoModel behaviorResultInfoModel = new ResultInfoModel();
		behaviorResultInfoModel
				.setChildResults(new ArrayList<ResultInfoModel>());
		behaviorResultInfoModel.setResult(behaviroResult);

		String[] fields = new String[] { "minResponseTime", "maxResponseTime",
				"totalResponseTimeThisTime" };
		for (String url : this.getScriptService().getScriptBehaviorUrl(
				runScenarioModel)) {
			ResultInfoModel urlResultInfoModel =

			getResultInfoModel(url, fields);
			behaviorResultInfoModel.getChildResults().add(urlResultInfoModel);
		}

		return behaviorResultInfoModel;
	}

	public ResultInfoModel getScriptPageBriefResultInfo(
			RunScenarioModel runScenarioModel) {

		ResultInfoModel pageInfoModel = new ResultInfoModel();
		pageInfoModel.setChildResults(new ArrayList<ResultInfoModel>());
		pageInfoModel.setResult("Pages");
		String[] fields = new String[] { "averageResponseTimeThisTime",
				"latestResponseTime", "throughputThisTime" };
		for (String page : this.getScriptService().getScriptPages(
				runScenarioModel)) {

			pageInfoModel.getChildResults().add(
					getResultInfoModel(page, fields));
		}
		return pageInfoModel;
	}

	public ResultInfoModel getResultInfoModel(String result, List<String> fields) {
		ResultInfoModel resultInfoModel = new ResultInfoModel();
		resultInfoModel.setResult(result);
		if (fields.size() <= 0)
			return resultInfoModel;

		resultInfoModel.setChildResults(new ArrayList<ResultInfoModel>());
		for (String field : fields) {
			ResultInfoModel scriptResultInfoModelChild = new ResultInfoModel();

			scriptResultInfoModelChild.setResult(field);
			resultInfoModel.getChildResults().add(scriptResultInfoModelChild);
		}
		return resultInfoModel;
	}

	public List<ResultModel> extractResultModelList(
			TestPlanScriptBriefResultModel testPlanScriptBriefResultModel,
			String fieldName) throws CustomGenericException,
			NoSuchFieldException {
		if (testPlanScriptBriefResultModel.getScriptBriefResultModels() == null)
			throw new CustomGenericException(
					"5",
					"testPlanScriptBriefResultModel:scriptBriefResultModels  empty  data",
					"");
		List<ResultModel> resultModels = new ArrayList<ResultModel>();
		for (ScriptBriefResultModel scriptBriefResultModel : testPlanScriptBriefResultModel
				.getScriptBriefResultModels()) {

			List<String> fieldNames = new ArrayList<String>();
			fieldNames.add(fieldName);
			ResultModel resultModel = ChangeToResultModel.getResultModel(
					scriptBriefResultModel, fieldNames);
			resultModel.setFinished(scriptBriefResultModel.isFinished());
			resultModel.setTime(scriptBriefResultModel.getSamplingTime());
			resultModels.add(resultModel);
		}
		return resultModels;

	}

	public boolean isValidateTestRunModel(
			TestPlanRequestModel testPlanRequestModel) {

		if (testPlanRequestModel.getScriptList() == null
				|| testPlanRequestModel.getScriptList().size() == 0)
			return true;
		if (testPlanRequestModel.getTestPlanName() == null
				|| testPlanRequestModel.getTestPlanName().equals(""))
			testPlanRequestModel.setTestPlanName("default");
		for (WebScriptModel scriptModel : testPlanRequestModel.getScriptList()) {
			if (isValidateScriptModel(scriptModel))
				return true;
		}

		return false;
	}

	private boolean isValidateScriptModel(WebScriptModel scriptModel) {
		if (scriptModel.getId() == 0)
			return true;
		if (scriptModel.getLoad() == 0)
			return true;
		if (scriptModel.getExecuteRange() == 0)
			return true;
		return false;
	}

	public Map<String, Double> getUrlContentTypePercent(
			Map<String, Long> urlContentTypeCountMap, long totalCount) {
		if (urlContentTypeCountMap == null)
			return null;

		if (totalCount == 0)
			return null;
		Map<String, Double> urlContentTypePercentMap = new HashMap<String, Double>();
		Set<String> keys = urlContentTypeCountMap.keySet();
		Iterator<String> iterator = keys.iterator();
		while (iterator.hasNext()) {
			String key = iterator.next();
			double percent = (double) urlContentTypeCountMap.get(key)
					/ (double) totalCount * 100;
			urlContentTypePercentMap.put(key, new Double(percent));
		}
		return urlContentTypePercentMap;

	}

	public long getTotalCount(Map<String, Long> urlContentTypeCountMap) {

		Set<String> keys = urlContentTypeCountMap.keySet();
		Iterator<String> iterator = keys.iterator();
		long totalCount = 0;
		while (iterator.hasNext()) {
			totalCount += urlContentTypeCountMap.get(iterator.next());
		}
		return totalCount;
	}

	public List<BehaviorStatusCodeResultModel> getBehaviorStatusCodeResultModels(
			ScriptBehaviorsBriefModel scriptBehaviorsBriefModel) {
		List<BehaviorStatusCodeResultModel> behaviorStatusCodeResultModels = new ArrayList<BehaviorStatusCodeResultModel>();
		for (BehaviorBriefModel behaviorBriefModel : scriptBehaviorsBriefModel
				.getTestBehaviorsBriefModel().getBehaviorBriefModels()) {

			if (behaviorBriefModel.getDetailStatusCodeResultModels() != null)
				behaviorStatusCodeResultModels.addAll(behaviorBriefModel
						.getDetailStatusCodeResultModels());
		}
		return behaviorStatusCodeResultModels;

	}

}
